@@ -1355,10 +1355,11 @@ ObjRecord::~ObjRecord()
 
 void ObjRecord::dump(std::ostream& out) const
 {
-    Q_ASSERT(m_object);
     out << "Obj" << std::endl;
-    out << "  id: " << m_object->id() << std::endl;
-    out << "  type: " << m_object->type() << std::endl;
+    if (m_object) {
+        out << "  id: " << m_object->id() << std::endl;
+        out << "  type: " << m_object->type() << std::endl;
+    }
 }
 
 void ObjRecord::setData(unsigned size, const unsigned char* data, const unsigned* /* continuePositions */)
@@ -3126,6 +3127,7 @@ bool ExcelReader::load(Workbook* workbook, const char* filename)
 
 #ifdef SWINDER_XLS2RAW
             std::cout << std::setfill('0') << std::setw(8) << std::dec << record->position() << " ";
+            if (!record->isValid()) std::cout << "Invalid ";
             std::cout << "Record 0x";
             std::cout << std::setfill('0') << std::setw(4) << std::hex << record->rtti();
             std::cout << " (";
@@ -3135,12 +3137,14 @@ bool ExcelReader::load(Workbook* workbook, const char* filename)
             std::cout << std::endl;
 #endif
 
-            if (record->rtti() == BOFRecord::id)
-                handleRecord(record);
-            if (!d->handlerStack.empty() && d->handlerStack.back())
-                d->handlerStack.back()->handleRecord(record);
-            if (record->rtti() == EOFRecord::id)
-                handleRecord(record);
+            if (record->isValid()) {
+                if (record->rtti() == BOFRecord::id)
+                    handleRecord(record);
+                if (!d->handlerStack.empty() && d->handlerStack.back())
+                    d->handlerStack.back()->handleRecord(record);
+                if (record->rtti() == EOFRecord::id)
+                    handleRecord(record);
+            }
 
             delete record;
         }
