@@ -379,40 +379,6 @@ private:
     Private *d;
 };
 
-class FilepassRecord : public Record
-{
-public:
-
-    static const unsigned int id;
-
-    unsigned int rtti() const {
-        return this->id;
-    }
-
-    /**
-     * Creates a new FILEPASS record.
-     */
-    FilepassRecord(Workbook *book);
-
-    /**
-     * Destroy the record.
-     */
-    virtual ~FilepassRecord();
-
-    virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
-
-    virtual const char* name() const {
-        return "FILEPASS";
-    }
-
-    virtual void dump(std::ostream& out) const;
-
-private:
-    // no copy or assign
-    FilepassRecord(const FilepassRecord&);
-    FilepassRecord& operator=(const FilepassRecord&);
-};
-
 /**
   \brief Formula.
 
@@ -620,6 +586,10 @@ public:
 
     void setDefinedName(const UString& name);
 
+    unsigned sheetIndex() const;
+
+    bool isBuiltin() const;
+
     virtual void setData(unsigned size, const unsigned char* data, const unsigned int* continuePositions);
 
     virtual const char* name() const {
@@ -1467,6 +1437,45 @@ private:
     Private* d;
 };
 
+class BkHimRecord : public Record
+{
+public:
+    static const unsigned id;
+
+    virtual unsigned rtti() const { return this->id; }
+
+    BkHimRecord(Workbook *book);
+    virtual ~BkHimRecord();
+
+    BkHimRecord( const BkHimRecord& record );
+    BkHimRecord& operator=( const BkHimRecord& record );
+
+    enum Format {
+        WindowsBitMap = 0x0009,
+        NativeFormat = 0x000e
+    };
+
+    static UString formatToString(Format format);
+
+    Format format() const;
+    void setFormat( Format format );
+
+    UString imagePath() const;
+    void setImagePath( UString imagePath );
+
+    unsigned imageSize() const;
+    void setImageSize( unsigned imageSize );
+
+    virtual void setData( unsigned size, const unsigned char* data, const unsigned* continuePositions );
+
+    virtual const char* name() const { return "BkHim"; }
+
+    virtual void dump( std::ostream& out ) const;
+
+private:
+    class Private;
+    Private * const d;
+};
 
 } // namespace Swinder
 
