@@ -34,144 +34,6 @@
 namespace Swinder
 {
 
-class HLinkRecord : public Record
-{
-public:
-    unsigned long m_firstRow;
-    unsigned long m_firstColumn;
-    unsigned long m_lastRow;
-    unsigned long m_lastColumn;
-    UString m_displayName;
-    UString m_targetFrameName;
-    UString m_location;
-
-    static const unsigned id;
-    virtual unsigned rtti() const {
-        return this->id;
-    }
-    virtual const char* name() const {
-        return "HLink";
-    }
-    virtual void dump(std::ostream&) const {}
-    static Record *createRecord(Workbook *book) {
-        return new HLinkRecord(book);
-    }
-    HLinkRecord(Workbook *book) : Record(book), m_firstRow(0), m_firstColumn(0), m_lastRow(0), m_lastColumn(0) {}
-    virtual ~HLinkRecord() {}
-    virtual void setData(unsigned size, const unsigned char* data, const unsigned* /* continuePositions */) {
-        if (size < 8) {
-            setIsValid(false);
-            return;
-        }
-
-        m_firstRow = readU16(data);
-        m_lastRow = readU16(data + 2);
-        m_firstColumn = readU16(data + 4);
-        m_lastColumn = readU16(data + 6);
-
-        const unsigned char* startHyperlinkObject = data + 16 /* skip CLDID */ + 8;
-        const unsigned long streamVersion = readU32(startHyperlinkObject);
-        if (streamVersion != 2) {
-            std::cerr << "Invalid stream version " << streamVersion << " in HLinkRecord" << std::endl;
-            setIsValid(false);
-            return;
-        }
-
-        // 10 bits options + 22 bits reserved
-        const unsigned long opts = readU32(startHyperlinkObject + 4);
-        const bool hlstmfHasMonikor = opts & 0x01;
-        //const bool hlstmfIsAbsolute = opts & 0x02;
-        //const bool hlstmfISiteGaveDisplayName = opts & 0x04;
-        bool hlstmfHasLocationStr = opts & 0x08;
-        const bool hlstmfHasDisplayName = opts & 0x10;
-        //const bool hlstmfHasGUID = opts & 0x20;
-        //const bool hlstmfHasCreationTime = opts & 0x60;
-        const bool hlstmfHasFrameName = opts & 0xC0;
-        const bool hlstmfMonikerSavedAsStr = opts & 0x180;
-        //const bool hlstmfAbsFromGetdataRel = opts & 0x300;
-        //Q_ASSERT( !hlstmfMonikerSavedAsStr || hlstmfHasMonikor );
-
-        startHyperlinkObject += 8;
-
-        unsigned long length = 0;
-        unsigned sizeReaded = 0;
-
-        if (hlstmfHasDisplayName) {
-            length = readU32(startHyperlinkObject);
-            m_displayName = readUnicodeChars(startHyperlinkObject + 4, length, -1, 0, &sizeReaded);
-            //printf("HLinkRecord: displayName=%s\n", m_displayName.ascii());
-            startHyperlinkObject += 4 + sizeReaded;
-        }
-
-        if (hlstmfHasFrameName) {
-            length = readU32(startHyperlinkObject);
-            m_targetFrameName = readUnicodeChars(startHyperlinkObject + 4, length, -1, 0, &sizeReaded);
-            //printf("HLinkRecord: targetFrameName=%s\n", m_targetFrameName.ascii());
-            startHyperlinkObject += 4 + sizeReaded;
-        }
-
-        if (hlstmfHasMonikor) {
-            if (hlstmfMonikerSavedAsStr) {  // moniker
-                //TODO: seems following code leads to a crash on readUnicodeChars...
-                //length = readU32(startHyperlinkObject);
-                //UString moniker = readUnicodeChars(startHyperlinkObject + 4, length, -1, 0, &sizeReaded);
-                //startHyperlinkObject += 4 + sizeReaded;
-                std::cerr << "HLinkRecord: Unhandled hlstmfMonikerSavedAsStr moniker" << std::endl;
-                setIsValid(false);
-                return;
-            } else { // oleMoniker
-                const unsigned long clsid = readU32(startHyperlinkObject);
-                startHyperlinkObject += 16; // the clsid is actually 16 byte long but we only need the first 4 to differ
-                switch (clsid) {
-                case 0x79EAC9E0: { // URLMoniker
-                    length = readU32(startHyperlinkObject);
-                    m_location = readTerminatedUnicodeChars(startHyperlinkObject + 4, &sizeReaded);
-                    startHyperlinkObject += length + 4;
-                }
-                break;
-                case 0x00000303: { // FileMoniker
-                    std::cout << "TODO: HLinkRecord FileMoniker" << std::endl;
-                    return; // abort
-                }
-                break;
-                case 0x00000309: { // CompositeMoniker
-                    std::cout << "TODO: HLinkRecord CompositeMoniker" << std::endl;
-                    return; // abort
-                }
-                break;
-                case 0x00000305: { // AntiMoniker
-                    std::cout << "TODO: HLinkRecord AntiMoniker" << std::endl;
-                    return; // abort
-                }
-                break;
-                case 0x00000304: { // ItemMoniker
-                    std::cout << "TODO: HLinkRecord ItemMoniker" << std::endl;
-                    return; // abort
-                }
-                break;
-                }
-            }
-        }
-
-        if (hlstmfHasLocationStr) {
-            length = readU32(startHyperlinkObject);
-            startHyperlinkObject += 4;
-            if(startHyperlinkObject+length > data+size) {
-                std::cerr << "HLinkRecord: expected location but got invalid size=" << length << std::endl;
-                setIsValid(false);
-                return;
-            }
-            m_location = readUnicodeChars(startHyperlinkObject, length, -1, 0, &sizeReaded);
-            std::cout << "HLinkRecord: m_displayName=" << m_displayName << " m_targetFrameName=" << m_targetFrameName << " location=" << m_location.ascii() << std::endl;
-            startHyperlinkObject += sizeReaded;
-        }
-
-        // ignore (16 bytes) guid and fileTime (8 bytes)
-    }
-};
-
-const unsigned HLinkRecord::id = 0x01B8;
-
 class WorksheetSubStreamHandler::Private
 {
 public:
@@ -217,8 +79,6 @@ WorksheetSubStreamHandler::WorksheetSubStreamHandler(Sheet* sheet, const Globals
     d->formulaStringCell = 0;
     d->noteCount = 0;
     d->lastDrawingObject = 0;
-
-    RecordRegistry::registerRecordClass(HLinkRecord::id, HLinkRecord::createRecord);
 }
 
 WorksheetSubStreamHandler::~WorksheetSubStreamHandler()
@@ -328,7 +188,7 @@ void WorksheetSubStreamHandler::handleRecord(Record* record)
     else if (type == TopMarginRecord::id)
         handleTopMargin(static_cast<TopMarginRecord*>(record));
     else if (type == HLinkRecord::id)
-        handleLink(static_cast<HLinkRecord*>(record));
+        handleHLink(static_cast<HLinkRecord*>(record));
     else if (type == NoteRecord::id)
         handleNote(static_cast<NoteRecord*>(record));
     else if (type == ObjRecord::id)
@@ -446,6 +306,8 @@ void WorksheetSubStreamHandler::handleColInfo(ColInfoRecord* record)
             column->setWidth( Column::columnUnitsToPts((double)width) );
             column->setFormat(d->globals->convertedFormat(xfIndex));
             column->setVisible(!hidden);
+            column->setOutlineLevel(record->outlineLevel());
+            column->setCollapsed(record->isCollapsed());
         }
     }
 }
@@ -797,6 +659,8 @@ void WorksheetSubStreamHandler::handleRow(RowRecord* record)
         row->setHeight(height / 20.0);
         row->setFormat(d->globals->convertedFormat(xfIndex));
         row->setVisible(!hidden);
+        row->setOutlineLevel(record->outlineLevel());
+        row->setCollapsed(record->isCollapsed());
     }
 }
 
@@ -853,15 +717,16 @@ void WorksheetSubStreamHandler::handleTopMargin(TopMarginRecord* record)
     d->sheet->setTopMargin(margin);
 }
 
-void WorksheetSubStreamHandler::handleLink(HLinkRecord* record)
+void WorksheetSubStreamHandler::handleHLink(HLinkRecord* record)
 {
     if (!record) return;
     if (!d->sheet) return;
 
     //FIXME we ignore the m_lastRow and m_lastColumn values, does ODF have something similar?
-    Cell *cell = d->sheet->cell(record->m_firstColumn, record->m_firstRow);
+    Cell *cell = d->sheet->cell(record->firstColumn(), record->firstRow());
     if (cell) {
-        cell->setHyperlink(record->m_displayName, record->m_location, record->m_targetFrameName);
+        UString url = record->urlMonikerUrl() + UString('#') + record->location();
+        cell->setHyperlink(record->displayName(), url, record->frameName());
     }
 }
 
