@@ -53,8 +53,6 @@
 #include <kspread/Util.h>
 #include <kspread/Validity.h>
 
-#include "kspread/commands/DataManipulators.h"
-
 #include <math.h>
 
 #define SECS_PER_DAY 86400
@@ -2010,16 +2008,12 @@ KoFilter::ConversionStatus GNUMERICFilter::convert(const QByteArray & from, cons
 void GNUMERICFilter::setText(KSpread::Sheet* sheet, int _row, int _column, const QString& _text,
                              bool asString)
 {
-    DataManipulator* const command = new DataManipulator();
-    command->setSheet(sheet);
-    command->setValue(Value(_text));
-    command->setParsing(!asString);
-    command->add(QPoint(_column, _row));
-    command->execute();
-
-    //refresh anchor
-    if ((!_text.isEmpty()) && (_text.at(0) == '!')) {
-        sheet->updateView(KSpread::Region(_column, _row, sheet));
+    Cell cell(sheet, _column, _row);
+    if (asString) {
+        cell.setUserInput(_text);
+        cell.setValue(Value(_text));
+    } else {
+        cell.parseUserInput(_text);
     }
 }
 
