@@ -28,8 +28,13 @@
 #include <MsooXmlUtils.h>
 #include <MsooXmlSchemas.h>
 #include <MsooXmlContentTypes.h>
+#include <MsooXmlRelationships.h>
 #include "DocxXmlDocumentReader.h"
 #include "DocxXmlStylesReader.h"
+#include "DocxXmlNumberingReader.h"
+#include "DocxXmlFootnoteReader.h"
+#include "DocxXmlCommentsReader.h"
+#include "DocxXmlEndnoteReader.h"
 #include "DocxXmlFontTableReader.h"
 
 #include <QColor>
@@ -228,9 +233,6 @@ KoFilter::ConversionStatus DocxImport::parseParts(KoOdfWriters *writers, MSOOXML
         "\n        </style:list-level-properties>"
         "\n      </text:outline-level-style>"
         "\n    </text:outline-style>"
-        "\n    <text:notes-configuration text:note-class=\"footnote\" style:num-format=\"1\" text:start-value=\"0\" text:footnotes-position=\"page\" text:start-numbering-at=\"document\"/>"
-        "\n    <text:notes-configuration text:note-class=\"endnote\" style:num-format=\"i\" text:start-value=\"0\"/>"
-        "\n    <text:linenumbering-configuration text:number-lines=\"false\" text:offset=\"0.499cm\" style:num-format=\"1\" text:number-position=\"left\" text:increment=\"5\"/>"
         "\n    <!-- /COPIED -->"
     );
 
@@ -241,22 +243,83 @@ KoFilter::ConversionStatus DocxImport::parseParts(KoOdfWriters *writers, MSOOXML
         RETURN_IF_ERROR( loadAndParseDocumentIfExists(
             MSOOXML::ContentTypes::wordFontTable, &fontTableReader, writers, errorMessage, &context) )
     }
+
     // 2. parse themes
     QMap<QString, MSOOXML::DrawingMLTheme*> themes;
     MSOOXML::Utils::ContainerDeleter< QMap<QString, MSOOXML::DrawingMLTheme*> > themesDeleter(themes);
     RETURN_IF_ERROR( parseThemes(themes, writers, errorMessage) )
+
     // 3. parse styles
-    {
-        DocxXmlStylesReader stylesReader(writers);
-        RETURN_IF_ERROR( loadAndParseDocumentIfExists(
-            MSOOXML::ContentTypes::wordStyles, &stylesReader, writers, errorMessage) )
+    QList<QByteArray> partNames = this->partNames(d->mainDocumentContentType());
+    if (partNames.count() != 1) {
+        errorMessage = i18n("Unable to find part for type %1", d->mainDocumentContentType());
+        return KoFilter::WrongFormat;
     }
-    // 4. parse document
+    const QString documentPathAndFile(partNames.first());
+    QString documentPath, documentFile;
+    MSOOXML::Utils::splitPathAndFile(documentPathAndFile, &documentPath, &documentFile);
+    kDebug() << documentPathAndFile << documentPath << documentFile;
     {
+        // get styles path from document's relationships, not from content types; typically returns /word/styles.xml
+        // ECMA-376, 11.3.12 Style Definitions Part, p. 65
+        // An instance of this part type contains the definition for a set of styles used by this document.
+        // A package shall contain at most two Style Definitions parts. One instance of that part shall be
+        // the target of an implicit relationship from the Main Document (§11.3.10) part, and the other shall
+        // be the target of an implicit relationship in from the Glossary Document (§11.3.8) part.
+        const QString stylesPath(relationships->targetForType(documentPath, documentFile,
+            QLatin1String(MSOOXML::Schemas::officeDocument::relationships) + "/styles"));
+        kDebug() << "stylesPath:" << stylesPath;
+        DocxXmlStylesReader stylesReader(writers);
+        if (!stylesPath.isEmpty()) {
+            RETURN_IF_ERROR( loadAndParseDocumentFromFileIfExists(
+                stylesPath, &stylesReader, writers, errorMessage) )
+        }
+    }
+
+    // 4. parse numbering
+    {
+        const QString numberingPath(relationships->targetForType(documentPath, documentFile,
+        QLatin1String(MSOOXML::Schemas::officeDocument::relationships) + "/numbering"));
+        kDebug() << "numberingPath:" << numberingPath;
+        DocxXmlNumberingReader numberingReader(writers);
+        if (!numberingPath.isEmpty()) {
+            RETURN_IF_ERROR( loadAndParseDocumentFromFileIfExists(
+                numberingPath, &numberingReader, writers, errorMessage) )
+        }
+    }
+
+    // 5. parse footnotes
+    {
+        const QString footnotePath(relationships->targetForType(documentPath, documentFile,
+        QLatin1String(MSOOXML::Schemas::officeDocument::relationships) + "/footnotes"));
         //! @todo use m_contentTypes.values() when multiple paths are expected, e.g. for ContentTypes::wordHeader
         DocxXmlDocumentReaderContext context(
-            *this, QLatin1String("word"), QLatin1String("document.xml"),
-            *relationships, themes);
+            *this, documentPath, documentFile, *relationships, themes);
+        DocxXmlFootnoteReader footnoteReader(writers);
+        if (!footnotePath.isEmpty()) {
+            RETURN_IF_ERROR( loadAndParseDocumentFromFileIfExists(
+                footnotePath, &footnoteReader, writers, errorMessage, &context) )
+        }
+
+    // 6. parse comments
+        const QString commentPath(relationships->targetForType(documentPath, documentFile,
+        QLatin1String(MSOOXML::Schemas::officeDocument::relationships) + "/comments"));
+        DocxXmlCommentReader commentReader(writers);
+        if (!commentPath.isEmpty()) {
+            RETURN_IF_ERROR( loadAndParseDocumentFromFileIfExists(
+                commentPath, &commentReader, writers, errorMessage, &context) )
+        }
+
+    // 7. parse endnotes
+        const QString endnotePath(relationships->targetForType(documentPath, documentFile,
+        QLatin1String(MSOOXML::Schemas::officeDocument::relationships) + "/endnotes"));
+        DocxXmlEndnoteReader endnoteReader(writers);
+        if (!endnotePath.isEmpty()) {
+            RETURN_IF_ERROR( loadAndParseDocumentFromFileIfExists(
+                endnotePath, &endnoteReader, writers, errorMessage, &context) )
+        }
+
+    // 8. parse document
         DocxXmlDocumentReader documentReader(writers);
         RETURN_IF_ERROR( loadAndParseDocument(
             d->mainDocumentContentType(), &documentReader, writers, errorMessage, &context) )
