@@ -29,7 +29,6 @@
 
 #include <MsooXmlCommonReader.h>
 #include <MsooXmlThemesReader.h>
-#include "DocxXmlCommentsReader.h"
 #include "DocxXmlNotesReader.h"
 
 #include <KoXmlWriter.h>
@@ -71,9 +70,13 @@ protected:
     };
     KoFilter::ConversionStatus read_rPr(rPrCaller caller);
     KoFilter::ConversionStatus read_pPr();
+    KoFilter::ConversionStatus read_numPr();
+    KoFilter::ConversionStatus read_numId();
+    KoFilter::ConversionStatus read_ilvl();
     KoFilter::ConversionStatus read_sectPr();
     KoFilter::ConversionStatus read_footerReference();
     KoFilter::ConversionStatus read_headerReference();
+    KoFilter::ConversionStatus read_cols();
     KoFilter::ConversionStatus read_pgSz();
     KoFilter::ConversionStatus read_textDirection();
     KoFilter::ConversionStatus read_pgMar();
@@ -83,10 +86,18 @@ protected:
     KoFilter::ConversionStatus read_outline();
     KoFilter::ConversionStatus read_framePr();
     KoFilter::ConversionStatus read_OLEObject();
+    KoFilter::ConversionStatus read_webHidden();
+    KoFilter::ConversionStatus read_bookmarkStart();
+    KoFilter::ConversionStatus read_bookmarkEnd();
     //KoFilter::ConversionStatus read_commentRangeEnd();
     KoFilter::ConversionStatus read_commentRangeStart();
     KoFilter::ConversionStatus read_endnoteReference();
     KoFilter::ConversionStatus read_footnoteReference();
+    KoFilter::ConversionStatus read_footnotePr();
+    KoFilter::ConversionStatus read_endnotePr();
+    KoFilter::ConversionStatus read_lnNumType();
+    KoFilter::ConversionStatus read_numFmt();
+    KoFilter::ConversionStatus read_suppressLineNumbers();
     KoFilter::ConversionStatus read_hyperlink();
     KoFilter::ConversionStatus read_drawing();
     KoFilter::ConversionStatus read_ptab();
@@ -98,9 +109,12 @@ protected:
     KoFilter::ConversionStatus read_sz();
     KoFilter::ConversionStatus read_jc();
     KoFilter::ConversionStatus read_spacing();
+    KoFilter::ConversionStatus read_trPr();
+    KoFilter::ConversionStatus read_trHeight();
     enum shdCaller {
         shd_rPr,
-        shd_pPr
+        shd_pPr,
+        shd_tcPr
     };
     KoFilter::ConversionStatus read_shd(shdCaller caller);
     KoFilter::ConversionStatus read_rFonts();
@@ -141,6 +155,7 @@ protected:
     KoFilter::ConversionStatus read_align(alignCaller caller);
 
     KoFilter::ConversionStatus read_inline();
+    KoFilter::ConversionStatus read_extent();
     KoFilter::ConversionStatus read_docPr();
     KoFilter::ConversionStatus read_anchor();
     KoFilter::ConversionStatus read_positionH();
@@ -155,6 +170,8 @@ protected:
 
     DocxXmlDocumentReaderContext* m_context;
 
+    KoOdfWriters *m_writers; // Needed to create new relationship for header/footer
+
 private:
     void init();
 
@@ -192,7 +209,8 @@ private:
     void applyBorders(KoGenStyle *style);
 
     enum ComplexFieldCharType {
-       NoComplexFieldCharType, HyperlinkComplexFieldCharType
+       NoComplexFieldCharType, HyperlinkComplexFieldCharType, ReferenceComplexFieldCharType,
+       ReferenceNextComplexFieldCharType
     };
     //! Type of complex field characters we have
     ComplexFieldCharType m_complexCharType;
@@ -218,6 +236,8 @@ private:
     QString m_dropCapLines;
     qreal   m_dropCapDistance;
 
+    QMap<QString, QString> m_bookmarks; //!< Bookmarks
+
     uint m_currentTableNumber; //!< table counter, from 0
     uint m_currentTableRowNumber; //!< row counter, from 0, initialized in read_tbl()
     uint m_currentTableColumnNumber; //!< column counter, from 0, initialized in read_tr()
@@ -226,6 +246,11 @@ private:
     KoGenStyle m_currentTableCellStyle;
     QString m_currentTableName;
     qreal m_currentTableWidth; //!< in cm
+    bool m_wasCaption; // bookkeeping to ensure next para is suppressed if a caption is encountered
+
+    bool m_closeHyperlink; // should read_r close hyperlink
+    bool m_listFound; // was there numPr element in ppr
+    QString m_currentListStyleName;
 
     /*! true if w:object/v:shape or w:object/o:OLEObject has been handled, .
      When w:object/o:OLEObject is visited and m_objectRectInitialized is true, handling
@@ -242,6 +267,9 @@ private:
     QString m_currentObjectXCm; //!< See m_currentObjectWidthCm for description
     QString m_currentObjectYCm; //!< See m_currentObjectWidthCm for description
 
+    QMap<QString, QString> m_headers;
+    QMap<QString, QString> m_footers;
+
 #include <MsooXmlCommonReaderMethods.h>
 #include <MsooXmlCommonReaderDrawingMLMethods.h>
 
@@ -264,27 +292,16 @@ public:
     const QString path;
     const QString file;
 
-    //! @return comment for identifier @a id. Use Comment::isNull() to check if the item was found.
-    DocxComment comment(KoOdfWriters *writers, int id);
-
-    //! @return endnote for identifier @a id. Use Comment::isNull() to check if the item was found.
-    DocxNote endnote(KoOdfWriters *writers, int id);
-
-    //! @return footnote for identifier @a id. Use Comment::isNull() to check if the item was found.
-    DocxNote footnote(KoOdfWriters *writers, int id);
-
     const QMap<QString, MSOOXML::DrawingMLTheme*>* themes;
-private:
-    KoFilter::ConversionStatus loadComments(KoOdfWriters *writers);
-    KoFilter::ConversionStatus loadEndnotes(KoOdfWriters *writers);
-    KoFilter::ConversionStatus loadFootnotes(KoOdfWriters *writers);
 
-    QMap<int, DocxComment> m_comments;
-    QMap<int, DocxNote> m_endnotes;
-    QMap<int, DocxNote> m_footnotes;
-    bool m_commentsLoaded;
-    bool m_endnotesLoaded;
-    bool m_footnotesLoaded;
+    // Contains footnotes when read, the styles of footnotes are already put to correct files.
+    QMap<QString, QString> m_footnotes;
+
+    QMap<QString, QString> m_comments;
+
+    QMap<QString, QString> m_endnotes;
+
+private:
 };
 
 #endif //DOCXXMLDOCREADER_H
