@@ -469,7 +469,7 @@ bool HtmlWorker::doFullParagraph(const QString& paraText,
 
     ProcessParagraphData(strTag, strParaText, layout, paraFormatDataList);
 
-    kDebug(30503) <<"Quiting HtmlWorker::doFullParagraph";
+    kDebug(30503) <<"Quitting HtmlWorker::doFullParagraph";
     return true;
 }
 
@@ -587,7 +587,7 @@ bool HtmlWorker::doOpenHead(void)
     *m_streamOut << (isXML()?" /":"") << ">\n" ;
 
     // Say who we are (with the CVS revision number) in case we have a bug in our filter output!
-    QString strVersion("$Revision: 695939 $");
+    QString strVersion("$Revision: 876880 $");
     // Eliminate the dollar signs
     //  (We don't want that the version number changes if the HTML file is itself put in a CVS storage.)
     *m_streamOut << "<meta name=\"Generator\" content=\"KWord HTML Export Filter Version"
