@@ -12,16 +12,18 @@
  *
  * You should have received a copy of the GNU Library General Public License
  * along with this library; see the file COPYING.LIB.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include "latex.h"
 #include <element.h>
+//Added by qt3to4:
+#include <Q3PtrList>
 
 Latex* Latex::_instance = 0;
 
-Element* Latex::getEnv(QPtrList<Element>* root, const char* name)
+Element* Latex::getEnv(Q3PtrList<Element>* root, const char* name)
 {
 	Element* elt;
 	for ( elt = root->first(); elt; elt = root->next() )
@@ -32,7 +34,7 @@ Element* Latex::getEnv(QPtrList<Element>* root, const char* name)
 	return 0;
 }
 
-Element* Latex::getCommand(QPtrList<Element>* root, const char* name)
+Element* Latex::getCommand(Q3PtrList<Element>* root, const char* name)
 {
 	Element* elt;
 	for ( elt = root->first(); elt; elt = root->next() )
