@@ -1,5 +1,3 @@
-// $Header$
-
 /*
    This file is part of the KDE project
    Copyright (C) 2001 Nicolas GOUTTE <goutte@kde.org>
@@ -16,21 +14,19 @@
 
    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
 */
 
 #ifndef KWEF_UTIL_H
 #define KWEF_UTIL_H
 
-#include <koffice_export.h>
+#include <kword_libexport_export.h>
 namespace KWEFUtil
 {
 
-KOFFICEFILTER_EXPORT QString EscapeSgmlText(const QTextCodec* codec, const QString& strIn,
+KWORD_LIBEXPORT_EXPORT QString EscapeSgmlText(const QTextCodec* codec, const QString& strIn,
     const bool quot = false, const bool apos = false );
-KOFFICEFILTER_EXPORT void GetNativePaperFormat(const int format,
-    QString& width, QString& height, QString& units);
 }
 
 // Deprecated! Use KWEFUtil::EscapeSgmlText
