@@ -24,6 +24,7 @@
 
 #include <QBuffer>
 
+#include <qlist.h>
 #include <KoGenStyle.h>
 #include <KoGenStyles.h>
 #include <KoXmlWriter.h>
@@ -40,11 +41,11 @@
 class Paragraph
 {
 public:
-    explicit Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml = false, bool isHeading = false, int outlineLevel = 0);
+    explicit Paragraph(KoGenStyles* mainStyles, bool inStylesDotXml = false, bool isHeading = false, bool inHeader = false, int outlineLevel = 0);
     ~Paragraph();
 
     void writeToFile(KoXmlWriter* writer);
-    void addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word97::CHP> chp, QString fontName, const wvWare::StyleSheet& styles);
+    void addRunOfText(QString text,  wvWare::SharedPtr<const wvWare::Word97::CHP> chp, QString fontName, const wvWare::StyleSheet& styles, bool addCompleteElement=false);
     void openInnerParagraph();
     void closeInnerParagraph();
     void setParagraphProperties(wvWare::SharedPtr<const wvWare::ParagraphProperties> properties);
@@ -52,13 +53,32 @@ public:
     // Set the general named style that applies to this paragraph
     void setParagraphStyle(const wvWare::Style* paragraphStyle);
     KoGenStyle* getOdfParagraphStyle();
+    bool containsPageNumberField() const {
+        return m_containsPageNumberField;
+    }
+    void setContainsPageNumberField(bool containsPageNumberField) {
+        m_containsPageNumberField = containsPageNumberField;
+    }
+
+    typedef enum { NoDropCap, IsDropCapPara, HasDropCapIntegrated }  DropCapStatus;
+    DropCapStatus dropCapStatus() const;
+    void getDropCapData(QString *string, int *type, int *lines, qreal *distance) const;
+    void addDropCap(QString &string, int type, int lines, qreal distance);
+
+    // debug:
+    int  strings() const;
+    QString string(int index) const;
 
     // Static functions for parsing wvWare properties and applying
     // them onto a KoGenStyle.
     static void applyParagraphProperties(const wvWare::ParagraphProperties& properties,
-                                         KoGenStyle* style, const wvWare::Style* parentStyle);
-    static void applyCharacterProperties(const wvWare::Word97::CHP* chp, KoGenStyle* style, const wvWare::Style* parentStyle, bool suppressFontSize=false);
+                                         KoGenStyle* style, const wvWare::Style* parentStyle,
+                                         bool setDefaultAlign, Paragraph *paragraph);
+    static void applyCharacterProperties(const wvWare::Word97::CHP* chp,
+                                         KoGenStyle* style, const wvWare::Style* parentStyle,
+                                         bool suppressFontSize=false, bool combineCharacters=false);
 
+    void setCombinedCharacters(bool isCombined);
 private:
     wvWare::SharedPtr<const wvWare::ParagraphProperties> m_paragraphProperties;
     wvWare::SharedPtr<const wvWare::ParagraphProperties> m_paragraphProperties2;
@@ -72,15 +92,28 @@ private:
     const wvWare::Style* m_paragraphStyle;  // style for the paragraph
     const wvWare::Style* m_paragraphStyle2; // style when in inner paragraph
 
-    std::vector<QString> m_textStrings; // list of text strings within a paragraph
-    std::vector<QString> m_textStrings2; // original list when in inner paragraph
+    //std::vector<QString> m_textStrings; // list of text strings within a paragraph
+    //std::vector<QString> m_textStrings2; // original list when in inner paragraph
+    QList<QString> m_textStrings; // list of text strings within a paragraph
+    QList<QString> m_textStrings2; // original list when in inner paragraph
     std::vector<const KoGenStyle*> m_textStyles; // list of styles for text within a paragraph
     std::vector<const KoGenStyle*> m_textStyles2; // original list when in inner paragraph
+    std::vector<bool> m_addCompleteElement;         // list of flags if we should output the complete parahraph instead of processing it
+    std::vector<bool> m_addCompleteElement2;        // original list when in inner paragraph
 
     bool m_inStylesDotXml; //let us know if we're in content.xml or styles.xml
     bool m_isHeading; //information for writing a heading instead of a paragraph
     // (odt looks formats them similarly)
     int m_outlineLevel;
+
+    DropCapStatus  m_dropCapStatus; // True if this paragraph has a dropcap 
+    int   m_dcs_fdct;
+    int   m_dcs_lines;
+    qreal m_dropCapDistance;
+
+    bool m_inHeaderFooter;
+    bool m_containsPageNumberField;
+    bool m_combinedCharacters;            // is true when the next characters are combined
 }; //end class Paragraph
 
 #endif //PARAGRAPH_H
