@@ -387,8 +387,7 @@ void KWordTextHandler::footnoteFound(wvWare::FootnoteData::Type type,
     //varElem.appendChild( footnoteElem );
 }
 
-void KWordTextHandler::bookmarkFound( wvWare::UString characters, wvWare::UString name, wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
-                                      const wvWare::BookmarkFunctor& parseBookmark)
+void KWordTextHandler::bookmarkFound( wvWare::UString characters, wvWare::UString name, wvWare::SharedPtr<const wvWare::Word97::CHP> chp)
 {
     // Get the text between the bookmark-start and the bookmark-end
     QString bookmarkText;
@@ -433,13 +432,6 @@ void KWordTextHandler::bookmarkFound( wvWare::UString characters, wvWare::UStrin
     m_bookmarkWriter->addAttribute("text:name", bookmarkName);
     m_bookmarkWriter->endElement();
 
-    //save the state of tables & paragraphs because we'll get new ones in the footnote
-    saveState();
-    //signal Document to parse the bookmark
-    emit bookmarkFound(new wvWare::BookmarkFunctor(parseBookmark));
-    //and now restore state
-    restoreState();
-
     m_insideBookmark = false;
 
     QString contents = QString::fromUtf8(m_bookmarkBuffer->buffer(), m_bookmarkBuffer->buffer().size());
