@@ -81,8 +81,7 @@ public:
                                wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                const wvWare::FootnoteFunctor& parseFootnote);
     virtual void bookmarkFound(wvWare::UString characters, wvWare::UString name,
-                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
-                               const wvWare::BookmarkFunctor& parseBookmark);
+                               wvWare::SharedPtr<const wvWare::Word97::CHP> chp);
     virtual void annotationFound(wvWare::UString characters,
                                  wvWare::SharedPtr<const wvWare::Word97::CHP> chp,
                                  const wvWare::AnnotationFunctor& parseAnnotation);
@@ -134,7 +133,6 @@ signals:
     void sectionEnd(wvWare::SharedPtr<const wvWare::Word97::SEP>);
     void subDocFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void footnoteFound(const wvWare::FunctorBase* parsingFunctor, int data);
-    void bookmarkFound(const wvWare::FunctorBase* parsingFunctor);
     void annotationFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void headersFound(const wvWare::FunctorBase* parsingFunctor, int data);
     void tableFound(KWord::Table* table);
