@@ -53,7 +53,6 @@
 
 using namespace wvWare;
 
-
 Parser9x::Position::Position( U32 cp, const PLCF<Word97::PCD>* plcfpcd ) :
         piece( 0 ), offset( cp )
 {
@@ -213,8 +212,8 @@ OLEStreamReader* Parser9x::getTable()
 void Parser9x::parseHeaders( const HeaderData& data )
 {
     m_subDocumentHandler->headersStart();
-    for ( unsigned char mask = HeaderData::HeaderEven; 
-          mask <= HeaderData::FooterFirst; mask <<= 1 ) 
+    for ( unsigned char mask = HeaderData::HeaderEven;
+          mask <= HeaderData::FooterFirst; mask <<= 1 )
     {
         if ( mask & data.headerMask ) {
             parseHeader( data, mask );
@@ -246,30 +245,6 @@ void Parser9x::parseFootnote( const FootnoteData& data )
 #endif
 }
 
-void Parser9x::parseBookmark( const BookmarkData& data )
-{
-#ifdef WV2_DEBUG_BOOKMARK
-    wvlog << "Parser9x::parseBookmark()" << endl;
-#endif
-    if ( data.startCP == 0 ) // shouldn't happen, but well...
-        return;
-
-    saveState( data.limCP - data.startCP, Bookmark );
-    m_subDocumentHandler->bookmarkStart();
-
-
-    U32 offset = data.startCP;
-
-    parseHelper( Position( offset, m_plcfpcd ) );
-
-    m_subDocumentHandler->bookmarkEnd();
-    restoreState();
-#ifdef WV2_DEBUG_BOOKMARK
-    wvlog << "Parser9x::parseBookmark() done" << endl;
-#endif
-}
-
-
 void Parser9x::parseAnnotation( const AnnotationData& data )
 {
 #ifdef WV2_DEBUG_ANNOTATIONS
@@ -295,7 +270,7 @@ void Parser9x::parseTableRow( const TableRowData& data )
 {
 #ifdef WV2_DEBUG_TABLES
     wvlog << "Parser9x::parseTableRow(): startPiece=" << data.startPiece << " startOffset="
-            << data.startOffset << " length=" << data.length << endl;
+          << data.startOffset << " length=" << data.length << endl;
 #endif
 
     if ( data.length == 0 ) // idiot safe ;-)
@@ -333,8 +308,8 @@ void Parser9x::parseTextBox( uint lid, bool bodyDrawing)
 
     PLCFIterator<Word97::FTXBXS> it( plcftxbxTxt->at( 0 ) );
 
-    for(size_t i = 0; i < plcftxbxTxt->count(); i++, ++it) {
-        if(it.current()->lid == (S32)lid) {
+    for (size_t i = 0; i < plcftxbxTxt->count(); i++, ++it) {
+        if (it.current()->lid == (S32)lid) {
             saveState( it.currentLim() - it.currentStart(), TextBox );
 
             U32 offset = m_fib.ccpText + it.currentStart();
@@ -385,7 +360,6 @@ void Parser9x::init()
     if ( m_fib.ccpAtn != 0 ) {
         m_annotations = new Annotations( m_table, m_fib );
     }
-
 }
 
 bool Parser9x::readPieceTable()
@@ -676,8 +650,8 @@ void Parser9x::processParagraph( U32 fc )
                     << m_tableRowStart->offset << endl;
 #endif
         }
-        m_tableRowLength += std::accumulate( m_currentParagraph->begin(), m_currentParagraph->end(),
-                                             1, &Parser9x::accumulativeLength ); // init == 1 because of the parag. mark!
+        // init == 1 because of the parag. mark!
+        m_tableRowLength += std::accumulate( m_currentParagraph->begin(), m_currentParagraph->end(), 1, &Parser9x::accumulativeLength );
         if ( props->pap().fTtp ) {
             // Restore the table properties of this row
             Word97::TAP* tap = m_properties->fullSavedTap( fc, m_data );
@@ -702,7 +676,8 @@ void Parser9x::processParagraph( U32 fc )
         // Now that we have the complete PAP, let's see if this paragraph belongs to a list
         props->createListInfo( *m_lists );
 
-        SharedPtr<const ParagraphProperties> sharedProps( props ); // keep it that way, else the ParagraphProperties get deleted!
+        // keep it that way, else the ParagraphProperties get deleted!
+        SharedPtr<const ParagraphProperties> sharedProps( props );
         m_textHandler->paragraphStart( sharedProps );
 
         // Get the appropriate style for this paragraph
@@ -728,7 +703,8 @@ void Parser9x::processParagraph( U32 fc )
                 length = length > limit - index ? limit - index : length;
 
                 m_properties->applyClxGrpprl( pcdIt.current(), m_fib.fcClx, chp, style );
-                SharedPtr<const Word97::CHP> sharedChp( chp ); // keep it that way, else the CHP gets deleted!
+                // keep it that way, else the CHP gets deleted!
+                SharedPtr<const Word97::CHP> sharedChp( chp );
                 processChunk( *it, chp, length, index, pcdIt.currentStart() );
                 index += length;
             }
@@ -772,7 +748,7 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
             disruption = nextFtn < nextEnd ? nextFtn : nextEnd;
 #ifdef WV2_DEBUG_FOOTNOTES
             wvlog << "nextFtn=" << nextFtn << " nextEnd=" << nextEnd << " disruption="
-                    << disruption << " length=" << length << endl;
+                  << disruption << " length=" << length << endl;
 #endif
         } else if ( m_bookmark ) {
             U32 nextBkf = m_bookmark->nextBookmarkStart();
@@ -780,7 +756,7 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
             disruption = nextBkf < nextBkl ? nextBkf : nextBkl;
 #ifdef WV2_DEBUG_BOOKMARK
             wvlog << "nextBkf=" << nextBkf << " nextBkl=" << nextBkl << " disruption="
-                    << disruption << " length=" << length << endl;
+                  << disruption << " length=" << length << endl;
 #endif
         }
         U32 startCP = currentStart + chunk.m_position.offset + index;
@@ -788,11 +764,13 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
         if ( disruption >= startCP && disruption < startCP + length ) {
 
 #if defined WV2_DEBUG_FOOTNOTES || defined WV2_DEBUG_BOOKMARK
-            wvlog << "startCP=" << startCP << " len=" << length << " disruption=" << disruption << endl;
+            wvlog << "startCP=" << startCP << " len=" << length << " disruption=" 
+                  << disruption << endl;
 #endif
             U32 disLen = disruption - startCP;
-            if ( disLen != 0 )
+            if ( disLen != 0 ) {
                 processRun( chunk, chp, disLen, index, currentStart );
+            }
             length -= disLen;
             index += disLen;
 
@@ -805,12 +783,11 @@ void Parser9x::processChunk( const Chunk& chunk, SharedPtr<const Word97::CHP> ch
                 emitBookmark( m_bookmarkText, disruption, chp, length );
                 m_bookmarkText = "";
             }
-            index+=length;
-            length=0;
+            index += length;
+            length = 0;
         }
         else {
-
-            // common case, no disruption at all (or the end of a disrupted chunk)
+            //common case, no disruption at all (or the end of a disrupted chunk)
             //In case of custom footnotes do not add label to footnote body.
             if ( m_footnotes ) {
                 if (m_customFootnote.find(chunk.m_text.substr(index, length), 0) != 0)
@@ -947,15 +924,15 @@ void Parser9x::emitBookmark( UString characters, U32 globalCP, SharedPtr<const W
         wvlog << "Bug: Found a bookmark, but m_bookmark == 0!" << endl;
         return;
     }
-#ifdef WV2_DEBUG_BOOKMARK
-    wvlog << "Bookmark found: CP=" << globalCP << endl;
-#endif
     bool ok;
     BookmarkData data( m_bookmark->bookmark( globalCP, ok ) );
 
     if (ok) {
+#ifdef WV2_DEBUG_BOOKMARK
+    wvlog << "Bookmark found: CP=" << globalCP << endl;
+#endif
         // We introduce the name of the bookmark here with data.name
-        m_textHandler->bookmarkFound( characters, data.name, chp, make_functor( *this, &Parser9x::parseBookmark, data ));
+        m_textHandler->bookmarkFound( characters, data.name, chp);
     }
 }
 
@@ -978,30 +955,31 @@ void Parser9x::emitAnnotation( UString characters, U32 globalCP, SharedPtr<const
 
 void Parser9x::emitHeaderData( SharedPtr<const Word97::SEP> sep )
 {
-    // We don't care about non-existant headers
+    // We don't care about non-existent headers
     if ( !m_headers ) {
         return;
     }
-
-    // MS Word stores headers in a very strange way, so we have to keep track
-    // of the section numbers. We use a 0-based index for convenience inside
-    // the header reading code. (Werner)
+    // NOTE: MS Word stores headers in a very strange way, so we have to keep
+    // track of the section numbers.  We use a 0-based index for convenience
+    // inside the header reading code. (Werner)
     //
-    // Of course the file format has changed between Word 6/7 and Word 8, so
-    // I had to add a workaround... oh well.
+    // Of course the file format has changed between Word 6/7 and Word 8, so I
+    // had to add a workaround... oh well.
     HeaderData data( m_sectionNumber++ );
-
+    
     if ( m_fib.nFib < Word8nFib ) {
         data.headerMask = sep->grpfIhdt;
-        m_headers->headerMask( sep->grpfIhdt );
+        m_headers->set_headerMask( sep->grpfIhdt );
     }
     else {
-        if ( sep->fTitlePage ) {
-            data.headerMask |= HeaderData::HeaderFirst | HeaderData::FooterFirst;
-        }
+	//check if an even header/footer is expected
         if ( dop().fFacingPages ) {
             data.headerMask |= HeaderData::HeaderEven | HeaderData::FooterEven;
         }
+	//check if a first page header/footer is expected
+        if ( sep->fTitlePage ) {
+            data.headerMask |= HeaderData::HeaderFirst | HeaderData::FooterFirst;
+        }
     }
     m_textHandler->headersFound( make_functor( *this, &Parser9x::parseHeaders, data ) );
 }
@@ -1074,6 +1052,7 @@ void Parser9x::emitPictureData( SharedPtr<const Word97::CHP> chp )
     PictureData data( offset, sharedPicf );
     m_textHandler->inlineObjectFound(data, chp);
 
+    //NOTE: the depreciated approach to parse an inline object follows
 //     PictureFunctor fnct = make_functor( *this, &Parser9x::parsePicture,
 //				 PictureData( static_cast<U32>( chp->fcPic_fcObj_lTagObj ),
 //					     sharedPicf ) );
