@@ -21,12 +21,12 @@
 #ifndef _IMPORT_DIALOG_H
 #define _IMPORT_DIALOG_H
 
-#include <qwidget.h>
+#include <QWidget>
 
-#include <kdialogbase.h>
+#include <kdialog.h>
 
 class ImportDialogUI;
-class MSWriteImportDialog : public KDialogBase
+class MSWriteImportDialog : public KDialog
 {
     Q_OBJECT
 public:
