@@ -17,17 +17,36 @@
  * Boston, MA 02110-1301, USA.
  */
 #include "KoFormulaShapeFactory.h"
+
+#include <klocale.h>
+#include <kdebug.h>
+
+#include <KoShapeFactoryBase.h>
+#include <KoXmlNS.h>
+
 #include "KoFormulaShape.h"
 
-#include <KoShapeFactory.h>
-#include <klocale.h>
 
 KoFormulaShapeFactory::KoFormulaShapeFactory( QObject *parent )
-    : KoShapeFactory( parent, KoFormulaShapeId, i18n( "A formula shape" ) )
+    : KoShapeFactoryBase( parent, KoFormulaShapeId, i18n( "Formula" ) )
 {
-    setToolTip( i18n( "A formula" ) );
-    setIcon( "formulashape" );
+    setToolTip(i18n( "A formula"));
+    setIcon( "x-shape-formula" );
 
+    // The following lines let the formula shape load both embedded and
+    // inline formulas.
+    //
+    // The line below tells the shape registry which XML elements that
+    // the shape supports.
+    //
+    // FIXME: Find out if inline formulas are supported by ODF.
+
+    QList<QPair<QString, QStringList> > elementNamesList;
+    elementNamesList.append(qMakePair(QString(KoXmlNS::draw), QStringList("object")));
+    elementNamesList.append(qMakePair(QString(KoXmlNS::math), QStringList("math")));
+    setOdfElements(elementNamesList);
+
+    setLoadingPriority( 1 );
 /*    KoShapeTemplate t;
     t.id = KoFormulaShapeId;
     t.name = i18n("Formula");
@@ -41,23 +60,26 @@ KoFormulaShapeFactory::KoFormulaShapeFactory( QObject *parent )
 KoFormulaShapeFactory::~KoFormulaShapeFactory()
 {}
 
-KoShape* KoFormulaShapeFactory::createDefaultShape() const
+KoShape *KoFormulaShapeFactory::createDefaultShape(KoResourceManager *resourceManager) const
 {
-    KoFormulaShape* formula = new KoFormulaShape();
+    KoFormulaShape* formula = new KoFormulaShape(resourceManager);
     formula->setShapeId( KoFormulaShapeId );
     return formula;
 }
 
-KoShape* KoFormulaShapeFactory::createShape( const KoProperties* params ) const
+bool KoFormulaShapeFactory::supports(const KoXmlElement& e) const
 {
-    Q_UNUSED( params );
+    bool retval = ((e.localName() == "math"
+                    && e.namespaceURI() == KoXmlNS::math)
+                   || (e.localName() == "object"
+                       && e.namespaceURI() == KoXmlNS::draw));
 
-    KoFormulaShape* formula = new KoFormulaShape();
-    if( !formula )
-        return 0;
+    // Should be 39001 (kformula) instead of 31000 (koffice)
+    //kDebug(31000) << e.nodeName() << " - "<< e.namespaceURI(); 
+    //kDebug(31000) << "Return value = " << retval;
 
-    formula->setShapeId( KoFormulaShapeId );
-    return formula;
+    return retval;
 }
 
+
 #include "KoFormulaShapeFactory.moc"
