@@ -23,6 +23,7 @@
 #include <QPainter>
 #include <QPaintEvent>
 #include <QWidget>
+#include <QVariant>
 #include <kapplication.h>
 #include <kdebug.h>
 #include <KoPageLayout.h>
@@ -243,6 +244,14 @@ bool KPrAnimationDirector::shapeShown( KoShape * shape )
     return true;
 }
 
+QTransform KPrAnimationDirector::shapeTransform( KoShape * shape )
+{
+    if(hasAnimation()){
+        return m_animationCache->value(shape, "transform", QTransform()).value<QTransform>();
+    }
+    return QTransform();
+}
+
 void KPrAnimationDirector::updateActivePage( KoPAPageBase * page )
 {
     if ( m_canvas == m_view->kopaCanvas() ) {
@@ -283,15 +292,14 @@ void KPrAnimationDirector::updateActivePage( KoPAPageBase * page )
 
 void KPrAnimationDirector::updatePageAnimation()
 {
-    if(m_animationCache != 0){
-        delete(m_animationCache);
-    }
-    KoPAPageBase * page = m_pages[m_pageIndex];
-    KPrPage * kprPage = dynamic_cast<KPrPage *>( page );
+    // TODO add a clear method to the cache so we don't need to recreated it again
+    delete m_animationCache;
     m_animationCache = new KPrAnimationCache();
 
+    m_animationCache->setPageSize(m_pages[m_pageIndex]->size());
+
     int i = 0;
-    foreach(KPrAnimationStep * step, m_animations){
+    foreach (KPrAnimationStep *step, m_animations) {
         step->init(m_animationCache, i);
         i++;
     }
@@ -438,7 +446,12 @@ void KPrAnimationDirector::previousStep()
         if ( m_pageIndex > 0 ) {
             --m_pageIndex;
             updateActivePage( m_pages[m_pageIndex] );
-            m_stepIndex = m_animations.size() - 1;
+            if(hasAnimation()) {
+                m_stepIndex = m_animations.size() - 1;
+            }
+            else {
+                m_stepIndex = m_animations.size();
+            }
             updatePageAnimation();
             // trigger repaint
             m_canvas->update();
@@ -464,6 +477,7 @@ void KPrAnimationDirector::animate()
     }
     else if ( hasAnimation() ) { //if there are animnations
         // set current time, to the current step
+        m_animationCache->next();
         m_animations.at(m_stepIndex)->setCurrentTime(m_timeLine.currentTime());
         m_canvas->update();
     }
