@@ -22,6 +22,9 @@
 
 #include "KPrAnimationBase.h"
 
+class KPrAnimationValue;
+class KPrAnimationAttribute;
+
 class KPrAnimate : public KPrAnimationBase
 {
 public:
@@ -36,14 +39,8 @@ protected:
     virtual void next(int currentTime);
 
 private:
-    enum Axis {
-        X,
-        Y
-    };
-
-    Axis m_axis;
-    int m_from;
-    int m_to;
+    KPrAnimationAttribute * m_attribute;
+    KPrAnimationValue * m_values;
 };
 
 #endif /* KPRANIMATE_H */
