@@ -21,7 +21,12 @@
 
 #include "KPrAnimationCache.h"
 
-KPrAnimateMotion::KPrAnimateMotion()
+#include <KoShapeLoadingContext.h>
+#include <KoShapeSavingContext.h>
+#include <KoXmlReader.h>
+
+KPrAnimateMotion::KPrAnimateMotion(KPrShapeAnimation *shapeAnimation)
+: KPrAnimationBase(shapeAnimation)
 {
 }
 
@@ -31,14 +36,21 @@ KPrAnimateMotion::~KPrAnimateMotion()
 
 bool KPrAnimateMotion::loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context)
 {
+    KPrAnimationBase::loadOdf(element, context);
     return false;
 }
 
 void KPrAnimateMotion::saveOdf( KoShapeSavingContext &context ) const
 {
+    Q_UNUSED(context);
 }
 
 
-void KPrAnimateMotion::init(KPrAnimationCache *animationCache) const
+void KPrAnimateMotion::init(KPrAnimationCache *animationCache, int step)
+{
+    Q_UNUSED(animationCache);
+}
+
+void KPrAnimateMotion::next(int currentTime)
 {
 }
