@@ -20,9 +20,14 @@
 #include "KPrAnimateTransform.h"
 
 #include "KPrAnimationCache.h"
+#include <KoShapeLoadingContext.h>
+#include <KoShapeSavingContext.h>
+#include <KoXmlReader.h>
 
-KPrAnimateTransform::KPrAnimateTransform()
+KPrAnimateTransform::KPrAnimateTransform(KPrShapeAnimation *shapeAnimation)
+: KPrAnimationBase(shapeAnimation)
 {
+    m_duration = 1;
 }
 
 KPrAnimateTransform::~KPrAnimateTransform()
@@ -31,13 +36,20 @@ KPrAnimateTransform::~KPrAnimateTransform()
 
 bool KPrAnimateTransform::loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context)
 {
+    KPrAnimationBase::loadOdf(element, context);
     return true;
 }
 
 void KPrAnimateTransform::saveOdf(KoShapeSavingContext &context) const
 {
+    Q_UNUSED(context);
 }
 
-void KPrAnimateTransform::init(KPrAnimationCache *animationCache) const
+void KPrAnimateTransform::init(KPrAnimationCache *animationCache, int step)
+{
+    Q_UNUSED(animationCache);
+}
+
+void KPrAnimateTransform::next(int currentTime)
 {
 }
