@@ -24,6 +24,7 @@
 #include <QDomElement>
 #include <QLinkedList>
 #include <QSharedData>
+#include <QVariant>
 
 #include "kspread_export.h"
 #include <KoXmlReader.h>
@@ -42,6 +43,7 @@ class StyleManager;
 
 /**
  * \class Conditional
+ * \ingroup Style
  * Conditional formatting.
  * Holds the actual condition and the applicable style for conditional
  * Cell formattings.
@@ -50,7 +52,8 @@ class KSPREAD_EXPORT Conditional
 {
 public:
     enum Type { None, Equal, Superior, Inferior, SuperiorEqual,
-        InferiorEqual, Between, Different, DifferentTo };
+                InferiorEqual, Between, Different, DifferentTo
+              };
 
     double         val1;
     double         val2;
@@ -59,20 +62,22 @@ public:
     QColor  *      colorcond;
     QFont   *      fontcond;
     QString *      styleName;
-    Style *        style;
     Type           cond;
 
     Conditional();
     ~Conditional();
-    Conditional( const Conditional& );
-    Conditional& operator=( const Conditional& );
+    Conditional(const Conditional&);
+    Conditional& operator=(const Conditional&);
     bool operator==(const Conditional& other) const;
-    inline bool operator!=( const Conditional& other ) const { return !operator==( other ); }
+    inline bool operator!=(const Conditional& other) const {
+        return !operator==(other);
+    }
 };
 
 
 /**
  * \class Conditions
+ * \ingroup Style
  * Manages a set of conditions for a cell.
  */
 class KSPREAD_EXPORT Conditions
@@ -86,7 +91,7 @@ public:
     /**
      * Copy Constructor.
      */
-    Conditions( const Conditions& other );
+    Conditions(const Conditions& other);
 
     /**
      * Destructor..
@@ -101,7 +106,7 @@ public:
     /**
      * \return the style that matches first (or 0 if no condition matches)
      */
-    Style* testConditions( const Cell& cell ) const;
+    Style* testConditions(const Cell& cell, const StyleManager* styleManager) const;
 
     /**
      * Retrieve the current list of conditions we're checking
@@ -111,39 +116,49 @@ public:
     /**
      * Replace the current list of conditions with this new one
      */
-    void setConditionList( const QLinkedList<Conditional> & list );
+    void setConditionList(const QLinkedList<Conditional> & list);
 
     /**
      * \ingroup NativeFormat
      * Takes a parsed DOM element and recreates the conditions structure out of
      * it
      */
-    void loadConditions( const StyleManager* styleManager, const KoXmlElement & element );
+    void loadConditions(const StyleManager* styleManager, const KoXmlElement & element);
 
     /**
      * \ingroup NativeFormat
      * Saves the conditions to a DOM tree structure.
      * \return the DOM element for the conditions.
      */
-    QDomElement saveConditions( QDomDocument & doc ) const;
+    QDomElement saveConditions(QDomDocument & doc) const;
+
+    /**
+     * \ingroup OpenDocument
+     * Loads the condtional formatting.
+     */
+    Conditional loadOdfCondition(const StyleManager* styleManager, const QString &conditionValue, const QString &applyStyleName);
 
     /**
      * \ingroup OpenDocument
      * Loads the condtional formattings.
      */
-    void loadOasisConditions( const StyleManager* styleManager, const KoXmlElement & element );
+    void loadOdfConditions(const StyleManager* styleManager, const KoXmlElement & element);
 
     /**
      * \ingroup OpenDocument
      * Saves the condtional formattings.
      */
-    void saveOasisConditions( KoGenStyle &currentCellStyle ) const;
+    void saveOdfConditions(KoGenStyle &currentCellStyle) const;
 
     /// \note fake implementation to make QMap happy
-    bool operator<( const Conditions& ) const { return true; }
-    void operator=( const Conditions& other );
-    bool operator==( const Conditions& other ) const;
-    inline bool operator!=( const Conditions& other ) const { return !operator==( other ); }
+    bool operator<(const Conditions&) const {
+        return true;
+    }
+    void operator=(const Conditions& other);
+    bool operator==(const Conditions& other) const;
+    inline bool operator!=(const Conditions& other) const {
+        return !operator==(other);
+    }
 
 private:
     /**
@@ -156,27 +171,27 @@ private:
      *
      * \return true if one of the conditions is true, false if not.
      */
-    bool currentCondition( const Cell& cell, Conditional & condition ) const;
+    bool currentCondition(const Cell& cell, Conditional & condition) const;
 
     /**
      * \ingroup OpenDocument
      */
-    void loadOasisCondition( QString &valExpression, Conditional &newCondition );
+    void loadOdfCondition(QString &valExpression, Conditional &newCondition);
 
     /**
      * \ingroup OpenDocument
      */
-    void loadOasisConditionValue( const QString &styleCondition, Conditional &newCondition );
+    void loadOdfConditionValue(const QString &styleCondition, Conditional &newCondition);
 
     /**
      * \ingroup OpenDocument
      */
-    void loadOasisValidationValue( const QStringList &listVal, Conditional &newCondition );
+    void loadOdfValidationValue(const QStringList &listVal, Conditional &newCondition);
 
     /**
      * \ingroup OpenDocument
      */
-    QString saveOasisConditionValue(Conditional &cond) const;
+    QString saveOdfConditionValue(Conditional &cond) const;
 
     class Private;
     QSharedDataPointer<Private> d;
@@ -184,6 +199,7 @@ private:
 
 } // namespace KSpread
 
-Q_DECLARE_TYPEINFO( KSpread::Conditions, Q_MOVABLE_TYPE );
+Q_DECLARE_METATYPE(KSpread::Conditions)
+Q_DECLARE_TYPEINFO(KSpread::Conditions, Q_MOVABLE_TYPE);
 
 #endif // KSPREAD_CONDITION_H
