@@ -60,6 +60,7 @@
 #include "Selection.h"
 #include "Sheet.h"
 #include "StyleManager.h"
+#include "StyleStorage.h"
 #include "Validity.h"
 #include "ValueCalc.h"
 #include "ValueConverter.h"
@@ -234,8 +235,11 @@ bool Map::isReadWrite() const
 bool Map::completeLoading(KoStore *store)
 {
     Q_UNUSED(store);
-    // update all dependencies and recalc all cells
-    addDamage(new WorkbookDamage(this, WorkbookDamage::Formula | WorkbookDamage::Value));
+    // Initial build of all cell dependencies.
+    d->dependencyManager->updateAllDependencies(this);
+    // Recalc the whole workbook now, since there may be formulas other spreadsheets support,
+    // but KSpread does not.
+    d->recalcManager->recalcMap();
     return true;
 }
 
@@ -849,7 +853,8 @@ void Map::increaseLoadedRowsCounter(int number)
 
 bool Map::isLoading() const
 {
-    return d->isLoading;
+    // The KoDocument state is necessary to avoid damages while importing a file (through a filter).
+    return d->isLoading || (d->doc && d->doc->isLoading());
 }
 
 int Map::syntaxVersion() const
@@ -892,7 +897,8 @@ void Map::addDamage(Damage* damage)
 {
     // Do not create a new Damage, if we are in loading process. Check for it before
     // calling this function. This prevents unnecessary memory allocations (new).
-    Q_ASSERT(!isLoading());
+    // see FIXME in Sheet::setSheetName().
+//     Q_ASSERT(!isLoading());
     Q_CHECK_PTR(damage);
 
 #ifndef NDEBUG
@@ -941,7 +947,13 @@ void Map::handleDamages(const QList<Damage*>& damages)
             kDebug(36007) << "Processing\t" << *cellDamage;
             Sheet* const damagedSheet = cellDamage->sheet();
             const Region region = cellDamage->region();
+            const CellDamage::Changes changes = cellDamage->changes();
 
+            // TODO Stefan: Detach the style cache from the CellView cache.
+            if ((changes.testFlag(CellDamage::Appearance))) {
+                // Rebuild the style storage cache.
+                damagedSheet->styleStorage()->invalidateCache(); // FIXME more fine-grained
+            }
             if ((cellDamage->changes() & CellDamage::Binding) &&
                     !workbookChanges.testFlag(WorkbookDamage::Value)) {
                 bindingChangedRegion.add(region, damagedSheet);
