@@ -734,16 +734,6 @@ public:
      */
     void removeRows(int row, int number);
 
-    /**
-     * Updates vertical border and view.
-     */
-    void emitHideRow();
-
-    /**
-     * Updates horizontal border and view.
-     */
-    void emitHideColumn();
-
     //
     //END Methods related column/row operations
     //
@@ -801,45 +791,9 @@ public:
     //
     //////////////////////////////////////////////////////////////////////////
     //
-    //BEGIN Methods related to painting
-    //
-
-    /**
-     * \ingroup Painting
-     * set a region of the spreadsheet to be 'paint dirty' meaning it
-     * needs repainted.  This is not a flag on the cell itself since quite
-     * often this needs set on a default cell
-     */
-    void setRegionPaintDirty(const Region & region);
-
-    /**
-     * \see setRegionPaintDirty(const Region &)
-     */
-    void setRegionPaintDirty(const QRect & rect);
-
-    /**
-     * \ingroup Painting
-     * repaints all visible cells
-     */
-    void updateView();
-
-    /**
-     * \ingroup Painting
-     * repaints all visible cells in \p region
-     */
-    void updateView(const Region& region);
-
-    //
-    //END Methods related to painting
-    //
-    //////////////////////////////////////////////////////////////////////////
-    //
     //BEGIN UNSORTED METHODS !!!
     //
 
-    void emit_updateRow(RowFormat* rowFormat, int _row, bool repaint = true);
-    void emit_updateColumn(ColumnFormat* columnFormat, int _column);
-
     /**
      * Shows a status \p message in the status bar for \p timeout msecs.
      */
@@ -882,19 +836,6 @@ public:
     //////////////////////////////////////////////////////////////////////////
 
 signals:
-    void sig_refreshView();
-    void sig_updateView(Sheet *_sheet);
-    void sig_updateView(Sheet *_sheet, const Region&);
-    void sig_updateColumnHeader(Sheet *_sheet);
-    void sig_updateRowHeader(Sheet *_sheet);
-    /**
-     * @see setSheetName
-     */
-    void sig_nameChanged(Sheet* sheet, const QString& old_name);
-
-    void sig_SheetHidden(Sheet* sheet);
-    void sig_SheetShown(Sheet* sheet);
-
     /**
      * Emitted, if the document size changed.
      * E.g. if some columns were inserted.
@@ -1051,10 +992,6 @@ protected:
                                     int NbCol, const QPoint& point, bool isColumnFixed,
                                     bool isRowFixed);
 
-    static Sheet* find(int _id);
-    static int s_id;
-    static QHash<int, Sheet*>* s_mapSheets;
-
 private:
     /**
      * \ingroup NativeFormat
