@@ -26,6 +26,7 @@
 
 #include "Format.h"
 #include "Number.h"
+#include "kspread_export.h"
 
 namespace KSpread
 {
@@ -33,10 +34,11 @@ class CalculationSettings;
 class Value;
 
 /**
+ * \ingroup Value
  * Generates a Value by parsing an user input text.
  * Determines the most probable Value type, e.g. integer or date.
  */
-class ValueParser
+class KSPREAD_TEST_EXPORT ValueParser
 {
 public:
     /**
@@ -53,35 +55,35 @@ public:
      * Parses the user input text \p str and tries to determine the correct
      * value type for it.
      */
-    Value parse( const QString& str ) const;
+    Value parse(const QString& str) const;
 
     /**
      * Tries for boolean type. If \p str can be interpreted as this
      * type, \p ok is set to \c true and the corresponding value will
      * be returned.
      */
-    Value tryParseBool( const QString& str, bool *ok = 0 ) const;
+    Value tryParseBool(const QString& str, bool *ok = 0) const;
 
     /**
      * Tries for floating point, integer, complex (and percentage) type.
      * If \p str can be interpreted as one of these types, \p ok is set to
      * \c true and the corresponding value will be returned.
      */
-    Value tryParseNumber( const QString& str, bool *ok = 0 ) const;
+    Value tryParseNumber(const QString& str, bool *ok = 0) const;
 
     /**
      * Tries for date type. If \p str can be interpreted as this
      * type, \p ok is set to \c true and the corresponding value will
      * be returned.
      */
-    Value tryParseDate( const QString& str, bool *ok = 0 ) const;
+    Value tryParseDate(const QString& str, bool *ok = 0) const;
 
     /**
      * Tries for time type. If \p str can be interpreted as this
      * type, \p ok is set to \c true and the corresponding value will
      * be returned.
      */
-    Value tryParseTime( const QString& str, bool *ok = 0 ) const;
+    Value tryParseTime(const QString& str, bool *ok = 0) const;
 
 protected:
     /**
@@ -92,18 +94,18 @@ protected:
     /**
      * A helper function to read numbers and distinguish integers and FPs.
      */
-    Value readNumber( const QString &_str, bool* ok ) const;
+    Value readNumber(const QString &_str, bool* ok) const;
 
     /**
      * A helper function to read the imaginary part of a complex number.
      */
-    Number readImaginary( const QString& str, bool* ok ) const;
+    Number readImaginary(const QString& str, bool* ok) const;
 
     /**
      * A helper function to read integers.
      * Used in the parsing process for date and time values.
      */
-    int readInt( const QString& str, uint& pos ) const;
+    int readInt(const QString& str, uint& pos) const;
 
 private:
     const CalculationSettings* m_settings;
