@@ -22,13 +22,12 @@
 
 #include "Cell.h"
 #include "CellStorage.h"
-#include "Doc.h"
 #include "Global.h"
 #include "Localization.h"
 #include "Map.h"
 #include "RowColumnFormat.h"
 #include "Sheet.h"
-#include "SheetPrint.h"
+// #include "SheetPrint.h"
 #include "Style.h"
 #include "StyleManager.h"
 
@@ -658,7 +657,6 @@ void UndoShowColumn::redo()
     sheet->showColumn(0,-1,listCol);
     doc()->setUndoLocked( false );
 }
-#endif
 
 
 /****************************************************************************
@@ -676,9 +674,9 @@ UndoPaperLayout::UndoPaperLayout( Doc *_doc, Sheet *_sheet )
     m_pl = _sheet->print()->paperLayout();
     m_hf = _sheet->print()->headFootLine();
     m_unit = doc()->unit();
-    m_printGrid = _sheet->print()->printGrid();
-    m_printCommentIndicator = _sheet->print()->printCommentIndicator();
-    m_printFormulaIndicator = _sheet->print()->printFormulaIndicator();
+    m_printGrid = _sheet->print()->settings()->printGrid();
+    m_printCommentIndicator = _sheet->print()->settings()->printCommentIndicator();
+    m_printFormulaIndicator = _sheet->print()->settings()->printFormulaIndicator();
     m_printRange = _sheet->print()->printRange();
     m_printRepeatColumns = _sheet->print()->printRepeatColumns();
     m_printRepeatRows = _sheet->print()->printRepeatRows();
@@ -701,9 +699,7 @@ void UndoPaperLayout::undo()
     doc()->setUndoLocked( true );
 
     m_plRedo = print->paperLayout();
-    print->setPaperLayout( m_pl.left,  m_pl.top,
-                           m_pl.right, m_pl.bottom,
-                           m_pl.format,  m_pl.orientation );
+    print->settings()->setPageLayout(m_pl);
 
     m_hfRedo = print->headFootLine();
     print->setHeadFootLine( m_hf.headLeft, m_hf.headMid, m_hf.headRight,
@@ -712,14 +708,14 @@ void UndoPaperLayout::undo()
     m_unitRedo = doc()->unit();
     doc()->setUnit( m_unit );
 
-    m_printGridRedo = print->printGrid();
-    print->setPrintGrid( m_printGrid );
+    m_printGridRedo = print->settings()->printGrid();
+    print->settings()->setPrintGrid( m_printGrid );
 
-    m_printCommentIndicatorRedo = print->printCommentIndicator();
-    print->setPrintCommentIndicator( m_printCommentIndicator );
+    m_printCommentIndicatorRedo = print->settings()->printCommentIndicator();
+    print->settings()->setPrintCommentIndicator( m_printCommentIndicator );
 
-    m_printFormulaIndicatorRedo = print->printFormulaIndicator();
-    print->setPrintFormulaIndicator( m_printFormulaIndicator );
+    m_printFormulaIndicatorRedo = print->settings()->printFormulaIndicator();
+    print->settings()->setPrintFormulaIndicator( m_printFormulaIndicator );
 
     m_printRangeRedo = print->printRange();
     print->setPrintRange( m_printRange );
@@ -757,18 +753,16 @@ void UndoPaperLayout::redo()
     SheetPrint* print = sheet->print();
 
     doc()->setUndoLocked( true );
-    print->setPaperLayout( m_plRedo.left,  m_plRedo.top,
-                           m_plRedo.right, m_plRedo.bottom,
-                           m_plRedo.format, m_plRedo.orientation );
+    print->settings()->setPageLayout(m_plRedo);
 
     print->setHeadFootLine( m_hfRedo.headLeft, m_hfRedo.headMid, m_hfRedo.headRight,
                             m_hfRedo.footLeft, m_hfRedo.footMid, m_hfRedo.footRight );
 
     doc()->setUnit( m_unitRedo );
 
-    print->setPrintGrid( m_printGridRedo );
-    print->setPrintCommentIndicator( m_printCommentIndicatorRedo );
-    print->setPrintFormulaIndicator( m_printFormulaIndicatorRedo );
+    print->settings()->setPrintGrid( m_printGridRedo );
+    print->settings()->setPrintCommentIndicator( m_printCommentIndicatorRedo );
+    print->settings()->setPrintFormulaIndicator( m_printFormulaIndicatorRedo );
 
     print->setPrintRange( m_printRangeRedo );
     print->setPrintRepeatColumns( m_printRepeatColumnsRedo );
@@ -782,7 +776,6 @@ void UndoPaperLayout::redo()
     doc()->setUndoLocked( false );
 }
 
-#if 0
 /****************************************************************************
  *
  * UndoCellFormat
@@ -999,7 +992,6 @@ void UndoCellFormat::undo()
     return;
 
   doc()->setUndoLocked( true );
-  doc()->emitBeginOperation();
   copyFormat( m_lstRedoFormats, m_lstRedoColFormats, m_lstRedoRowFormats, sheet );
   Region::ConstIterator endOfList(m_region.constEnd());
   for (Region::ConstIterator it = m_region.constBegin(); it != endOfList; ++it)
@@ -1046,7 +1038,6 @@ void UndoCellFormat::redo()
     return;
 
   doc()->setUndoLocked( true );
-  doc()->emitBeginOperation();
 
   Region::ConstIterator endOfList(m_region.constEnd());
   for (Region::ConstIterator it = m_region.constBegin(); it != endOfList; ++it)
@@ -1085,8 +1076,7 @@ void UndoCellFormat::redo()
   sheet->updateView( m_region );
   doc()->setUndoLocked( false );
 }
-#endif
-#if 0
+
 /****************************************************************************
  *
  * UndoChangeAngle
@@ -1277,7 +1267,6 @@ void UndoSort::undo()
     return;
 
   doc()->setUndoLocked( true );
-  doc()->emitBeginOperation();
 
   copyAll( m_lstRedoFormats, m_lstRedoColFormats,
            m_lstRedoRowFormats, sheet );
@@ -1331,7 +1320,6 @@ void UndoSort::redo()
 	return;
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     if( Region::Range( m_rctRect ).isColumn() )
     {
@@ -1452,7 +1440,6 @@ void UndoDelete::undo()
     createListCell( m_dataRedo, m_lstRedoColumn, m_lstRedoRow, sheet );
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     {
         QLinkedList<columnSize>::Iterator it2;
@@ -1493,7 +1480,6 @@ void UndoDelete::redo()
 	return;
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     {
         QLinkedList<columnSize>::Iterator it2;
@@ -1531,16 +1517,16 @@ void UndoDelete::redo()
  *
  ***************************************************************************/
 
-UndoDragDrop::UndoDragDrop( Doc * _doc, Sheet * _sheet,
+UndoDragDrop::UndoDragDrop( Sheet * _sheet,
                             const Region& _source,
                             const Region& _target )
-  : UndoAction( _doc ),
+  : UndoAction(),
     m_selectionSource( _source ),
     m_selectionTarget( _target )
 {
-    name = i18n( "Drag & Drop" );
+    setText(i18n("Drag & Drop"));
 
-    m_sheetName = _sheet->sheetName();
+    m_sheet = _sheet;
 
     saveCellRect( m_dataTarget, _sheet, _target );
     saveCellRect( m_dataSource, _sheet, _source );
@@ -1568,25 +1554,18 @@ void UndoDragDrop::saveCellRect( QByteArray & cells, Sheet * sheet,
 
 void UndoDragDrop::undo()
 {
-    Sheet * sheet = doc()->map()->findSheet( m_sheetName );
-    if ( !sheet )
-	return;
+    Sheet * sheet = m_sheet;
 
     saveCellRect( m_dataRedoSource, sheet, m_selectionSource );
     saveCellRect( m_dataRedoTarget, sheet, m_selectionTarget );
 
-    doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
-
     sheet->deleteCells( m_selectionTarget );
-    sheet->paste( m_dataTarget, m_selectionTarget.boundingRect() );
+    sheet->paste( m_dataTarget, m_selectionTarget.boundingRect(), false /* no undo */ );
 
     sheet->deleteCells( m_selectionSource );
-    sheet->paste( m_dataSource, m_selectionSource.boundingRect() );
+    sheet->paste( m_dataSource, m_selectionSource.boundingRect(), false /* no undo */ );
 
     sheet->updateView();
-
-    doc()->setUndoLocked( false );
 }
 
 void UndoDragDrop::redo()
@@ -1598,25 +1577,18 @@ void UndoDragDrop::redo()
         return;
     }
 
-    Sheet * sheet = doc()->map()->findSheet( m_sheetName );
-    if ( !sheet )
-	return;
-
-    doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
+    Sheet * sheet = m_sheet;
 
     //move next line to refreshView
     //because I must know what is the real rect
     //that I must refresh, when there is cell Merged
 
     sheet->deleteCells( m_selectionTarget );
-    sheet->paste( m_dataRedoTarget, m_selectionTarget.boundingRect() );
+    sheet->paste( m_dataRedoTarget, m_selectionTarget.boundingRect(), false /* no undo */ );
     sheet->deleteCells( m_selectionSource );
-    sheet->paste( m_dataRedoSource, m_selectionSource.boundingRect() );
+    sheet->paste( m_dataRedoSource, m_selectionSource.boundingRect(), false /* no undo */ );
 
     sheet->updateView();
-
-    doc()->setUndoLocked( false );
 }
 
 #if 0
@@ -1817,7 +1789,6 @@ void UndoResizeColRow::redo()
 
     doc()->setUndoLocked( false );
 }
-#endif
 
 /****************************************************************************
  *
@@ -1923,7 +1894,6 @@ void UndoChangeAreaTextCell::undo()
 	return;
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     kDebug() <<"creating redo list...";
     createList( m_lstRedoTextCell, sheet );
@@ -1980,7 +1950,6 @@ void UndoChangeAreaTextCell::undo()
     }
 
     //sheet->updateView();
-    doc()->emitEndOperation();
     doc()->setUndoLocked( false );
 }
 
@@ -1999,7 +1968,6 @@ void UndoChangeAreaTextCell::redo()
 	return;
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     Region::ConstIterator endOfList(m_region.constEnd());
     for (Region::ConstIterator it = m_region.constBegin(); it != endOfList; ++it)
@@ -2051,11 +2019,9 @@ void UndoChangeAreaTextCell::redo()
     }
 
     //sheet->updateView();
-    doc()->emitEndOperation();
     doc()->setUndoLocked( false );
 }
 
-#if 0
 /****************************************************************************
  *
  * UndoMergedCell
@@ -2148,12 +2114,10 @@ void UndoAutofill::undo()
     createListCell( m_dataRedo, sheet );
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     sheet->deleteCells( Region(m_selection) );
     sheet->paste( m_data, m_selection );
 
-    doc()->emitEndOperation();
     //sheet->updateView();
 
     doc()->setUndoLocked( false );
@@ -2167,8 +2131,6 @@ void UndoAutofill::redo()
     if ( !sheet )
 	return;
 
-    doc()->emitBeginOperation();
-
     sheet->deleteCells( Region(m_selection) );
     doc()->setUndoLocked( true );
     sheet->paste( m_dataRedo, m_selection );
@@ -2426,17 +2388,17 @@ void UndoConditional::redo()
  *
  ***************************************************************************/
 
-UndoCellPaste::UndoCellPaste(Doc *_doc, Sheet* sheet,
+UndoCellPaste::UndoCellPaste(Sheet* sheet,
                              int _xshift, int _yshift,
                              const Region& region, bool insert, int _insertTo)
-    : UndoAction( _doc )
+    : UndoAction()
 {
     if(!insert)
-        name=i18n("Paste");
+        setText(i18n("Paste"));
     else
-        name=i18n("Paste & Insert");
+        setText(i18n("Paste & Insert"));
 
-    m_sheetName = sheet->sheetName();
+    m_sheet = sheet;
     m_region = region;
     xshift=_xshift;
     yshift=_yshift;
@@ -2517,15 +2479,10 @@ void UndoCellPaste::createListCell(QByteArray& listCell,
 
 void UndoCellPaste::undo()
 {
-  Sheet* sheet = doc()->map()->findSheet( m_sheetName );
-  if ( !sheet )
-      return;
+  Sheet* sheet = m_sheet;
 
   createListCell( m_dataRedo, m_lstRedoColumn, m_lstRedoRow, sheet );
 
-  doc()->setUndoLocked( true );
-  doc()->emitBeginOperation();
-
   uint numCols = 0;
   uint numRows = 0;
 
@@ -2629,11 +2586,10 @@ void UndoCellPaste::undo()
   }
   else // without insertion
   {
-    sheet->paste(m_data, m_region.boundingRect());
+    sheet->paste(m_data, m_region.boundingRect(), false /* no undo */);
   }
 
   sheet->updateView();
-  doc()->setUndoLocked( false );
 }
 
 void UndoCellPaste::redo()
@@ -2645,12 +2601,7 @@ void UndoCellPaste::redo()
         return;
     }
 
-  Sheet* sheet = doc()->map()->findSheet( m_sheetName );
-  if ( !sheet )
-      return;
-
-  doc()->setUndoLocked( true );
-  doc()->emitBeginOperation();
+  Sheet* sheet = m_sheet;
 
   uint numCols = 0;
   uint numRows = 0;
@@ -2745,10 +2696,9 @@ void UndoCellPaste::redo()
     }
   } // for (Region::...
 
-  sheet->paste( m_dataRedo, m_region.boundingRect() );
+  sheet->paste( m_dataRedo, m_region.boundingRect(), false /* no undo */ );
 
   sheet->updateView();
-  doc()->setUndoLocked( false );
 }
 
 #if 0
@@ -2839,7 +2789,6 @@ void UndoStyleCell::undo()
     createListCell( m_lstRedoStyleCell, sheet );
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
 
     QLinkedList<styleCell>::Iterator it2;
@@ -2861,7 +2810,6 @@ void UndoStyleCell::redo()
 	return;
 
     doc()->setUndoLocked( true );
-    doc()->emitBeginOperation();
 
     QLinkedList<styleCell>::Iterator it2;
     for ( it2 = m_lstRedoStyleCell.begin(); it2 != m_lstRedoStyleCell.end(); ++it2 )
@@ -2873,10 +2821,10 @@ void UndoStyleCell::redo()
 
     doc()->setUndoLocked( false );
 }
-#endif
 
 UndoInsertData::UndoInsertData( Doc * _doc, Sheet * _sheet, QRect & _selection )
     : UndoChangeAreaTextCell( _doc, _sheet, Region(_selection) )
 {
     name = i18n("Insert Data From Database");
 }
+#endif
