@@ -30,37 +30,42 @@
 #include <QCloseEvent>
 #include <QLabel>
 #include <QEvent>
-#include <Q3ListBoxItem>
+#include <QListView>
 
 #include <kcompletion.h>
 #include <kdialog.h>
 
-class KComboBox;
 class QLabel;
-class KLineEdit;
-class Q3ListBox;
 class QPushButton;
 class KTabWidget;
-class QTextBrowser;
+class KTextBrowser;
+class QStringListModel;
+class QSortFilterProxyModel;
+class KComboBox;
 class KLineEdit;
 
 namespace KSpread
 {
+class CellEditor;
 class FunctionDescription;
-class View;
+class Selection;
 
+/**
+ * \ingroup UI
+ * Dialog to compose a formula.
+ */
 class FormulaDialog : public KDialog
 {
     Q_OBJECT
 public:
-  FormulaDialog( View* parent, const char* name,const QString& formulaName=0);
-  ~FormulaDialog();
+    FormulaDialog(QWidget* parent, Selection* selection, CellEditor* editor, const QString& expression = 0);
+    ~FormulaDialog();
 private:
     /**
      * Turns the @p text into a parameter that koscript can understand. The type
      * of this parameter is extracted by looking at parameter number @p param in @ref #m_desc.
      */
-    QString createParameter( const QString& _text, int param );
+    QString createParameter(const QString& _text, int param);
     /**
      * Reads the text out of @ref #firstElement and friends and creates a parameter
      * list for the function.
@@ -80,27 +85,32 @@ private slots:
      * Called if a function name was selected but not double clicked.
      * This will just show the help page for the function.
      */
-    void slotSelected( const QString& function );
+    void slotSelected(const QString& function = QString());
+    /**
+     * Called if a function name was selected but not double clicked.
+     * This will just show the help page for the function.
+     */
+    void slotIndexSelected(const QModelIndex& index);
     /**
      * Called if the user clicked on one of the "related function"
      * This will switch the active function and show help page
      * of the function as well.
      */
-    void slotShowFunction( const QString& function );
+    void slotShowFunction(const QString& function);
     /**
      * Called if the user double clicked on some method name.
      * That will switch into editing mode, allowing the user
      * to enter the parameters for the function.
      */
-    void slotDoubleClicked( Q3ListBoxItem* item );
+    void slotDoubleClicked(QModelIndex item = QModelIndex());
     /**
      * Called if a category of methods has been selected.
      */
-    void slotActivated(const QString& category );
+    void slotActivated(const QString& category);
     /**
      * Called if the text of @ref #firstElement, @ref #secondElement etc. changes.
      */
-    void slotChangeText(const QString& text );
+    void slotChangeText(const QString& text);
     /**
      * Connected to @ref View to get notified if the selection in the
      * sheet changes.
@@ -114,7 +124,7 @@ private slots:
     /**
      * Called if the user changes some character in @ref #searchFunct.
      */
-    void slotSearchText(const QString& text );
+    void slotSearchText(const QString& text);
     /**
      * Called if the user pressed return in @ref #searchFunct.
      */
@@ -124,19 +134,22 @@ public:
     /**
      * Find out which widget got focus.
      */
-    bool eventFilter( QObject* obj, QEvent* ev );
+    bool eventFilter(QObject* obj, QEvent* ev);
 protected:
-       virtual void closeEvent ( QCloseEvent * );
+    virtual void closeEvent(QCloseEvent *);
 private:
-    View* m_pView;
+    Selection* m_selection;
+    CellEditor* m_editor;
 
     KTabWidget* m_tabwidget;
-    QTextBrowser* m_browser;
+    KTextBrowser* m_browser;
     QWidget* m_input;
 
     QPushButton *selectFunction;
     KComboBox *typeFunction;
-    Q3ListBox *functions;
+    QListView *functions;
+    QStringListModel *functionsModel;
+    QSortFilterProxyModel *proxyModel;
     KLineEdit *result;
 
     KLineEdit *searchFunct;
