@@ -35,7 +35,6 @@
 #include <KTextEdit>
 
 #include "commands/AutoFillCommand.h"
-#include "part/Factory.h" // FIXME detach from part
 #include "Localization.h"
 
 using namespace KSpread;
@@ -59,8 +58,8 @@ public:
 static const int numBuiltinLists = 4;
 
 ListDialog::ListDialog(QWidget* parent)
-    : KDialog(parent)
-    , d(new Private)
+        : KDialog(parent)
+        , d(new Private)
 {
     setCaption(i18n("Custom Lists"));
     setButtons(Ok | Cancel);
@@ -135,8 +134,9 @@ void ListDialog::slotCurrentRowChanged(int row)
     //we can't remove the first built-in items
     const bool state = row >= numBuiltinLists;
     d->removeButton->setEnabled(state);
-    d->modifyButton->setEnabled(state);
     d->copyButton->setEnabled(row >= 0);
+    d->textEdit->setEnabled(false);
+    d->textEdit->clear();
 }
 
 void ListDialog::init()
@@ -190,7 +190,7 @@ void ListDialog::init()
     sday += i18n("Sun");
     lst.append(sday);
 
-    d->config = Factory::global().config();
+    d->config = KGlobal::activeComponent().config();
     const QStringList other = d->config->group("Parameters").readEntry("Other list", QStringList());
     QString tmp;
     for (QStringList::ConstIterator it = other.begin(); it != other.end();++it) {
@@ -265,8 +265,8 @@ void ListDialog::slotRemove()
         return;
     }
     int ret = KMessageBox::warningContinueCancel(this,
-                                                 i18n("Do you really want to remove this list?"),
-                                                 i18n("Remove List"), KStandardGuiItem::del());
+              i18n("Do you really want to remove this list?"),
+              i18n("Remove List"), KStandardGuiItem::del());
     if (ret == Cancel) { // reponse = No
         return;
     }
