@@ -147,11 +147,6 @@ Canvas::Canvas(View *view)
     d->view = view;
 
     setMouseTracking(true);
-    d->mousePressed = false;
-    d->dragging = false;
-
-    connect(d->view, SIGNAL(autoScroll(const QPoint &)),
-            this, SLOT(slotAutoScroll(const QPoint &)));
 
     installEventFilter(this);   // for TAB key processing, otherwise focus change
     setAcceptDrops(true);
@@ -306,74 +301,42 @@ void Canvas::validateSelection()
         return;
 
     if (selection()->isSingular()) {
-        int col = selection()->marker().x();
-        int row = selection()->marker().y();
-        Cell cell(sheet, col, row);
+        const Cell cell = Cell(sheet, selection()->marker()).masterCell();
         Validity validity = cell.validity();
         if (validity.displayValidationInformation()) {
-            QString title = validity.titleInfo();
+            const QString title = validity.titleInfo();
             QString message = validity.messageInfo();
             if (title.isEmpty() && message.isEmpty())
                 return;
 
-            if (!d->validationInfo)
+            if (!d->validationInfo) {
                 d->validationInfo = new QLabel(this);
-            kDebug(36001) << " display info validation";
-            double u = cell.width();
-            double v = cell.height();
-            double xpos = sheet->columnPosition(selection()->marker().x()) - xOffset();
-            double ypos = sheet->rowPosition(selection()->marker().y()) - yOffset();
-            // Special treatment for obscured cells.
-            if (cell.isPartOfMerged()) {
-                cell = cell.masterCell();
-                int moveX = cell.column();
-                int moveY = cell.row();
-
-                // Use the obscuring cells dimensions
-                u = cell.width();
-                v = cell.height();
-                xpos = sheet->columnPosition(moveX);
-                ypos = sheet->rowPosition(moveY);
+                QPalette palette = d->validationInfo->palette();
+                palette.setBrush(QPalette::Window, palette.toolTipBase());
+                palette.setBrush(QPalette::WindowText, palette.toolTipText());
+                d->validationInfo->setPalette(palette);
+//                 d->validationInfo->setWindowFlags(Qt::ToolTip);
+                d->validationInfo->setFrameShape(QFrame::Box);
+                d->validationInfo->setAlignment(Qt::AlignVCenter);
+                d->validationInfo->setTextFormat(Qt::RichText);
             }
-            //d->validationInfo->setGeometry( 3, y + 3, len + 2, hei + 2 );
-            d->validationInfo->setAlignment(Qt::AlignVCenter);
-            QPainter painter;
-            painter.begin(this);
-            int len = 0;
-            int hei = 0;
-            QString resultText;
+
+            QString resultText("<html><body>");
             if (!title.isEmpty()) {
-                len = painter.fontMetrics().width(title);
-                hei = painter.fontMetrics().height();
-                resultText = title + '\n';
+                resultText += "<h2>" + title + "</h2>";
             }
             if (!message.isEmpty()) {
-                int i = 0;
-                int pos = 0;
-                QString t;
-                do {
-                    i = message.indexOf("\n", pos);
-                    if (i == -1)
-                        t = message.mid(pos, message.length() - pos);
-                    else {
-                        t = message.mid(pos, i - pos);
-                        pos = i + 1;
-                    }
-                    hei += painter.fontMetrics().height();
-                    len = qMax(len, painter.fontMetrics().width(t));
-                } while (i != -1);
-                resultText += message;
+                message.replace(QChar('\n'), QString("<br>"));
+                resultText += "<p>" + message + "</p>";
             }
-            painter.end();
+            resultText += "</body></html>";
             d->validationInfo->setText(resultText);
 
-            QRectF unzoomedMarker(xpos - xOffset() + u,
-                                  ypos - yOffset() + v,
-                                  len,
-                                  hei);
-            QRectF marker(viewConverter()->documentToView(unzoomedMarker));
-
-            d->validationInfo->setGeometry(marker.toRect());
+            const double xpos = sheet->columnPosition(cell.column()) + cell.width();
+            const double ypos = sheet->rowPosition(cell.row()) + cell.height();
+            const QPointF position = QPointF(xpos, ypos) - offset();
+            const QPoint viewPosition = viewConverter()->documentToView(position).toPoint();
+            d->validationInfo->move(/*mapToGlobal*/(viewPosition)); // Qt::ToolTip!
             d->validationInfo->show();
         } else {
             delete d->validationInfo;
@@ -385,31 +348,6 @@ void Canvas::validateSelection()
     }
 }
 
-
-void Canvas::scrollToCell(const QPoint& location) const
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-    if (d->view->isLoading())
-        return;
-
-    // Adjust the maximum accessed column and row for the scrollbars.
-    view()->sheetView(sheet)->updateAccessedCellRange(location);
-
-    // The cell geometry expanded by the size of one column or one row, resp., in each direction.
-    const Cell cell = Cell(sheet, location).masterCell();
-    const double xpos = sheet->columnPosition(cell.cellPosition().x());
-    const double ypos = sheet->rowPosition(cell.cellPosition().y());
-    const double width = sheet->map()->defaultColumnFormat()->width();
-    const double height = sheet->map()->defaultRowFormat()->height();
-    QRectF rect(xpos, ypos, cell.width(), cell.height());
-    rect.adjust(-width - 2, -height - 2, width + 2, height + 2);
-    rect = rect & QRectF(QPointF(0.0, 0.0), sheet->documentSize());
-
-    d->view->canvasController()->ensureVisible(rect, true);
-}
-
 void Canvas::setDocumentOffset(const QPoint& offset)
 {
     const QPoint delta = offset - viewConverter()->documentToView(d->offset).toPoint();
@@ -425,120 +363,30 @@ void Canvas::setDocumentSize(const QSizeF& size)
     emit documentSizeChanged(s);
 }
 
-#if 0
-void Canvas::slotScrollHorz(int _value)
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    kDebug(36005) << "slotScrollHorz: value =" << _value;
-    //kDebug(36005) << kBacktrace();
-
-    if (sheet->layoutDirection() == Qt::RightToLeft)
-        _value = horzScrollBar()->maximum() - _value;
-
-    if (_value < 0) {
-        kDebug(36001)
-        << "Canvas::slotScrollHorz: value out of range (_value: "
-        << _value << ')' << endl;
-        _value = 0;
-    }
-
-    double xpos = sheet->columnPosition(qMin(KS_colMax, sheet->maxColumn() + 10)) - d->xOffset;
-    if (_value > (xpos + d->xOffset))
-        _value = (int)(xpos + d->xOffset);
-
-    // Relative movement
-    // NOTE Stefan: Always scroll by whole pixels, otherwise we'll get offsets.
-    int dx = qRound(viewConverter()->documentToViewX(d->xOffset - _value));
-
-    // New absolute position
-    // NOTE Stefan: Always store whole pixels, otherwise we'll get offsets.
-    d->xOffset -=  viewConverter()->viewToDocumentX(dx);
-    if (d->xOffset < 0.05)
-        d->xOffset = 0.0;
-
-    // scrolling the widgets in the right direction
-    if (sheet->layoutDirection() == Qt::RightToLeft)
-        dx = -dx;
-    scroll(dx, 0);
-    columnHeader()->scroll(dx, 0);
-}
-
-void Canvas::slotScrollVert(int _value)
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    if (_value < 0) {
-        _value = 0;
-        kDebug(36001) << "Canvas::slotScrollVert: value out of range (_value:" <<
-        _value << ')' << endl;
-    }
-
-    double ypos = sheet->rowPosition(qMin(KS_rowMax, sheet->maxRow() + 10));
-    if (_value > ypos)
-        _value = (int) ypos;
-
-    // Relative movement
-    // NOTE Stefan: Always scroll by whole pixels, otherwise we'll get offsets.
-    int dy = qRound(viewConverter()->documentToViewY(d->yOffset - _value));
-    scroll(0, dy);
-    rowHeader()->scroll(0, dy);
-
-    // New absolute position
-    // NOTE Stefan: Always store whole pixels, otherwise we'll get offsets.
-    d->yOffset -= viewConverter()->viewToDocumentY(dy);
-    if (d->yOffset < 0.05)
-        d->yOffset = 0.0;
-}
-
-void Canvas::slotMaxColumn(int _max_column)
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    int oldValue = horzScrollBar()->maximum() - horzScrollBar()->value();
-    double xpos = sheet->columnPosition(qMin(KS_colMax, _max_column + 10)) - xOffset();
-    double unzoomWidth = viewConverter()->viewToDocumentX(width());
-
-    //Don't go beyond the maximum column range (KS_colMax)
-    double sizeMaxX = sheet->documentSize().width();
-    if (xpos > sizeMaxX - xOffset() - unzoomWidth)
-        xpos = sizeMaxX - xOffset() - unzoomWidth;
-
-    horzScrollBar()->setRange(0, (int)(xpos + xOffset()));
-
-    if (sheet->layoutDirection() == Qt::RightToLeft)
-        horzScrollBar()->setValue(horzScrollBar()->maximum() - oldValue);
-}
-
-void Canvas::slotMaxRow(int _max_row)
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    double ypos = sheet->rowPosition(qMin(KS_rowMax, _max_row + 10)) - yOffset();
-    double unzoomHeight = viewConverter()->viewToDocumentY(height());
-
-    //Don't go beyond the maximum row range (KS_rowMax)
-    double sizeMaxY = sheet->documentSize().height();
-    if (ypos > sizeMaxY - yOffset() - unzoomHeight)
-        ypos = sizeMaxY - yOffset() - unzoomHeight;
-
-    vertScrollBar()->setRange(0, (int)(ypos + yOffset()));
-}
-#endif
-
 void Canvas::mousePressEvent(QMouseEvent* event)
 {
+    QMouseEvent *const origEvent = event;
+    QPointF documentPosition;
+    if (layoutDirection() == Qt::LeftToRight) {
+        documentPosition = viewConverter()->viewToDocument(event->pos()) + offset();
+    } else {
+        const QPoint position(width() - event->x(), event->y());
+        const QPointF offset(this->offset().x(), this->offset().y());
+        documentPosition = viewConverter()->viewToDocument(position) + offset;
+kDebug() << "----------------------------";
+kDebug() << "event->pos():" << event->pos();
+kDebug() << "event->globalPos():" << event->globalPos();
+kDebug() << "position:" << position;
+kDebug() << "offset:" << offset;
+kDebug() << "documentPosition:" << documentPosition;
+        event = new QMouseEvent(QEvent::MouseButtonPress, position, mapToGlobal(position), event->button(), event->buttons(), event->modifiers());
+kDebug() << "newEvent->pos():" << event->pos();
+kDebug() << "newEvent->globalPos():" << event->globalPos();
+    }
+
     // flake
     if(d->toolProxy) {
-        d->toolProxy->mousePressEvent(event, viewConverter()->viewToDocument(event->pos()) + offset());
+        d->toolProxy->mousePressEvent(event, documentPosition);
 
         if (!event->isAccepted() && event->button() == Qt::RightButton) {
             d->view->unplugActionList("toolproxy_action_list");
@@ -546,32 +394,77 @@ void Canvas::mousePressEvent(QMouseEvent* event)
             QMenu* menu = dynamic_cast<QMenu*>(d->view->factory()->container("default_canvas_popup", d->view));
             // Only show the menu, if there are items. The plugged action list counts as one action.
             if (menu && menu->actions().count() > 1) {
-                menu->exec(event->globalPos());
+                menu->exec(origEvent->globalPos());
             }
-            event->setAccepted(true);
+            origEvent->setAccepted(true);
         }
     }
+    if (layoutDirection() == Qt::RightToLeft) {
+        delete event;
+    }
 }
 
 void Canvas::mouseReleaseEvent(QMouseEvent* event)
 {
+    QPointF documentPosition;
+    if (layoutDirection() == Qt::LeftToRight) {
+        documentPosition = viewConverter()->viewToDocument(event->pos()) + offset();
+    } else {
+        const QPoint position(width() - event->x(), event->y());
+        const QPointF offset(this->offset().x(), this->offset().y());
+        documentPosition = viewConverter()->viewToDocument(position) + offset;
+        event = new QMouseEvent(QEvent::MouseButtonRelease, position, mapToGlobal(position), event->button(), event->buttons(), event->modifiers());
+    }
+
     // flake
     if(d->toolProxy)
-        d->toolProxy->mouseReleaseEvent(event, viewConverter()->viewToDocument(event->pos()) + offset());
+        d->toolProxy->mouseReleaseEvent(event, documentPosition);
+
+    if (layoutDirection() == Qt::RightToLeft) {
+        delete event;
+    }
 }
 
 void Canvas::mouseMoveEvent(QMouseEvent* event)
 {
+    QPointF documentPosition;
+    if (layoutDirection() == Qt::LeftToRight) {
+        documentPosition = viewConverter()->viewToDocument(event->pos()) + offset();
+    } else {
+        const QPoint position(width() - event->x(), event->y());
+        const QPointF offset(this->offset().x(), this->offset().y());
+        documentPosition = viewConverter()->viewToDocument(position) + offset;
+        event = new QMouseEvent(QEvent::MouseMove, position, mapToGlobal(position), event->button(), event->buttons(), event->modifiers());
+    }
+
     // flake
     if(d->toolProxy)
-        d->toolProxy->mouseMoveEvent(event, viewConverter()->viewToDocument(event->pos()) + offset());
+        d->toolProxy->mouseMoveEvent(event, documentPosition);
+
+    if (layoutDirection() == Qt::RightToLeft) {
+        delete event;
+    }
 }
 
 void Canvas::mouseDoubleClickEvent(QMouseEvent* event)
 {
+    QPointF documentPosition;
+    if (layoutDirection() == Qt::LeftToRight) {
+        documentPosition = viewConverter()->viewToDocument(event->pos()) + offset();
+    } else {
+        const QPoint position(width() - event->x(), event->y());
+        const QPointF offset(this->offset().x(), this->offset().y());
+        documentPosition = viewConverter()->viewToDocument(position) + offset;
+        event = new QMouseEvent(QEvent::MouseButtonDblClick, position, mapToGlobal(position), event->button(), event->buttons(), event->modifiers());
+    }
+
     // flake
     if(d->toolProxy)
-        d->toolProxy->mouseDoubleClickEvent(event, viewConverter()->viewToDocument(event->pos()) + offset());
+        d->toolProxy->mouseDoubleClickEvent(event, documentPosition);
+
+    if (layoutDirection() == Qt::RightToLeft) {
+        delete event;
+    }
 }
 
 void Canvas::keyPressEvent(QKeyEvent* event)
@@ -601,63 +494,9 @@ void Canvas::inputMethodEvent(QInputMethodEvent *event)
         d->toolProxy->inputMethodEvent(event);
 }
 
-bool Canvas::highlightRangeSizeGripAt(double x, double y)
-{
-    if (!selection()->referenceSelectionMode())
-        return false;
-
-    Region::ConstIterator end = selection()->constEnd();
-    for (Region::ConstIterator it = selection()->constBegin(); it != end; ++it) {
-        // TODO Stefan: adapt to Selection::selectionHandleArea
-        QRectF visibleRect = activeSheet()->cellCoordinatesToDocument((*it)->rect());
-
-        QPoint bottomRight((int) visibleRect.right(), (int) visibleRect.bottom());
-        QRect handle(((int) bottomRight.x() - 6),
-                     ((int) bottomRight.y() - 6),
-                     (6),
-                     (6));
-
-        if (handle.contains(QPoint((int) x, (int) y))) {
-            return true;
-        }
-    }
-
-    return false;
-}
-
-void Canvas::startTheDrag()
-{
-    register Sheet * const sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    // right area for start dragging
-    setCursor(Qt::PointingHandCursor);
-
-    QDomDocument doc = CopyCommand::saveAsXml(*selection(), true);
-
-    // Save to buffer
-    QBuffer buffer;
-    buffer.open(QIODevice::WriteOnly);
-    QTextStream str(&buffer);
-    str.setCodec("UTF-8");
-    str << doc;
-    buffer.close();
-
-    QMimeData* mimeData = new QMimeData();
-    mimeData->setText(sheet->copyAsText(selection()));
-    mimeData->setData("application/x-kspread-snippet", buffer.buffer());
-
-    QDrag *drag = new QDrag(this);
-    drag->setMimeData(mimeData);
-    drag->start();
-
-    setCursor(Qt::ArrowCursor);
-}
-
 void Canvas::paintEvent(QPaintEvent* event)
 {
-    if (d->view->doc()->isLoading() || d->view->isLoading())
+    if (d->view->doc()->map()->isLoading() || d->view->isLoading())
         return;
 
     register Sheet * const sheet = activeSheet();
@@ -667,18 +506,21 @@ void Canvas::paintEvent(QPaintEvent* event)
 //     ElapsedTime et("Painting cells", ElapsedTime::PrintOnlyTime);
 
     QPainter painter(this);
-    const QPointF offset = viewConverter()->documentToView(this->offset());
-    painter.translate(-offset);
-    painter.setClipRegion(event->region().translated(offset.x(), offset.y()));
+    painter.setClipRegion(event->region());
     painter.setRenderHints(QPainter::Antialiasing | QPainter::TextAntialiasing);
     painter.save();
 
+    // After the scaling, the painter methods need document coordinates!
     qreal zoomX, zoomY;
     viewConverter()->zoom(&zoomX, &zoomY);
     painter.scale(zoomX, zoomY);
 
+    const bool layoutReversed = sheet->layoutDirection() == Qt::RightToLeft;
+    const QPointF offset(layoutReversed ? -this->offset().x() : this->offset().x(), this->offset().y());
+    painter.translate(-offset);
+
     // erase background
-    const QRectF paintRect = viewConverter()->viewToDocument(rect()).translated(this->offset());
+    const QRectF paintRect(viewConverter()->viewToDocument(rect()).translated(offset));
     painter.fillRect(paintRect, painter.background());
 
     // paint visible cells
@@ -689,7 +531,13 @@ void Canvas::paintEvent(QPaintEvent* event)
 
     // flake
     painter.restore();
+    // d->offset is the negated CanvasController offset in document coordinates.
+//     painter.save();
+    painter.translate(-viewConverter()->documentToView(offset));
     d->shapeManager->paint(painter, *viewConverter(), false);
+//     painter.restore();
+//     const QPointF p = -viewConverter()->documentToView(this->offset());
+//     painter.translate(p.x() /*+ width()*/, p.y());
     painter.setRenderHint(QPainter::Antialiasing, false);
     if(d->toolProxy)
         d->toolProxy->paint(painter, *viewConverter());
@@ -707,8 +555,6 @@ void Canvas::focusInEvent(QFocusEvent*)
 
 void Canvas::focusOutEvent(QFocusEvent*)
 {
-    d->mousePressed = false;
-    d->view->disableAutoScroll();
 }
 
 void Canvas::dragEnterEvent(QDragEnterEvent* event)
@@ -778,7 +624,7 @@ void Canvas::dragMoveEvent(QDragMoveEvent* event)
     // determine the current position
     double eventPosX;
     if (sheet->layoutDirection() == Qt::RightToLeft) {
-        eventPosX = this->width() - viewConverter()->viewToDocumentX(event->pos().x()) + xOffset();
+        eventPosX = viewConverter()->viewToDocumentX(this->width() - event->pos().x()) + xOffset();
     } else {
         eventPosX = viewConverter()->viewToDocumentX(event->pos().x()) + xOffset();
     }
@@ -805,8 +651,6 @@ void Canvas::dragLeaveEvent(QDragLeaveEvent *)
 
 void Canvas::dropEvent(QDropEvent *event)
 {
-    d->dragging = false;
-    d->view->disableAutoScroll();
     register Sheet * const sheet = activeSheet();
     // FIXME Sheet protection: Not all cells have to be protected.
     if (!sheet || sheet->isProtected()) {
@@ -831,7 +675,7 @@ void Canvas::dropEvent(QDropEvent *event)
 
     double ev_PosX;
     if (sheet->layoutDirection() == Qt::RightToLeft) {
-        ev_PosX = this->width() - viewConverter()->viewToDocumentX(event->pos().x()) + xOffset();
+        ev_PosX = viewConverter()->viewToDocumentX(this->width() - event->pos().x()) + xOffset();
     } else {
         ev_PosX = viewConverter()->viewToDocumentX(event->pos().x()) + xOffset();
     }
@@ -867,24 +711,14 @@ void Canvas::dropEvent(QDropEvent *event)
     selection()->initialize(QRect(col, row, columns, rows), sheet);
 }
 
-void Canvas::slotAutoScroll(const QPoint &scrollDistance)
-{
-    // NOTE Stefan: This slot is triggered by the same signal as
-    //              ColumnHeader::slotAutoScroll and RowHeader::slotAutoScroll.
-    //              Therefore, nothing has to be done except the scrolling was
-    //              initiated in the canvas.
-    if (!d->mousePressed)
-        return;
-    d->view->canvasController()->scrollContentsBy(scrollDistance.x(), scrollDistance.y());
-}
-
 QRect Canvas::viewToCellCoordinates(const QRectF& viewRect) const
 {
     register Sheet * const sheet = activeSheet();
     if (!sheet)
         return QRect();
 
-    const QRectF rect = d->view->zoomHandler()->viewToDocument(viewRect).translated(offset());
+    // NOTE Stefan: Do not consider the layout direction in this case.
+    const QRectF rect = viewConverter()->viewToDocument(viewRect.normalized()).translated(offset());
 
     double tmp;
     const int left = sheet->leftColumn(rect.left(), tmp);
