@@ -117,10 +117,6 @@ public:
     static QList<Doc*> s_docs;
     static int s_docId;
 
-    // true if loading is in process, otherwise false.
-    // This flag is used to avoid updates etc. during loading.
-    bool isLoading;
-
     // document properties
     bool configLoadFromFile : 1;
     QStringList spellListIgnoreAll;
@@ -152,8 +148,6 @@ Doc::Doc(QWidget *parentWidget, QObject* parent, bool singleViewMode)
     setComponentData(Factory::global(), false);
     setTemplateType("kspread_template");
 
-    d->isLoading = false;
-
     d->sheetAccessModel = new SheetAccessModel(d->map);
 
     // Init chart shape factory with KSpread's specific configuration panels.
@@ -224,9 +218,6 @@ void Doc::saveConfig()
     if (isEmbedded() || !isReadWrite())
         return;
     KSharedConfigPtr config = Factory::global().config();
-#ifdef KSPREAD_DOC_ZOOM
-    config->group("Parameters").writeEntry("Zoom", zoomInPercent());
-#endif // KSPREAD_DOC_ZOOM
 }
 
 void Doc::initConfig()
@@ -235,22 +226,6 @@ void Doc::initConfig()
 
     const int page = config->group("KSpread Page Layout").readEntry("Default unit page", 0);
     setUnit(KoUnit((KoUnit::Unit) page));
-
-#if 0 // UNDOREDOLIMIT
-    const int undo = config->group("Misc").readEntry("UndoRedo", 30);
-    setUndoRedoLimit(undo);
-#endif
-
-#ifdef KSPREAD_DOC_ZOOM
-    const int zoom = config->group("Parameters").readEntry("Zoom", 100);
-    setZoomAndResolution(zoom, KoGlobal::dpiX(), KoGlobal::dpiY());
-#endif // KSPREAD_DOC_ZOOM
-}
-
-bool Doc::isLoading() const
-{
-    // The KoDocument state is necessary to avoid damages while importing a file (through a filter).
-    return d->isLoading || KoDocument::isLoading();
 }
 
 int Doc::syntaxVersion() const
@@ -450,7 +425,6 @@ bool Doc::loadOdf(KoOdfReadStore & odfStore)
     dt.start();
 
     emit sigProgress(0);
-    d->isLoading = true;
     d->spellListIgnoreAll.clear();
 
     KoXmlElement content = odfStore.contentDoc().documentElement();
@@ -483,7 +457,6 @@ bool Doc::loadOdf(KoOdfReadStore & odfStore)
 
     // all <sheet:sheet> goes to workbook
     if (!map()->loadOdf(body, context)) {
-        d->isLoading = false;
         map()->deleteLoadingInfo();
         return false;
     }
@@ -506,13 +479,11 @@ bool Doc::loadXML(const KoXmlDocument& doc, KoStore*)
     dt.start();
 
     emit sigProgress(0);
-    d->isLoading = true;
     d->spellListIgnoreAll.clear();
     // <spreadsheet>
     KoXmlElement spread = doc.documentElement();
 
     if (spread.attribute("mime") != "application/x-kspread" && spread.attribute("mime") != "application/vnd.kde.kspread") {
-        d->isLoading = false;
         setErrorMessage(i18n("Invalid document. Expected mimetype application/x-kspread or application/vnd.kde.kspread, got %1" , spread.attribute("mime")));
         return false;
     }
@@ -575,7 +546,6 @@ bool Doc::loadXML(const KoXmlDocument& doc, KoStore*)
     if (!styles.isNull()) {
         if (!map()->styleManager()->loadXML(styles)) {
             setErrorMessage(i18n("Styles cannot be loaded."));
-            d->isLoading = false;
             return false;
         }
     }
@@ -584,11 +554,9 @@ bool Doc::loadXML(const KoXmlDocument& doc, KoStore*)
     KoXmlElement mymap = spread.namedItem("map").toElement();
     if (mymap.isNull()) {
         setErrorMessage(i18n("Invalid document. No map tag."));
-        d->isLoading = false;
         return false;
     }
     if (!map()->loadXML(mymap)) {
-        d->isLoading = false;
         return false;
     }
 
@@ -627,8 +595,6 @@ bool Doc::loadXML(const KoXmlDocument& doc, KoStore*)
 
     kDebug(36001) << "Loading took" << (float)(dt.elapsed()) / 1000.0 << " seconds";
 
-    emit sig_refreshView();
-
     return true;
 }
 
@@ -699,15 +665,10 @@ bool Doc::completeLoading(KoStore* store)
 {
     kDebug(36001) << "------------------------ COMPLETING --------------------";
 
-    d->isLoading = false;
-
-    foreach(KoView* view, views())
-    static_cast<View *>(view)->initialPosition();
-
-    kDebug(36001) << "------------------------ COMPLETION DONE --------------------";
-
     setModified(false);
     bool ok = map()->completeLoading(store);
+
+  kDebug(36001) <<"------------------------ COMPLETION DONE --------------------";
     return ok;
 }
 
@@ -732,27 +693,13 @@ QStringList Doc::spellListIgnoreAll() const
     return d->spellListIgnoreAll;
 }
 
-void Doc::newZoomAndResolution(bool updateViews, bool /*forPrint*/)
-{
-    /*    layout();
-        updateAllFrames();*/
-    if (updateViews) {
-        emit sig_refreshView();
-    }
-}
-
 void Doc::paintContent(QPainter& painter, const QRect& rect)
 {
-    paintContent(painter, rect, map()->sheet(0), false);
-}
-
-void Doc::paintContent(QPainter& painter, const QRect& rect, Sheet* sheet, bool drawCursor)
-{
-    Q_UNUSED(drawCursor);
-
     if (rect.isEmpty()) {
         return;
     }
+    Sheet *const sheet = d->map->sheet(0);
+
     const KoPageLayout pageLayout = sheet->printSettings()->pageLayout();
     QPixmap thumbnail(pageLayout.width, pageLayout.height);
     thumbnail.fill(Qt::white);
@@ -774,52 +721,9 @@ void Doc::paintContent(QPainter& painter, const QRect& rect, Sheet* sheet, bool
     painter.drawPixmap(rect & QRect(0, 0, 100, 100), thumbnail);
 }
 
-void Doc::paintUpdates()
+void Doc::updateAllViews()
 {
-    foreach(KoView* view, views()) {
-        static_cast<View *>(view)->paintUpdates();
-    }
-}
-
-void Doc::paintRegion(QPainter &painter, const QRectF &viewRegion,
-                      View* view, const QRect &cellRegion, const Sheet* sheet)
-{
-    // cellRegion has cell coordinates (col,row) while viewRegion has
-    // world coordinates.  cellRegion is the cells to update and
-    // viewRegion is the area actually onscreen.
-
-    if (cellRegion.left() <= 0 || cellRegion.top() <= 0)
-        return;
-
-    const QRectF viewRegionF(viewRegion.left(), viewRegion.right(), viewRegion.width(), viewRegion.height());
-
-    // Get the world coordinates of the upper left corner of the
-    // cellRegion The view is 0, when cellRegion is called from
-    // paintContent, which itself is only called, when we should paint
-    // the output for INACTIVE embedded view.  If inactive embedded,
-    // then there is no view and we alwas start at top/left, so the
-    // offset is 0.
-    //
-    QPointF topLeft;
-    if (view == 0)   //Most propably we are embedded and inactive, so no offset
-        topLeft = QPointF(sheet->columnPosition(cellRegion.left()),
-                          sheet->rowPosition(cellRegion.top()));
-    else
-        topLeft = QPointF(sheet->columnPosition(cellRegion.left()) - view->canvasWidget()->xOffset(),
-                          sheet->rowPosition(cellRegion.top()) - view->canvasWidget()->yOffset());
-
-    SheetView sheetView(sheet);   // FIXME Stefan: make member, otherwise cache lost
-    if (view) {
-        sheetView.setPaintDevice(view->canvasWidget());
-        sheetView.setViewConverter(view->zoomHandler());
-    }
-    sheetView.setPaintCellRange(cellRegion);
-    sheetView.paintCells(view ? view->canvasWidget() : 0, painter, viewRegionF, topLeft);
-}
-
-void Doc::refreshInterface()
-{
-    emit sig_refreshView();
+    emit updateView();
 }
 
 void Doc::updateBorderButton()
@@ -856,37 +760,8 @@ bool Doc::configLoadFromFile() const
     return d->configLoadFromFile;
 }
 
-void Doc::repaint(const QRectF& rect)
-{
-    QRectF r;
-    foreach(KoView* koview, views()) {
-        const View* view = static_cast<View*>(koview);
-        Canvas* canvas = view->canvasWidget();
-
-        r = view->zoomHandler()->documentToView(rect);
-        r.translate(-canvas->xOffset() * view->zoomHandler()->zoomedResolutionX(),
-                    -canvas->yOffset() * view->zoomHandler()->zoomedResolutionY());
-        canvas->update(r.toRect());
-    }
-}
-
 SheetAccessModel *Doc::sheetAccessModel() const
 {
     return d->sheetAccessModel;
 }
-
-#if 0 // UNDOREDOLIMIT
-int Doc::undoRedoLimit() const
-{
-    return d->commandHistory->undoLimit();
-}
-
-void Doc::setUndoRedoLimit(int val)
-{
-    d->commandHistory->setUndoLimit(val);
-    d->commandHistory->setRedoLimit(val);
-}
-#endif
-
 #include "Doc.moc"
-
