@@ -154,6 +154,7 @@
 #include "ui/CellView.h"
 #include "ui/Editors.h"
 #include "ui/MapViewModel.h"
+#include "ui/RightToLeftPaintingStrategy.h"
 #include "ui/SheetView.h"
 
 // D-Bus
@@ -205,16 +206,8 @@ public:
     QMap<Sheet*, QPoint> savedMarkers;
     QMap<Sheet*, QPointF> savedOffsets;
 
-    // the tools
-    struct ToolEntry {
-        QString command;
-        KDataToolInfo info;
-    };
-    QList<ToolEntry*> toolList;
-
     void initActions();
     void adjustActions(bool mode);
-    QAbstractButton* newIconButton(const char *_file, bool _kbutton = false, QWidget *_parent = 0);
 
     // On timeout this will execute the status bar operation (e.g. SUM).
     // This is delayed to speed up the selection.
@@ -285,8 +278,6 @@ void View::Private::initActions()
 
     KActionCollection* ac = view->actionCollection();
 
-    // -- cell operation actions --
-
     // -- sheet/workbook actions --
     actions->sheetProperties  = new KAction(i18n("Sheet Properties"), view);
     ac->addAction("sheetProperties", actions->sheetProperties);
@@ -554,22 +545,6 @@ void View::Private::adjustActions(bool mode)
         selection->update();
 }
 
-QAbstractButton* View::Private::newIconButton(const char *_file, bool _kbutton, QWidget *_parent)
-{
-    if (_parent == 0)
-        _parent = view;
-
-    if (!_kbutton) {
-        QPushButton* pb = new QPushButton(_parent);
-        pb->setIcon(KIcon(_file));
-        return pb;
-    } else {
-        QToolButton* pb = new QToolButton(_parent);
-        pb->setIcon(KIcon(_file));
-        return pb;
-    }
-}
-
 
 /*****************************************************************************
  *
@@ -582,7 +557,6 @@ View::View(QWidget *_parent, Doc *_doc)
         , d(new Private)
 {
     ElapsedTime et("View constructor");
-//     kDebug() <<"sizeof(Cell)=" << sizeof(Cell);
 
     d->view = this;
     d->doc = _doc;
@@ -602,14 +576,20 @@ View::View(QWidget *_parent, Doc *_doc)
 
     d->initActions();
 
-    connect(doc(), SIGNAL(sig_refreshView()), this, SLOT(slotRefreshView()));
-
+    // Connect updateView() signal to View::update() in order to repaint its
+    // child widgets: the column/row headers and the select all button.
+    // Connect to Canvas::update() explicitly as it lives in the viewport
+    // of the KoCanvasController.
+    connect(doc(), SIGNAL(updateView()),
+            this, SLOT(update()));
+    connect(doc(), SIGNAL(updateView()),
+            d->canvas, SLOT(update()));
     connect(doc()->map(), SIGNAL(sheetAdded(Sheet*)),
-            this, SLOT(slotAddSheet(Sheet*)));
+            this, SLOT(addSheet(Sheet*)));
     connect(doc()->map(), SIGNAL(sheetRemoved(Sheet*)),
             this, SLOT(removeSheet(Sheet*)));
     connect(doc()->map(), SIGNAL(sheetRevived(Sheet*)),
-            this, SLOT(reviveSheet(Sheet*)));
+            this, SLOT(addSheet(Sheet*)));
     connect(doc()->map(), SIGNAL(damagesFlushed(const QList<Damage*>&)),
             this, SLOT(handleDamages(const QList<Damage*>&)));
     if (statusBar()) {
@@ -617,20 +597,14 @@ View::View(QWidget *_parent, Doc *_doc)
                 statusBar(), SLOT(showMessage(const QString&, int)));
     }
 
-    if (!doc()->isReadWrite()) {
-        setZoom(100, true);
-    }
-
     connect(&d->statusBarOpTimer, SIGNAL(timeout()), this, SLOT(calcStatusBarOp()));
 
-    // Delay the setting of the initial position, because
-    // we have to wait for the widget to be shown. Otherwise,
-    // we get a wrong widget size.
-    // This is the last operation for the "View loading" process.
-    // The loading flag will be unset at its end.
-    // Don't try to delay this init cause following operations will assume
-    // the sheets are already fully setup.
-    initialPosition();
+    // Delay the setting of the initial position, because we need to have
+    // a sensible widget size, which is not always the case from the beginning
+    // of the View's lifetime.
+    // Therefore, initialPosition(), the last operation in the "View loading"
+    // process, is called from resizeEvent(). The loading flag will be unset
+    // at the end of initialPosition().
 
     new ViewAdaptor(this);
     d->canvas->setFocus();
@@ -638,7 +612,6 @@ View::View(QWidget *_parent, Doc *_doc)
 
 View::~View()
 {
-    //  ElapsedTime el( "~View" );
     if (doc()->isReadWrite())   // make sure we're not embedded in Konq
         selection()->emitCloseEditor(true); // save changes
 
@@ -687,7 +660,6 @@ Doc* View::doc() const
 
 // should be called only once, from the constructor
 /*
- * Top part is the formula bar.
  * Central part is the canvas, row header and vertical scrollbar.
  * Bottom part is the tab bar and horizontal scrollbar.
  *
@@ -705,7 +677,7 @@ void View::initView()
     d->canvas = new Canvas(this);
     d->canvasController = new KoCanvasController(this);
     d->canvasController->setCanvas(d->canvas);
-    d->canvasController->setCanvasMode(KoCanvasController::Infinite);
+    d->canvasController->setCanvasMode(KoCanvasController::Spreadsheet);
     d->canvasController->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
     d->canvasController->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 
@@ -776,7 +748,6 @@ void View::initView()
     d->calcLabel  = 0;
     d->vertScrollBar = new QScrollBar(this);
     d->canvasController->setVerticalScrollBar(d->vertScrollBar);
-//     d->vertScrollBar->setRange( 0, 4096 );
     d->vertScrollBar->setOrientation(Qt::Vertical);
     d->vertScrollBar->setSingleStep(60);  //just random guess based on what feels okay
     d->vertScrollBar->setPageStep(60);  //This should be controlled dynamically, depending on how many rows are shown
@@ -791,7 +762,6 @@ void View::initView()
     d->canvasController->setHorizontalScrollBar(d->horzScrollBar);
     d->tabScrollBarLayout->addWidget(d->horzScrollBar, 0, 1, 2, 1, Qt::AlignVCenter);
 
-//     d->horzScrollBar->setRange( 0, 4096 );
     d->horzScrollBar->setOrientation(Qt::Horizontal);
     d->horzScrollBar->setSingleStep(60); //just random guess based on what feels okay
     d->horzScrollBar->setPageStep(60);
@@ -829,8 +799,6 @@ void View::initView()
     }
 
     // signal slot
-//     connect( d->vertScrollBar, SIGNAL( valueChanged(int) ), d->canvas, SLOT( slotScrollVert(int) ) );
-//     connect( d->horzScrollBar, SIGNAL( valueChanged(int) ), d->canvas, SLOT( slotScrollHorz(int) ) );
     connect(d->canvas, SIGNAL(documentSizeChanged(const QSize&)),
             d->canvasController, SLOT(setDocumentSize(const QSize&)));
     connect(d->canvasController, SIGNAL(moveDocumentOffset(const QPoint&)),
@@ -894,12 +862,7 @@ Selection* View::selection() const
     return d->selection;
 }
 
-const Sheet* View::activeSheet() const
-{
-    return d->activeSheet;
-}
-
-Sheet* View::activeSheet()
+Sheet* View::activeSheet() const
 {
     return d->activeSheet;
 }
@@ -1139,17 +1102,13 @@ void View::finishLoading()
     d->loading = false;
     doc()->map()->deleteLoadingInfo();
 
-    refreshView();
+    setHeaderMinima();
 
     // Activate the cell tool.
     if (shell())
         KoToolManager::instance()->switchToolRequested("KSpreadCellToolId");
 }
 
-void View::activateFormulaEditor()
-{
-}
-
 void View::updateReadWrite(bool readwrite)
 {
     // inform the cell tool
@@ -1170,6 +1129,7 @@ void View::updateReadWrite(bool readwrite)
         d->actions->hideSheet->setEnabled(true);
     }
     d->actions->showPageBorders->setEnabled(true);
+    d->tabBar->setReadOnly(!doc()->isReadWrite() || doc()->map()->isProtected());
 }
 
 void View::createTemplate()
@@ -1195,43 +1155,6 @@ void View::createTemplate()
             "data", "kspread/templates/");
 }
 
-void View::helpUsing()
-{
-    KToolInvocation::invokeHelp();
-}
-
-void View::addSheet(Sheet * _t)
-{
-    reviveSheet(_t);
-
-    // Connect some signals
-    connect(_t, SIGNAL(sig_refreshView()), SLOT(slotRefreshView()));
-    connect(_t, SIGNAL(sig_updateView(Sheet*)), SLOT(slotUpdateView(Sheet*)));
-    connect(_t->print(), SIGNAL(sig_updateView(Sheet*)), SLOT(slotUpdateView(Sheet*)));
-    connect(_t, SIGNAL(sig_updateView(Sheet *, const Region&)),
-            SLOT(slotUpdateView(Sheet*, const Region&)));
-    connect(_t, SIGNAL(sig_updateColumnHeader(Sheet *)),
-            SLOT(slotUpdateColumnHeader(Sheet *)));
-    connect(_t, SIGNAL(sig_updateRowHeader(Sheet *)),
-            SLOT(slotUpdateRowHeader(Sheet *)));
-    connect(_t, SIGNAL(sig_nameChanged(Sheet*, const QString&)),
-            this, SLOT(slotSheetRenamed(Sheet*, const QString&)));
-    connect(_t, SIGNAL(sig_SheetHidden(Sheet*)),
-            this, SLOT(slotSheetHidden(Sheet*)));
-    connect(_t, SIGNAL(sig_SheetShown(Sheet*)),
-            this, SLOT(slotSheetShown(Sheet*)));
-    connect(_t, SIGNAL(shapeAdded(Sheet *, KoShape *)),
-            d->mapViewModel, SLOT(addShape(Sheet *, KoShape *)));
-    connect(_t, SIGNAL(shapeRemoved(Sheet *, KoShape *)),
-            d->mapViewModel, SLOT(removeShape(Sheet *, KoShape *)));
-}
-
-void View::removeAllSheets()
-{
-    d->tabBar->clear();
-    setActiveSheet(0);
-}
-
 void View::setActiveSheet(Sheet* sheet, bool updateSheet)
 {
     if (sheet == d->activeSheet)
@@ -1256,8 +1179,27 @@ void View::setActiveSheet(Sheet* sheet, bool updateSheet)
     // Tell the Canvas about the new visible sheet size.
     sheetView(d->activeSheet)->updateAccessedCellRange();
 
-    if (oldSheet && oldSheet->layoutDirection() != d->activeSheet->layoutDirection())
-        refreshView();
+    // If there was no sheet before or the layout directions differ.
+    if (!oldSheet || oldSheet->layoutDirection() != d->activeSheet->layoutDirection()) {
+        // Propagate the layout direction to the canvas and horz. scrollbar.
+        const Qt::LayoutDirection direction = d->activeSheet->layoutDirection();
+        d->canvas->setLayoutDirection(direction);
+        d->horzScrollBar->setLayoutDirection(direction);
+        // Replace the painting strategy for painting shapes.
+        KoShapeManager *const shapeManager = d->canvas->shapeManager();
+        KoShapeManagerPaintingStrategy *paintingStrategy = 0;
+        if (direction == Qt::LeftToRight) {
+            paintingStrategy = new KoShapeManagerPaintingStrategy(shapeManager);
+        } else {
+            paintingStrategy = new RightToLeftPaintingStrategy(shapeManager, d->canvas);
+        }
+        shapeManager->setPaintingStrategy(paintingStrategy);
+    }
+    // If there was no sheet before or the formula visibilities differ.
+    if (!oldSheet || oldSheet->getShowFormula() != d->activeSheet->getShowFormula()) {
+        const bool showFormulas = d->activeSheet->getShowFormula();
+        stateChanged("show_formulas", showFormulas ? StateNoReverse : StateReverse);
+    }
 
     // Restore the old scrolling offset.
     QMap<Sheet*, QPointF>::Iterator it3 = d->savedOffsets.find(d->activeSheet);
@@ -1320,22 +1262,6 @@ void View::setActiveSheet(Sheet* sheet, bool updateSheet)
     calcStatusBarOp();
 }
 
-void View::slotSheetRenamed(Sheet* sheet, const QString& old_name)
-{
-    d->tabBar->renameTab(old_name, sheet->sheetName());
-}
-
-void View::slotSheetHidden(Sheet*)
-{
-    updateShowSheetMenu();
-}
-
-void View::slotSheetShown(Sheet*)
-{
-    d->tabBar->setTabs(doc()->map()->visibleSheets());
-    updateShowSheetMenu();
-}
-
 void View::changeSheet(const QString& _name)
 {
     if (activeSheet()->sheetName() == _name)
@@ -1371,6 +1297,7 @@ void View::sheetProperties()
     if (d->activeSheet->isProtected()) return;
 
     bool directionChanged = false;
+    bool formulaVisibilityChanged = false;
 
     SheetPropertiesDialog* dlg = new SheetPropertiesDialog(this);
     dlg->setLayoutDirection(d->activeSheet->layoutDirection());
@@ -1390,6 +1317,9 @@ void View::sheetProperties()
 
         if (d->activeSheet->layoutDirection() != dlg->layoutDirection())
             directionChanged = true;
+        if (d->activeSheet->getShowFormula() != dlg->showFormula()) {
+            formulaVisibilityChanged = true;
+        }
 
         command->setLayoutDirection(dlg->layoutDirection());
         command->setAutoCalculationEnabled(dlg->autoCalc());
@@ -1409,9 +1339,24 @@ void View::sheetProperties()
 
     if (directionChanged) {
         // the scrollbar and hborder remain reversed otherwise
-        d->horzScrollBar->setValue(d->horzScrollBar->maximum() -
-                                   d->horzScrollBar->value());
+        d->canvas->setLayoutDirection(d->activeSheet->layoutDirection()); // for scrolling
+        d->horzScrollBar->setLayoutDirection(d->activeSheet->layoutDirection());
         d->columnHeader->update();
+        // Replace the painting strategy for painting shapes.
+        KoShapeManager *const shapeManager = d->canvas->shapeManager();
+        KoShapeManagerPaintingStrategy *paintingStrategy = 0;
+        if (d->activeSheet->layoutDirection() == Qt::LeftToRight) {
+            paintingStrategy = new KoShapeManagerPaintingStrategy(shapeManager);
+        } else {
+            paintingStrategy = new RightToLeftPaintingStrategy(shapeManager, d->canvas);
+        }
+        shapeManager->setPaintingStrategy(paintingStrategy);
+    }
+    if (formulaVisibilityChanged) {
+        const bool showFormulas = d->activeSheet->getShowFormula();
+        stateChanged("show_formulas", showFormulas ? StateNoReverse : StateReverse);
+        sheetView(d->activeSheet)->invalidate();
+        d->canvas->update();
     }
 }
 
@@ -1522,6 +1467,7 @@ void View::toggleProtectDoc(bool mode)
 
     doc()->setModified(true);
     stateChanged("map_is_protected", mode ? StateNoReverse : StateReverse);
+    d->tabBar->setReadOnly(!doc()->isReadWrite() || doc()->map()->isProtected());
 }
 
 void View::toggleProtectSheet(bool mode)
@@ -1544,8 +1490,12 @@ void View::toggleProtectSheet(bool mode)
 
     doc()->setModified(true);
     d->adjustActions(!mode);
-    // d->activeSheet->setRegionPaintDirty( QRect(QPoint( 0, 0 ), QPoint( KS_colMax, KS_rowMax ) ) );
-    refreshView();
+
+    // The sheet protection change may hide/unhide some values or formulas,
+    // so the cached visual data has become invalid.
+    refreshSheetViews();
+    d->canvas->update();
+
     // inform the cell tool
     emit sheetProtectionToggled(mode);
 }
@@ -1560,60 +1510,53 @@ void View::togglePageBorders(bool mode)
 
 void View::viewZoom(KoZoomMode::Mode mode, qreal zoom)
 {
-    Q_UNUSED(zoom);
+    Q_UNUSED(zoom)
 #ifdef NDEBUG
     Q_UNUSED(mode);
 #endif
     Q_ASSERT(mode == KoZoomMode::ZOOM_CONSTANT);
     selection()->emitCloseEditor(true); // save changes
-    doc()->refreshInterface();
-}
-
-void View::setZoom(int zoom, bool /*updateViews*/)
-{
-    kDebug(36005) << "---------SetZoom:" << zoom;
-
-    // Set the zoom in KoView (for embedded views)
-    zoomHandler()->setZoomAndResolution(zoom, KoDpi::dpiX(), KoDpi::dpiY());
-    //KoView::setZoom( zoomHandler()->zoomedResolutionY() /* KoView only supports one zoom */ );
-
-    doc()->refreshInterface();
+    setHeaderMinima();
 }
 
 void View::showColumnHeader(bool enable)
 {
     doc()->map()->settings()->setShowColumnHeader(enable);
-    refreshView();
+    d->columnHeader->setVisible(enable);
+    d->selectAllButton->setVisible(enable && d->rowHeader->isVisible());
 }
 
 void View::showRowHeader(bool enable)
 {
     doc()->map()->settings()->setShowRowHeader(enable);
-    refreshView();
+    d->rowHeader->setVisible(enable);
+    d->selectAllButton->setVisible(enable && d->columnHeader->isVisible());
 }
 
 void View::showHorizontalScrollBar(bool enable)
 {
     doc()->map()->settings()->setShowHorizontalScrollBar(enable);
-    refreshView();
+    d->horzScrollBar->setVisible(enable);
 }
 
 void View::showVerticalScrollBar(bool enable)
 {
     doc()->map()->settings()->setShowVerticalScrollBar(enable);
-    refreshView();
+    d->vertScrollBar->setVisible(enable);
 }
 
-void View::showStatusBar(bool b)
+void View::showStatusBar(bool enable)
 {
-    doc()->map()->settings()->setShowStatusBar(b);
-    refreshView();
+    doc()->map()->settings()->setShowStatusBar(enable);
+    if (statusBar()) {
+        statusBar()->setVisible(enable);
+    }
 }
 
-void View::showTabBar(bool b)
+void View::showTabBar(bool enable)
 {
-    doc()->map()->settings()->setShowTabBar(b);
-    refreshView();
+    doc()->map()->settings()->setShowTabBar(enable);
+    d->tabBar->setVisible(enable);
 }
 
 void View::optionsNotifications()
@@ -1624,24 +1567,7 @@ void View::optionsNotifications()
 void View::preference()
 {
     PreferenceDialog dialog(this);
-    if (dialog.exec()) {
-        d->canvas->update();
-        d->columnHeader->update();
-        d->rowHeader->update();
-    }
-}
-
-bool View::showSheet(const QString& sheetName)
-{
-    Sheet *t = doc()->map()->findSheet(sheetName);
-    if (!t) {
-        kDebug(36001) << "Unknown sheet" << sheetName;
-        return false;
-    }
-    selection()->emitCloseEditor(true); // save changes
-    setActiveSheet(t);
-
-    return true;
+    dialog.exec();
 }
 
 void View::nextSheet()
@@ -1728,59 +1654,15 @@ int View::bottomBorder() const
     return d->horzScrollBar->height();
 }
 
-void View::refreshView()
+void View::setHeaderMinima()
 {
     if (d->loading)   // "View Loading" not finished yet
         return;
-    kDebug(36004) << "refreshing view";
-
-    Sheet * sheet = activeSheet();
-    if (!sheet)
-        return;
-
-    d->adjustActions(!sheet->isProtected());
-
-    bool active = sheet->getShowFormula();
-    Q_UNUSED(active);
-    if (sheet && !sheet->isProtected()) {
-        // FIXME Stefan: action move
-//     d->actions->alignLeft->setEnabled( !active );
-//     d->actions->alignCenter->setEnabled( !active );
-//     d->actions->alignRight->setEnabled( !active );
-    }
-
-    d->tabBar->setReadOnly(!doc()->isReadWrite() || doc()->map()->isProtected());
-
-    d->columnHeader->setVisible(doc()->map()->settings()->showColumnHeader());
-    d->rowHeader->setVisible(doc()->map()->settings()->showRowHeader());
-    d->selectAllButton->setVisible(doc()->map()->settings()->showColumnHeader() && doc()->map()->settings()->showRowHeader());
-    d->vertScrollBar->setVisible(doc()->map()->settings()->showVerticalScrollBar());
-    d->horzScrollBar->setVisible(doc()->map()->settings()->showHorizontalScrollBar());
-    d->tabBar->setVisible(doc()->map()->settings()->showTabBar());
-    if (statusBar()) statusBar()->setVisible(doc()->map()->settings()->showStatusBar());
-
     QFont font(KoGlobal::defaultFont());
     d->columnHeader->setMinimumHeight(qRound(zoomHandler()->zoomItY(font.pointSizeF() + 3)));
     d->rowHeader->setMinimumWidth(qRound(zoomHandler()->zoomItX(YBORDER_WIDTH)));
     d->selectAllButton->setMinimumHeight(qRound(zoomHandler()->zoomItY(font.pointSizeF() + 3)));
     d->selectAllButton->setMinimumWidth(qRound(zoomHandler()->zoomItX(YBORDER_WIDTH)));
-
-    Qt::LayoutDirection sheetDir = sheet->layoutDirection();
-    bool interfaceIsRTL = QApplication::isRightToLeft();
-
-//   kDebug(36004)<<" sheetDir == Qt::LeftToRight :"<<( sheetDir == Qt::LeftToRight );
-    if ((sheetDir == Qt::LeftToRight && !interfaceIsRTL) ||
-            (sheetDir == Qt::RightToLeft && interfaceIsRTL)) {
-        d->viewLayout->setOriginCorner(Qt::TopLeftCorner);
-        d->tabBar->setReverseLayout(interfaceIsRTL);
-    } else {
-        d->viewLayout->setOriginCorner(Qt::TopRightCorner);
-        d->tabBar->setReverseLayout(!interfaceIsRTL);
-    }
-}
-
-void View::resizeEvent(QResizeEvent *)
-{
 }
 
 void View::paperLayoutDlg()
@@ -1803,7 +1685,6 @@ void View::paperLayoutDlg()
 
     PageLayoutDialog dialog(this, d->activeSheet);
     dialog.exec();
-//     dialog.show(); // Use show() to make selecting columns/rows possible.
 }
 
 void View::resetPrintRange()
@@ -1866,8 +1747,6 @@ void View::slotRename()
         QUndoCommand* command = new RenameSheetCommand(sheet, newName);
         doc()->addCommand(command);
 
-        //sheet->setSheetName( newName );
-
         doc()->setModified(true);
     }
 }
@@ -1878,62 +1757,6 @@ void View::slotRename()
 //
 //------------------------------------------------
 
-void View::slotAddSheet(Sheet *_sheet)
-{
-    addSheet(_sheet);
-}
-
-void View::slotRefreshView()
-{
-    refreshView();
-    d->canvas->repaint();
-    d->rowHeader->repaint();
-    d->columnHeader->repaint();
-    d->selectAllButton->repaint();
-}
-
-void View::slotUpdateView(Sheet *_sheet)
-{
-    // Do we display this sheet ?
-    if ((!activeSheet()) || (_sheet != d->activeSheet))
-        return;
-
-    d->activeSheet->setRegionPaintDirty(d->canvas->visibleCells());
-}
-
-void View::slotUpdateView(Sheet * _sheet, const Region& region)
-{
-    // qDebug("void View::slotUpdateView( Sheet *_sheet, const QRect& %i %i|%i %i )\n",_rect.left(),_rect.top(),_rect.right(),_rect.bottom());
-
-    // Do we display this sheet ?
-    if (_sheet != d->activeSheet)
-        return;
-
-    d->activeSheet->setRegionPaintDirty(region);
-}
-
-void View::slotUpdateColumnHeader(Sheet * _sheet)
-{
-    // kDebug(36001)<<"void View::slotUpdateColumnHeader( Sheet *_sheet )";
-
-    // Do we display this sheet ?
-    if (_sheet != d->activeSheet)
-        return;
-
-    d->columnHeader->update();
-}
-
-void View::slotUpdateRowHeader(Sheet *_sheet)
-{
-    // kDebug("void View::slotUpdateRowHeader( Sheet *_sheet )";
-
-    // Do we display this sheet ?
-    if (_sheet != d->activeSheet)
-        return;
-
-    d->rowHeader->update();
-}
-
 void View::slotChangeSelection(const KSpread::Region& changedRegion)
 {
     if (!changedRegion.isValid())
@@ -1949,8 +1772,9 @@ void View::slotChangeSelection(const KSpread::Region& changedRegion)
     d->statusBarOpTimer.setSingleShot(true);
     d->statusBarOpTimer.start(250);
 
-    if (!d->loading)
+    if (!d->loading && !doc()->map()->isLoading()) {
         doc()->map()->addDamage(new SelectionDamage(changedRegion));
+    }
     d->rowHeader->update();
     d->columnHeader->update();
     d->selectAllButton->update();
@@ -1960,8 +1784,6 @@ void View::slotChangeSelection(const KSpread::Region& changedRegion)
     }
 
     d->canvas->validateSelection();
-
-    d->canvas->scrollToCell(selection()->marker());
 }
 
 void View::slotScrollChoice(const KSpread::Region& changedRegion)
@@ -1969,7 +1791,6 @@ void View::slotScrollChoice(const KSpread::Region& changedRegion)
     if (!selection()->referenceSelectionMode() || !changedRegion.isValid()) {
         return;
     }
-    d->canvas->scrollToCell(selection()->marker());
 }
 
 void View::calcStatusBarOp()
@@ -2064,16 +1885,6 @@ void View::menuCalc(bool)
     calcStatusBarOp();
 }
 
-
-QMatrix View::matrix() const
-{
-    QMatrix m;
-    m.scale(zoomHandler()->zoomedResolutionX(),
-            zoomHandler()->zoomedResolutionY());
-    m.translate(- d->canvas->xOffset(), - d->canvas->yOffset());
-    return m;
-}
-
 QWidget* View::canvas() const
 {
     return d->canvas;
@@ -2081,15 +1892,16 @@ QWidget* View::canvas() const
 
 void View::guiActivateEvent(KParts::GUIActivateEvent *ev)
 {
+    // We need a width/height > 0 for setting the initial position properly.
+    // This is not always the case from the beginning of the View's lifetime.
+    if (ev->activated()) {
+        initialPosition();
+    }
+
     if (d->activeSheet) {
         if (ev->activated()) {
             if (d->calcLabel)
                 calcStatusBarOp();
-        } else {
-            /*if (d->calcLabel)
-              {
-              disconnect(d->calcLabel,SIGNAL(pressed( int )),this,SLOT(statusBarClicked(int)));
-              }*/
         }
     }
 
@@ -2144,27 +1956,33 @@ void View::updateBorderButton()
         d->actions->showPageBorders->setChecked(d->activeSheet->isShowPageBorders());
 }
 
-void View::removeSheet(Sheet *_t)
+void View::addSheet(Sheet *sheet)
 {
-    QString m_tablName = _t->sheetName();
-    d->tabBar->removeTab(m_tablName);
-    setActiveSheet(doc()->map()->findSheet(doc()->map()->visibleSheets().first()));
-
-    bool state = doc()->map()->visibleSheets().count() > 1;
+    if (!sheet->isHidden()) {
+        d->tabBar->addTab(sheet->sheetName());
+    }
+    const bool state = (doc()->map()->visibleSheets().count() > 1);
     d->actions->deleteSheet->setEnabled(state);
     d->actions->hideSheet->setEnabled(state);
+
+    // Connect some signals
+    connect(sheet, SIGNAL(shapeAdded(Sheet *, KoShape *)),
+            d->mapViewModel, SLOT(addShape(Sheet *, KoShape *)));
+    connect(sheet, SIGNAL(shapeRemoved(Sheet *, KoShape *)),
+            d->mapViewModel, SLOT(removeShape(Sheet *, KoShape *)));
 }
 
-void View::reviveSheet(Sheet* sheet)
+void View::removeSheet(Sheet *sheet)
 {
-    QString tabName = sheet->sheetName();
-    if (!sheet->isHidden()) {
-        d->tabBar->addTab(tabName);
-    }
+    d->tabBar->removeTab(sheet->sheetName());
+    setActiveSheet(doc()->map()->sheet(0));
 
-    bool state = (doc()->map()->visibleSheets().count() > 1);
+    const bool state = (doc()->map()->visibleSheets().count() > 1);
     d->actions->deleteSheet->setEnabled(state);
     d->actions->hideSheet->setEnabled(state);
+
+    // Disconnect signals.
+    disconnect(sheet, 0, d->mapViewModel, 0);
 }
 
 QColor View::borderColor() const
@@ -2175,32 +1993,13 @@ QColor View::borderColor() const
 void View::updateShowSheetMenu()
 {
     if (d->activeSheet) {
-        if (d->activeSheet->isProtected())
+        if (d->activeSheet->map()->isProtected())
             d->actions->showSheet->setEnabled(false);
         else
             d->actions->showSheet->setEnabled(doc()->map()->hiddenSheets().count() > 0);
     }
 }
 
-void View::markSelectionAsDirty()
-{
-    doc()->map()->addDamage(new SelectionDamage(*selection()));
-}
-
-void View::paintUpdates()
-{
-    /* don't do any begin/end operation here -- this is what is called at an
-       endOperation
-    */
-    d->canvas->update();
-}
-
-void View::initialiseMarkerFromSheet(Sheet* sheet, const QPoint& point)
-{
-    d->savedMarkers.remove(sheet);
-    d->savedMarkers.insert(sheet, point);
-}
-
 QPoint View::markerFromSheet(Sheet* sheet) const
 {
     QMap<Sheet*, QPoint>::Iterator it = d->savedMarkers.find(sheet);
@@ -2233,9 +2032,8 @@ void View::saveCurrentSheetSelection()
 
 void View::handleDamages(const QList<Damage*>& damages)
 {
-    bool refreshView = false;
     QRegion paintRegion;
-    bool paintClipped = true;
+    enum { Nothing, Everything, Clipped } paintMode = Nothing;
 
     QList<Damage*>::ConstIterator end(damages.end());
     for (QList<Damage*>::ConstIterator it = damages.begin(); it != end; ++it) {
@@ -2250,18 +2048,34 @@ void View::handleDamages(const QList<Damage*>& damages)
 
             if (cellDamage->changes() & CellDamage::Appearance) {
                 sheetView(damagedSheet)->invalidateRegion(region);
-                paintClipped = false;
+                paintMode = Everything;
             }
             continue;
         }
 
         if (damage->type() == Damage::Sheet) {
             SheetDamage* sheetDamage = static_cast<SheetDamage*>(damage);
-            kDebug(36007) << "Processing\t" << *sheetDamage;
-
-            if (sheetDamage->changes() & SheetDamage::PropertiesChanged) {
-                d->activeSheet->setRegionPaintDirty(d->canvas->visibleCells());
-                refreshView = true;
+            kDebug(36007) << *sheetDamage;
+            const SheetDamage::Changes changes = sheetDamage->changes();
+            if (changes & (SheetDamage::Name | SheetDamage::Shown)) {
+                d->tabBar->setTabs(doc()->map()->visibleSheets());
+                paintMode = Everything;
+            }
+            if (changes & (SheetDamage::Shown | SheetDamage::Hidden)) {
+                updateShowSheetMenu();
+                paintMode = Everything;
+            }
+            // The following changes only affect the active sheet.
+            if (sheetDamage->sheet() != d->activeSheet) {
+                continue;
+            }
+            if (changes.testFlag(SheetDamage::ContentChanged)) {
+                update();
+                paintMode = Everything;
+            }
+            if (changes.testFlag(SheetDamage::PropertiesChanged)) {
+                sheetView(d->activeSheet)->invalidate();
+                paintMode = Everything;
             }
             if (sheetDamage->changes() & SheetDamage::ColumnsChanged)
                 columnHeader()->update();
@@ -2275,9 +2089,11 @@ void View::handleDamages(const QList<Damage*>& damages)
             kDebug(36007) << "Processing\t" << *selectionDamage;
             const Region region = selectionDamage->region();
 
-            if (paintClipped) {
+            if (paintMode == Clipped) {
                 const QRectF rect = canvasWidget()->cellCoordinatesToView(region.boundingRect());
                 paintRegion += rect.toRect().adjusted(-3, -3, 4, 4);
+            } else {
+                paintMode = Everything;
             }
             continue;
         }
@@ -2286,12 +2102,11 @@ void View::handleDamages(const QList<Damage*>& damages)
     }
 
     // At last repaint the dirty cells.
-    if (paintClipped)
+    if (paintMode == Clipped) {
         canvas()->update(paintRegion);
-    else
+    } else if (paintMode == Everything) {
         canvas()->update();
-    if (refreshView)
-        this->refreshView();
+    }
 }
 
 KoPrintJob * View::createPrintJob()
