@@ -45,7 +45,7 @@
 
 #include "AutoFillStrategy.h"
 #include "Cell.h"
-#include "Limits.h"
+#include "kspread_limits.h"
 #include "Map.h"
 #include "MergeStrategy.h"
 #include "Selection.h"
@@ -172,6 +172,11 @@ QSizeF TableTool::size() const
     return d->tableShape->size();
 }
 
+QPointF TableTool::canvasOffset() const
+{
+    return QPointF();
+}
+
 int TableTool::maxCol() const
 {
     return d->tableShape->columns();
