@@ -75,6 +75,11 @@ public:
     bool createEditor(bool clear = true, bool focus = true);
     CellEditor* editor() const;
 
+    /**
+     * Scrolls to the cell located at \p location.
+     */
+    void scrollToCell(const QPoint &location);
+
 public Q_SLOTS:
     virtual void activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes);
     virtual void deactivate();
@@ -86,12 +91,20 @@ protected:
     virtual QWidget* createOptionWidget();
     void applyUserInput(bool expandMatrix = false);
     virtual KoInteractionStrategy* createStrategy(KoPointerEvent* event);
+
+    /**
+     * The shape offset in document coordinates.
+     */
     virtual QPointF offset() const = 0;
     virtual QSizeF size() const = 0;
+
+    /**
+     * The canvas scrolling offset in document coordinates.
+     */
+    virtual QPointF canvasOffset() const = 0;
     virtual int maxCol() const = 0;
     virtual int maxRow() const = 0;
     virtual SheetView* sheetView(const Sheet* sheet) const = 0;
-    bool eventFilter(QObject *obj, QEvent *event);
 
 protected Q_SLOTS:
     void selectionChanged(const Region&);
