@@ -56,7 +56,9 @@
 #include "commands/DataManipulators.h"
 #include "commands/StyleCommand.h"
 
-// dialogs
+// ui
+#include "ui/CellToolOptionWidget.h"
+#include "ui/SheetView.h"
 
 // KOffice
 #include <KoCanvasBase.h>
@@ -82,49 +84,14 @@ void CellToolBase::Private::updateEditor(const Cell& cell)
     const Cell& theCell = cell.isPartOfMerged() ? cell.masterCell() : cell;
     const Style style = theCell.style();
     if (q->selection()->activeSheet()->isProtected() && style.hideFormula()) {
-        userInput->setPlainText(theCell.displayText());
+        optionWidget->editor()->setPlainText(theCell.displayText());
     } else if (q->selection()->activeSheet()->isProtected() && style.hideAll()) {
-        userInput->clear();
+        optionWidget->editor()->clear();
     } else {
-        userInput->setPlainText(theCell.userInput());
+        optionWidget->editor()->setPlainText(theCell.userInput());
     }
 }
 
-void CellToolBase::Private::updateLocationComboBox()
-{
-    QString address;
-    const QList< QPair<QRectF, QString> > names = q->selection()->activeSheet()->cellStorage()->namedAreas(*q->selection());
-    {
-        QRect range;
-        if (q->selection()->isSingular()) range = QRect(q->selection()->marker(), QSize(1, 1));
-        else range = q->selection()->lastRange();
-        for (int i = 0; i < names.size(); i++) {
-            if (names[i].first.toRect() == range) {
-                address = names[i].second;
-            }
-        }
-    }
-    if (address.isEmpty()) {
-        if (q->selection()->activeSheet()->getLcMode()) {
-            if (q->selection()->isSingular()) {
-                address = 'L' + QString::number(q->selection()->marker().y()) +
-                        'C' + QString::number(q->selection()->marker().x());
-            } else {
-                const QRect lastRange = q->selection()->lastRange();
-                address = QString::number(lastRange.bottom() - lastRange.top() + 1) + "Lx";
-                if (Region::Range(lastRange).isRow())
-                    address += QString::number(q->maxCol() - lastRange.left() + 1) + 'C';
-                else
-                    address += QString::number(lastRange.right() - lastRange.left() + 1) + 'C';
-            }
-        } else {
-            address = q->selection()->name();
-        }
-    }
-    if (address != locationComboBox->lineEdit()->text())
-        locationComboBox->lineEdit()->setText(address);
-}
-
 #define ACTION_EXEC( name, command ) { \
         QAction *a = q->action(name); \
         const bool blocked = a->blockSignals(true); \
@@ -160,6 +127,11 @@ void CellToolBase::Private::updateActions(const Cell& cell)
     ACTION_EXEC("percent", setChecked(ft == Format::Percentage));
     ACTION_EXEC("currency", setChecked(ft == Format::Money));
 
+    const bool showFormulas = q->selection()->activeSheet()->getShowFormula();
+    q->action("alignLeft")->setEnabled(showFormulas);
+    q->action("alignCenter")->setEnabled(showFormulas);
+    q->action("alignRight")->setEnabled(showFormulas);
+
     if (!q->selection()->activeSheet()->isProtected() || style.notProtected()) {
         q->action("clearComment")->setEnabled(!cell.comment().isEmpty());
         q->action("decreaseIndentation")->setEnabled(style.indentation() > 0.0);
@@ -208,8 +180,8 @@ void CellToolBase::Private::setProtectedActionsEnabled(bool enable)
     const QList<KAction*> actions = q->actions().values();
     for (int i = 0; i < actions.count(); ++i)
         actions[i]->setEnabled(enable);
-    formulaButton->setEnabled(enable);
-    userInput->setEnabled(enable);
+    optionWidget->formulaButton()->setEnabled(enable);
+    optionWidget->editor()->setEnabled(enable);
 
     // These actions are always enabled.
     q->action("copy")->setEnabled(true);
@@ -370,6 +342,7 @@ bool CellToolBase::Private::processHomeKey(QKeyEvent* event)
         } else {
             q->selection()->initialize(destination, sheet);
         }
+        q->scrollToCell(destination);
         event->accept(); // QKeyEvent
     }
     return true;
@@ -409,6 +382,7 @@ bool CellToolBase::Private::processEndKey(QKeyEvent *event)
         } else {
             q->selection()->initialize(destination, sheet);
         }
+        q->scrollToCell(destination);
         event->accept(); // QKeyEvent
     }
     return true;
@@ -430,6 +404,7 @@ bool CellToolBase::Private::processPriorKey(QKeyEvent *event)
     } else {
         q->selection()->initialize(destination, q->selection()->activeSheet());
     }
+    q->scrollToCell(destination);
     event->accept(); // QKeyEvent
     return true;
 }
@@ -451,6 +426,7 @@ bool CellToolBase::Private::processNextKey(QKeyEvent *event)
     } else {
         q->selection()->initialize(destination, q->selection()->activeSheet());
     }
+    q->scrollToCell(destination);
     event->accept(); // QKeyEvent
     return true;
 }
@@ -488,11 +464,11 @@ void CellToolBase::Private::processDeleteKey(QKeyEvent* event)
 
 void CellToolBase::Private::processF2Key(QKeyEvent*  event)
 {
-    userInput->setFocus();
+    optionWidget->editor()->setFocus();
     if (q->editor()) {
-        QTextCursor textCursor = userInput->textCursor();
+        QTextCursor textCursor = optionWidget->editor()->textCursor();
         textCursor.setPosition(q->editor()->cursorPosition());
-        userInput->setTextCursor(textCursor);
+        optionWidget->editor()->setTextCursor(textCursor);
     }
     event->accept(); // QKeyEvent
 }
@@ -503,9 +479,9 @@ void CellToolBase::Private::processF4Key(QKeyEvent* event)
     */
     if (q->editor()) {
         q->editor()->handleKeyPressEvent(event);
-        QTextCursor textCursor = userInput->textCursor();
+        QTextCursor textCursor = optionWidget->editor()->textCursor();
         textCursor.setPosition(q->editor()->cursorPosition());
-        userInput->setTextCursor(textCursor);
+        optionWidget->editor()->setTextCursor(textCursor);
     }
 }
 
@@ -795,6 +771,7 @@ bool CellToolBase::Private::processControlArrowKey(QKeyEvent *event)
     } else {
         q->selection()->initialize(destination, sheet);
     }
+    q->scrollToCell(destination);
     return true;
 }
 
@@ -956,6 +933,7 @@ QRect CellToolBase::Private::moveDirection(KSpread::MoveTo direction, bool exten
     } else {
         q->selection()->initialize(destination, sheet);
     }
+    q->scrollToCell(destination);
     updateEditor(Cell(q->selection()->activeSheet(), q->selection()->cursor()));
 
     return QRect(cursor, destination);
@@ -966,6 +944,7 @@ void CellToolBase::Private::paintSelection(QPainter &painter, const QRectF &view
     if (q->selection()->referenceSelection() || q->editor()) {
         return;
     }
+    Sheet *const sheet = q->selection()->activeSheet();
 
     // save the painter state
     painter.save();
@@ -996,11 +975,26 @@ void CellToolBase::Private::paintSelection(QPainter &painter, const QRectF &view
         double top =    positions[1];
         double right =  positions[2];
         double bottom = positions[3];
+        if (sheet->layoutDirection() == Qt::RightToLeft) {
+            // The painter's origin is translated by the negative canvas offset.
+            // viewRect.left() is the canvas offset. Add it once to the
+            // coordinates. Then, the upper left corner of the canvas has to
+            // match the correct document position, which is the scrolling
+            // offset (viewRect.left()) plus the width of the visible area
+            // (viewRect.width()); that's the right border (left+width).
+            const qreal offset = /*2 * viewRect.left() +*/ viewRect.width();
+            left = offset - positions[2];
+            right = offset - positions[0];
+        }
 
         bool paintLeft =   paintSides[0];
         bool paintTop =    paintSides[1];
         bool paintRight =  paintSides[2];
         bool paintBottom = paintSides[3];
+        if (sheet->layoutDirection() == Qt::RightToLeft) {
+            paintLeft  = paintSides[2];
+            paintRight = paintSides[0];
+        }
 
         const double unzoomedPixelX = q->canvas()->viewConverter()->viewToDocumentX(1.0);
         const double unzoomedPixelY = q->canvas()->viewConverter()->viewToDocumentY(1.0);
@@ -1010,10 +1004,20 @@ void CellToolBase::Private::paintSelection(QPainter &painter, const QRectF &view
         if (current) {
             // save old clip region
             const QRegion clipRegion = painter.clipRegion();
-            // clip out the marker region
-            const QRect effMarker = selection->extendToMergedAreas(QRect(selection->marker(), selection->marker()));
-            const QRectF markerRect = selection->activeSheet()->cellCoordinatesToDocument(effMarker);
-            painter.setClipRegion(clipRegion.subtracted(markerRect.toRect()));
+            // clip out the cursor region
+            const QRect cursor = QRect(selection->cursor(), selection->cursor());
+            const QRect extCursor = selection->extendToMergedAreas(cursor);
+            QRectF cursorRect = sheet->cellCoordinatesToDocument(extCursor);
+            if (sheet->layoutDirection() == Qt::RightToLeft) {
+                // See comment above.
+                const qreal offset = /*2 * viewRect.left() +*/ viewRect.width();
+                const qreal left = offset - cursorRect.right();
+                const qreal right = offset - cursorRect.left();
+                cursorRect.setLeft(left);
+                cursorRect.setRight(right);
+            }
+            cursorRect.adjust(unzoomedPixelX, unzoomedPixelY, unzoomedPixelX, unzoomedPixelY);
+            painter.setClipRegion(clipRegion.subtracted(cursorRect.toRect()));
             // draw the transparent selection background
             painter.fillRect(QRectF(left, top, right - left, bottom - top), selectionColor);
             // restore clip region
@@ -1088,43 +1092,59 @@ void CellToolBase::Private::paintReferenceSelection(QPainter &painter, const QRe
     // save painter state
     painter.save();
 
+    // Define the reference selection handle.
+    const qreal pixelX = q->canvas()->viewConverter()->viewToDocumentX(1);
+    const qreal pixelY = q->canvas()->viewConverter()->viewToDocumentY(1);
+    const QRectF handleArea(-3 * pixelX, -3 * pixelY, 6 * pixelX, 6 * pixelY);
+
+    // The colors for the referenced ranges and the color index.
     const QList<QColor> colors = q->selection()->colors();
-    const double unzoomedPixelX = q->canvas()->viewConverter()->viewToDocumentX(1.0);
-    const double unzoomedPixelY = q->canvas()->viewConverter()->viewToDocumentY(1.0);
     int index = 0;
-    Region::ConstIterator end(q->selection()->constEnd());
-    for (Region::ConstIterator it = q->selection()->constBegin(); it != end; ++it) {
+
+    // Iterate over the referenced ranges.
+    const Region::ConstIterator end(q->selection()->constEnd());
+    for (Region::ConstIterator it(q->selection()->constBegin()); it != end; ++it) {
+        Sheet *const sheet = (*it)->sheet();
         // Only paint ranges or cells on the current sheet
-        if ((*it)->sheet() != q->selection()->activeSheet()) {
+        if (sheet != q->selection()->activeSheet()) {
             index++;
             continue;
         }
 
         const QRect range = q->selection()->extendToMergedAreas((*it)->rect());
-
-        QRectF unzoomedRect = q->selection()->activeSheet()->cellCoordinatesToDocument(range);
+        QRectF area = sheet->cellCoordinatesToDocument(range);
 
         // Convert region from sheet coordinates to canvas coordinates for use with the painter
         // retrieveMarkerInfo(region,viewRect,positions,paintSides);
 
         // Now adjust the highlight rectangle is slightly inside the cell borders (this means
         // that multiple highlighted cells look nicer together as the borders do not clash)
-        unzoomedRect.adjust(unzoomedPixelX, unzoomedPixelY, -unzoomedPixelX, -unzoomedPixelY);
+        area.adjust(pixelX, pixelY, -pixelX, -pixelY);
+
+        // The current color.
+        const QColor color = colors[index++ % colors.size()];
+
+        // Paint the reference range's outline.
+        if ((*it)->sheet()->layoutDirection() == Qt::RightToLeft) {
+            // See comment in paintSelection().
+            const qreal offset = /*2 * viewRect.left() +*/ viewRect.width();
+            const qreal left = offset - area.right();
+            const qreal right = offset - area.left();
+            area.setLeft(left);
+            area.setRight(right);
+        }
 
         painter.setBrush(QBrush());
-        painter.setPen(colors[(index) % colors.size()]);
-        painter.drawRect(unzoomedRect);
+        painter.setPen(color);
+        painter.drawRect(area);
 
         // Now draw the size grip (the little rectangle on the bottom right-hand corner of
         // the range which the user can click and drag to resize the region)
         painter.setPen(Qt::white);
-        painter.setBrush(colors[(index) % colors.size()]);
-
-        painter.drawRect(QRectF(unzoomedRect.right() - 3 * unzoomedPixelX,
-                                unzoomedRect.bottom() - 3 * unzoomedPixelY,
-                                6 * unzoomedPixelX,
-                                6 * unzoomedPixelY));
-        index++;
+        painter.setBrush(color);
+        const bool rtl = sheet->layoutDirection() == Qt::RightToLeft;
+        const QPointF corner(rtl ? area.bottomLeft() : area.bottomRight());
+        painter.drawRect(handleArea.translated(corner));
     }
 
     // restore painter state
@@ -1134,6 +1154,9 @@ void CellToolBase::Private::paintReferenceSelection(QPainter &painter, const QRe
 void CellToolBase::Private::retrieveMarkerInfo(const QRect &cellRange, const QRectF &viewRect,
         double positions[], bool paintSides[])
 {
+    // Everything is in document coordinates here.
+    // The layout direction, which is view dependent, is applied afterwards.
+
     const Sheet* sheet = q->selection()->activeSheet();
     const QRectF visibleRect = sheet->cellCoordinatesToDocument(cellRange);
 
@@ -1142,11 +1165,6 @@ void CellToolBase::Private::retrieveMarkerInfo(const QRect &cellRange, const QRe
     qreal top = visibleRect.top();
     qreal right = visibleRect.right();
     qreal bottom = visibleRect.bottom();
-    if (sheet->layoutDirection() == Qt::RightToLeft) {
-        const double docWidth = q->size().width();
-        left = docWidth - visibleRect.right();
-        right = docWidth - visibleRect.left();
-    }
 
     /* left, top, right, bottom */
     paintSides[0] = (viewRect.left() <= left) && (left <= viewRect.right()) &&
@@ -1361,16 +1379,3 @@ bool CellToolBase::Private::testListChoose(Selection *selection) const
     }
     return false;
 }
-
-void CellToolBase::Private::relayoutDocker(bool wide)
-{
-    // user input is moved accordingly to the "wide" param, the rest stays in one place
-    if (userInput->hasFocus()) return;  // do nothing while the user input has focus
-    widgetLayout->removeWidget(userInput);
-    if (wide)
-        widgetLayout->addWidget(userInput, 0, 3);
-    else
-        widgetLayout->addWidget(userInput, 1, 0, 1, 5);
-    hasWideLayout = wide;
-}
-
