@@ -20,7 +20,7 @@
 #include "DragAndDropStrategy.h"
 
 #include "Cell.h"
-#include "Limits.h"
+#include "kspread_limits.h"
 #include "Selection.h"
 #include "Sheet.h"
 
