@@ -43,7 +43,7 @@ public:
     /**
      * Constructor.
      */
-    SelectionStrategy(KoTool* parent, KoCanvasBase* canvas, Selection* selection,
+    SelectionStrategy(KoToolBase *parent, Selection *selection,
                       const QPointF position, Qt::KeyboardModifiers modifiers);
 
     /**
@@ -51,6 +51,8 @@ public:
      */
     virtual ~SelectionStrategy();
 
+    virtual void handleMouseMove(const QPointF &position, Qt::KeyboardModifiers modifiers);
+
 private:
     class Private;
     Private * const d;
