@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006, 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006, 2007, 2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -27,7 +27,7 @@
 // koffice
 #include <KoShapeRegistry.h>
 #include <KoInlineNote.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 #include <KoShapeContainer.h>
 #include <KoStyleManager.h>
 #include <KoParagraphStyle.h>
@@ -41,6 +41,7 @@
 #include <KoColorBackground.h>
 #include <KoImageCollection.h>
 #include <KoImageData.h>
+#include <KoBorder.h>
 
 // KDE + Qt includes
 #include <QTextBlock>
@@ -54,7 +55,7 @@ KWDLoader::KWDLoader(KWDocument *parent, KoStore *store)
         m_pageStyle(m_pageManager->defaultPageStyle()),
         m_foundMainFS(false)
 {
-    connect(this, SIGNAL(sigProgress(int)), m_document, SIGNAL(sigProgress(int)));
+    connect(this, SIGNAL(progressUpdate(int)), m_document, SIGNAL(sigProgress(int)));
 }
 
 KWDLoader::~KWDLoader()
@@ -65,7 +66,7 @@ bool KWDLoader::load(KoXmlElement &root)
 {
     QTime dt;
     dt.start();
-    emit sigProgress(0);
+    emit progressUpdate(0);
     kDebug(32001) << "KWDocument::loadXML";
 
     QString mime = root.attribute("mime");
@@ -80,9 +81,9 @@ bool KWDLoader::load(KoXmlElement &root)
     }
     //KWLoadingInfo *loadingInfo = new KWLoadingInfo();
 
-    emit sigProgress(5);
+    emit progressUpdate(5);
 
-    KoPageLayout pgLayout = KoPageLayout::standardLayout();
+    KoPageLayout pgLayout;
     // <PAPER>
     KoXmlElement paper = root.namedItem("PAPER").toElement();
     if (!paper.isNull()) {
@@ -166,7 +167,7 @@ bool KWDLoader::load(KoXmlElement &root)
                 pos = KWord::FootNoteSeparatorCenter;
             else if (tmp == "right")
                 pos = KWord::FootNoteSeparatorRight;
-            else // default: if ( tmp =="left" )
+            else // default: if (tmp =="left")
                 pos = KWord::FootNoteSeparatorLeft;
             m_pageStyle.setFootNoteSeparatorLinePosition(pos);
         }
@@ -182,24 +183,24 @@ bool KWDLoader::load(KoXmlElement &root)
         // <PAPERBORDERS>
         KoXmlElement paperborders = paper.namedItem("PAPERBORDERS").toElement();
         if (!paperborders.isNull()) {
-            pgLayout.left = paperborders.attribute("left").toDouble();
-            pgLayout.top = paperborders.attribute("top").toDouble();
-            pgLayout.right = paperborders.attribute("right").toDouble();
-            pgLayout.bottom = paperborders.attribute("bottom").toDouble();
+            pgLayout.leftMargin   = paperborders.attribute("left").toDouble();
+            pgLayout.topMargin    = paperborders.attribute("top").toDouble();
+            pgLayout.rightMargin  = paperborders.attribute("right").toDouble();
+            pgLayout.bottomMargin = paperborders.attribute("bottom").toDouble();
 
             // Support the undocumented syntax actually used by KDE 2.0 for some of the above (:-().
-            if (pgLayout.left == 0.0)
-                pgLayout.left = paperborders.attribute("left").toDouble();
-            if (pgLayout.top == 0.0)
-                pgLayout.top = paperborders.attribute("top").toDouble();
-            if (pgLayout.right == 0.0)
-                pgLayout.right = paperborders.attribute("right").toDouble();
-            if (pgLayout.bottom == 0.0)
-                pgLayout.bottom = paperborders.attribute("bottom").toDouble();
+            if (pgLayout.leftMargin == 0.0)
+                pgLayout.leftMargin = paperborders.attribute("left").toDouble();
+            if (pgLayout.topMargin == 0.0)
+                pgLayout.topMargin = paperborders.attribute("top").toDouble();
+            if (pgLayout.rightMargin == 0.0)
+                pgLayout.rightMargin = paperborders.attribute("right").toDouble();
+            if (pgLayout.bottomMargin == 0.0)
+                pgLayout.bottomMargin = paperborders.attribute("bottom").toDouble();
         } else
-            kWarning() << "No <PAPERBORDERS> tag!";
+            kWarning(32001) << "No <PAPERBORDERS> tag!";
     } else
-        kWarning() << "No <PAPER> tag! This is a mandatory tag! Expect weird page sizes...";
+        kWarning(32001) << "No <PAPER> tag! This is a mandatory tag! Expect weird page sizes...";
 
     m_pageManager->pageStyle("Standard").setPageLayout(pgLayout);
 
@@ -207,11 +208,11 @@ bool KWDLoader::load(KoXmlElement &root)
     KoXmlElement attributes = root.namedItem("ATTRIBUTES").toElement();
     if (!attributes.isNull()) {
         if (attributes.attribute("processing", "0") == "1") {
-            m_pageStyle.setMainTextFrame(false); // DTP type document.
+            m_pageStyle.setHasMainTextFrame(false); // DTP type document.
             m_foundMainFS = true; // we will not reuse the main FS now.
         }
 
-        //KWDocument::getAttribute( attributes, "standardpage", QString::null );
+        //KWDocument::getAttribute(attributes, "standardpage", QString::null);
         if (attributes.attribute("hasHeader") != "1") {
             m_pageStyle.setHeaderPolicy(KWord::HFTypeNone);
             if (m_firstPageStyle.isValid())
@@ -229,16 +230,16 @@ bool KWDLoader::load(KoXmlElement &root)
         //    m_document->m_tabStop = attributes.attribute("tabStopValue").toDouble();
         /* TODO
                 m_initialEditing = new InitialEditing();
-                m_initialEditing->m_initialFrameSet = attributes.attribute( "activeFrameset" );
-                m_initialEditing->m_initialCursorParag = attributes.attribute( "cursorParagraph" ).toInt();
-                m_initialEditing->m_initialCursorIndex = attributes.attribute( "cursorIndex" ).toInt();
+                m_initialEditing->m_initialFrameSet = attributes.attribute("activeFrameset");
+                m_initialEditing->m_initialCursorParag = attributes.attribute("cursorParagraph").toInt();
+                m_initialEditing->m_initialCursorIndex = attributes.attribute("cursorIndex").toInt();
         */
     }
     if (m_firstPageStyle.isValid()
             && m_firstPageStyle.footerPolicy() != KWord::HFTypeNone
             && m_firstPageStyle.headerPolicy() != KWord::HFTypeNone) {
         m_firstPageStyle.setColumns(m_pageStyle.columns());
-        m_firstPageStyle.setMainTextFrame(m_pageStyle.hasMainTextFrame());
+        m_firstPageStyle.setHasMainTextFrame(m_pageStyle.hasMainTextFrame());
         m_firstPageStyle.setHeaderDistance(m_pageStyle.headerDistance());
         m_firstPageStyle.setFooterDistance(m_pageStyle.footerDistance());
         m_firstPageStyle.setEndNoteDistance(m_pageStyle.endNoteDistance());
@@ -252,7 +253,7 @@ bool KWDLoader::load(KoXmlElement &root)
     if (!isReadWrite())
         variableCollection()->variableSetting()->setDisplayFieldCode(false);
 
-    emit sigProgress(10);
+    emit progressUpdate(10);
 
     KoXmlElement mailmerge = root.namedItem("MAILMERGE").toElement();
     if (mailmerge != KoXmlElement()) {
@@ -260,14 +261,14 @@ bool KWDLoader::load(KoXmlElement &root)
     }
 #endif
 
-    emit sigProgress(15);
+    emit progressUpdate(15);
 
     // Load all styles before the corresponding paragraphs try to use them!
     KoXmlElement stylesElem = root.namedItem("STYLES").toElement();
     if (!stylesElem.isNull())
         loadStyleTemplates(stylesElem);
 
-    emit sigProgress(17);
+    emit progressUpdate(17);
 #if 0
 
     KoXmlElement frameStylesElem = root.namedItem("FRAMESTYLES").toElement();
@@ -276,7 +277,7 @@ bool KWDLoader::load(KoXmlElement &root)
     else // load default styles
         loadDefaultFrameStyleTemplates();
 
-    emit sigProgress(18);
+    emit progressUpdate(18);
 
     KoXmlElement tableStylesElem = root.namedItem("TABLESTYLES").toElement();
     if (!tableStylesElem.isNull())
@@ -284,11 +285,11 @@ bool KWDLoader::load(KoXmlElement &root)
     else // load default styles
         loadDefaultTableStyleTemplates();
 
-    emit sigProgress(19);
+    emit progressUpdate(19);
 
     loadDefaultTableTemplates();
 
-    emit sigProgress(20);
+    emit progressUpdate(20);
 
     KoXmlElement bookmark = root.namedItem("BOOKMARKS").toElement();
     if (!bookmark.isNull()) {
@@ -339,14 +340,14 @@ bool KWDLoader::load(KoXmlElement &root)
     }
 
 
-    emit sigProgress(25);
+    emit progressUpdate(25);
 
 
     KoXmlElement framesets = root.namedItem("FRAMESETS").toElement();
     if (!framesets.isNull())
         loadFrameSets(framesets);
 
-    emit sigProgress(85);
+    emit progressUpdate(85);
 
     insertAnchors();
     insertNotes();
@@ -355,7 +356,7 @@ bool KWDLoader::load(KoXmlElement &root)
 
     loadPictureMap(root);
 
-    emit sigProgress(90);
+    emit progressUpdate(90);
 
     // <EMBEDDED>
     loadEmbeddedObjects(root);
@@ -365,7 +366,7 @@ bool KWDLoader::load(KoXmlElement &root)
         m_pageManager->appendPage(m_pageStyle);
     }
 
-    emit sigProgress(100); // the rest is only processing, not loading
+    emit progressUpdate(100); // the rest is only processing, not loading
 
     kDebug(32001) << "Loading took" << (float)(dt.elapsed()) / 1000 << " seconds";
 
@@ -393,7 +394,7 @@ void KWDLoader::loadFrameSets(const KoXmlElement &framesets)
     }
 }
 
-void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames, bool loadFootnote)
+void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem)
 {
     QString fsname = framesetElem.attribute("name");
 
@@ -401,26 +402,6 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
     case 1: { // FT_TEXT
         QString tableName = framesetElem.attribute("grpMgr");
         if (!tableName.isEmpty()) {   // Text frameset belongs to a table -> find table by name
-            /*
-                        KWTableFrameSet *table = 0;
-                        Q3PtrListIterator<KWFrameSet> fit = framesetsIterator();
-                        for ( ; fit.current() ; ++fit ) {
-                            KWFrameSet *f = fit.current();
-                            if( f->type() == FT_TABLE &&
-                                f->isVisible() &&
-                                f->name() == tableName ) {
-                                table = static_cast<KWTableFrameSet *> (f);
-                                break;
-                            }
-                        }
-                        // No such table yet -> create
-                        if ( !table ) {
-                            table = new KWTableFrameSet( this, tableName );
-                            addFrameSet(table, false);
-                        }
-                        // Load the cell
-                        return table->loadCell( framesetElem );
-             */
             return; // TODO support backwards compatible tables
         } else {
             KWord::TextFrameSetType type;
@@ -498,7 +479,7 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
         ImageKey imageKey;
         fill(&imageKey, key);
         if (imageKey.filename.isEmpty()) {
-            kWarning() << "could not find image in the store\n";
+            kWarning(32001) << "could not find image in the store";
             return;
         }
 
@@ -506,18 +487,18 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
         fill(fs, framesetElem);
         fs->setName(fsname);
 
-        KoShapeFactory *factory = KoShapeRegistry::instance()->value("PictureShape");
+        KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value("PictureShape");
         Q_ASSERT(factory);
-        KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
+        KoShape *shape = factory->createDefaultShape(m_document->resourceManager());
         shape->setKeepAspectRatio(image.attribute("keepAspectRatio", "true") == "true");
 
-        KoImageCollection * collection = dynamic_cast< KoImageCollection *>( m_document->dataCenterMap()["ImageCollection"] );
-        Q_ASSERT( collection );
-        KoImageData * data = collection->getImage( imageKey.filename, m_store );
-        shape->setUserData( data );
+        KoImageCollection *collection = m_document->resourceManager()->imageCollection();
+        Q_ASSERT(collection);
+        KoImageData *data = collection->createImageData(imageKey.filename, m_store);
+        shape->setUserData(data);
 
-        KWFrame * f = new KWFrame( shape, fs );
-        fill( f, frame );
+        KWFrame *f = new KWFrame(shape, fs);
+        fill(f, frame);
         m_document->addFrameSet(fs);
         return;
     }
@@ -550,8 +531,18 @@ void KWDLoader::loadFrameSet(const KoXmlElement &framesetElem, bool loadFrames,
 
 void KWDLoader::fill(KWFrameSet *fs, const KoXmlElement &framesetElem)
 {
-    //m_visible = static_cast<bool>( KWDocument::getAttribute( framesetElem, "visible", true ) ); // TODO
-    //m_protectSize=static_cast<bool>( KWDocument::getAttribute( framesetElem, "protectSize", false ) ); TODO
+    if (framesetElem.hasAttribute("visible")) {
+        bool visible = framesetElem.attribute("visible").toLower() == "true";
+        foreach (KWFrame *frame, fs->frames()) {
+            frame->shape()->setVisible(visible);
+        }
+    }
+    if (framesetElem.hasAttribute("protectSize")) {
+        bool protectSize = framesetElem.attribute("protectSize").toLower() == "true";
+        foreach (KWFrame *frame, fs->frames()) {
+            frame->shape()->setGeometryProtected(protectSize);
+        }
+    }
 
 }
 
@@ -562,9 +553,9 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
     KoXmlElement frameElem;
     forEachElement(frameElem, framesetElem) {
         if (frameElem.tagName() == "FRAME") {
-            KoShapeFactory *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+            KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
             Q_ASSERT(factory);
-            KoShape *shape = factory->createDefaultShapeAndInit(m_document->dataCenterMap());
+            KoShape *shape = factory->createDefaultShape(m_document->resourceManager());
             KWTextFrame *frame = new KWTextFrame(shape, fs);
             fill(frame, frameElem);
 
@@ -573,12 +564,12 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
     }
 
 
-    //m_info = static_cast<KWFrameSet::Info>( KWDocument::getAttribute( framesetElem, "frameInfo", KWFrameSet::FI_BODY ) ); // TODO
+    //m_info = static_cast<KWFrameSet::Info>(KWDocument::getAttribute(framesetElem, "frameInfo", KWFrameSet::FI_BODY)); // TODO
     switch (framesetElem.attribute("frameInfo").toInt()) {
     case 0: ;
     }
-    if (framesetElem.hasAttribute("protectContent"))
-        fs->setProtectContent((bool)framesetElem.attribute("protectContent").toInt());
+    if (framesetElem.hasAttribute("protectContent") && fs->frameCount())
+        fs->frames().first()->shape()->setContentProtected((bool)framesetElem.attribute("protectContent").toInt());
 
     QTextCursor cursor(fs->document());
     cursor.select(QTextCursor::Document);
@@ -595,7 +586,7 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                 cursor.insertBlock(emptyTbf, emptyCf);
             }
 
-            KoStyleManager * styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
+            KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
             Q_ASSERT(styleManager);
 
             firstParag = false;
@@ -669,11 +660,11 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                             formatCursor.setPosition(block.position() + pos);
                             formatCursor.setPosition(block.position() + pos + length, QTextCursor::KeepAnchor);
                         } else {
-                            kWarning("Format has missing or invalid 'len' value, ignoring\n");
+                            kWarning(32001) << "Format has missing or invalid 'len' value, ignoring";
                             continue;
                         }
                     } else {
-                        kWarning("Format has missing or invalid 'pos' value, ignoring\n");
+                        kWarning(32001) << "Format has missing or invalid 'pos' value, ignoring";
                         continue;
                     }
                     if (id == "1") {
@@ -682,27 +673,27 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                         fill(&s2, format);
                         s2.applyStyle(&formatCursor);
                     } else if (id == "2") {
-                        kWarning("File to old, image can not be recovered\n");
+                        kWarning(32001) << "File to old, image can not be recovered";
                     } else if (id == "4") {
                         KoXmlElement variable = format.namedItem("VARIABLE").toElement();
                         if (variable.isNull()) {
-                            kWarning() << "Missing VARIABLE tag\n";
+                            kWarning(32001) << "Missing VARIABLE tag";
                             continue;
                         }
                         KoXmlElement type = variable.namedItem("TYPE").toElement();
                         int typeId = type.attribute("type", "1").toInt();
-                        switch(typeId) {
+                        switch (typeId) {
                         case 11: { // footnote
                             KoXmlElement footnote = variable.namedItem("FOOTNOTE").toElement();
                             KoInlineNote *note = new KoInlineNote(KoInlineNote::Footnote);
                             note->setLabel(footnote.attribute("value"));
-                            note->setAutoNumbering( footnote.attribute("numberingtype", "auto") == "auto" );
-                            note->setText("Unable to locate footnote text");
-                            KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(
+                            note->setAutoNumbering(footnote.attribute("numberingtype", "auto") == "auto");
+                            note->setText(i18n("Unable to locate footnote text"));
+                            KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(
                                     fs->document()->documentLayout());
                             Q_ASSERT(layout);
-                            Q_ASSERT(layout->inlineObjectTextManager());
-                            layout->inlineObjectTextManager()->insertInlineObject(formatCursor, note);
+                            Q_ASSERT(layout->inlineTextObjectManager());
+                            layout->inlineTextObjectManager()->insertInlineObject(formatCursor, note);
                             NotesData notesData;
                             notesData.note = note;
                             notesData.frameSetName = footnote.attribute("frameset");
@@ -716,13 +707,13 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                                         ? KoInlineNote::Footnote : KoInlineNote::Endnote;
                             KoInlineNote *note = new KoInlineNote(type);
                             note->setLabel(footEndNote.attribute("value"));
-                            note->setAutoNumbering( footEndNote.attribute("numberingtype", "auto") == "auto" );
-                            note->setText("Unable to locate note-text");
-                            KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(
+                            note->setAutoNumbering(footEndNote.attribute("numberingtype", "auto") == "auto");
+                            note->setText(i18n("Unable to locate note-text"));
+                            KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(
                                     fs->document()->documentLayout());
                             Q_ASSERT(layout);
-                            Q_ASSERT(layout->inlineObjectTextManager());
-                            layout->inlineObjectTextManager()->insertInlineObject(formatCursor, note);
+                            Q_ASSERT(layout->inlineTextObjectManager());
+                            layout->inlineTextObjectManager()->insertInlineObject(formatCursor, note);
                             NotesData notesData;
                             notesData.note = note;
                             notesData.frameSetName = footEndNote.attribute("frameset");
@@ -741,17 +732,17 @@ void KWDLoader::fill(KWTextFrameSet *fs, const KoXmlElement &framesetElem)
                         }
                         }
                     } else if (id == "5") {
-                        kWarning("File to old, footnote can not be recovered\n");
+                        kWarning(32001) << "File to old, footnote can not be recovered";
                     } else if (id == "6") { // anchor for floating frame.
                         KoXmlElement anchor = format.namedItem("ANCHOR").toElement();
                         if (anchor.isNull()) {
-                            kWarning() << "Missing ANCHOR tag\n";
+                            kWarning(32001) << "Missing ANCHOR tag";
                             continue;
                         }
                         QString type = anchor.attribute("type", "frameset");
                         if (type == "frameset") {
                             if (! anchor.hasAttribute("instance")) {
-                                kWarning() << "ANCHOR is missing its instance attribute\n";
+                                kWarning(32001) << "ANCHOR is missing its instance attribute";
                                 continue;
                             }
                             AnchorData data;
@@ -859,7 +850,7 @@ void KWDLoader::fill(KoParagraphStyle *style, const KoXmlElement &layout)
         case 10: llp.setStyle(KoListStyle::DiscItem); break;
         case 11: llp.setStyle(KoListStyle::BoxItem); break;
         case 7: llp.setStyle(KoListStyle::CustomCharItem);
-            kWarning() << "According to spec COUNTER with type 7 is not supported, ignoring\n";
+            kWarning(32001) << "According to spec COUNTER with type 7 is not supported, ignoring";
             // fall through
         default: {
             lstyle = 0;
@@ -891,20 +882,20 @@ void KWDLoader::fill(KoParagraphStyle *style, const KoXmlElement &layout)
                     innerWidth = 0.0;
             spacing = 0.0;
             switch (element.attribute("style").toInt()) {
-            case 0: borderStyle = KoParagraphStyle::BorderSolid; break;
-            case 1: borderStyle = KoParagraphStyle::BorderDashed; break;
-            case 2: borderStyle = KoParagraphStyle::BorderDotted; break;
-            case 3: borderStyle = KoParagraphStyle::BorderDashDotPattern; break;
-            case 4: borderStyle = KoParagraphStyle::BorderDashDotDotPattern; break;
+            case 0: borderStyle = KoBorder::BorderSolid; break;
+            case 1: borderStyle = KoBorder::BorderDashed; break;
+            case 2: borderStyle = KoBorder::BorderDotted; break;
+            case 3: borderStyle = KoBorder::BorderDashDotPattern; break;
+            case 4: borderStyle = KoBorder::BorderDashDotDotPattern; break;
             case 5:
-                borderStyle = KoParagraphStyle::BorderDouble;
+                borderStyle = KoBorder::BorderDouble;
                 spacing = width;
                 innerWidth = width;
                 break;
             }
         }
         qreal width, innerWidth, spacing;
-        KoParagraphStyle::BorderStyle borderStyle;
+        KoBorder::BorderStyle borderStyle;
     };
     element = layout.namedItem("LEFTBORDER").toElement();
     if (!element.isNull()) {
@@ -1090,7 +1081,7 @@ void KWDLoader::fill(KWFrame *frame, const KoXmlElement &frameElem)
     if (tf) {
         if (zIndex <= 0 && static_cast<KWTextFrameSet*>(tf->frameSet())->textFrameSetType() == KWord::OtherTextFrameSet)
             zIndex = 1; // OtherTextFrameSet types always live on top of the main frames.
-        KoTextShapeData *textShapeData = dynamic_cast<KoTextShapeData*>(frame->shape()->userData());
+        KoTextShapeData *textShapeData = qobject_cast<KoTextShapeData*>(frame->shape()->userData());
         Q_ASSERT(textShapeData);
         KoInsets margins;
         margins.left = frameElem.attribute("bleftpt", "0.0").toDouble();
@@ -1133,7 +1124,8 @@ void KWDLoader::fill(ImageKey *key, const KoXmlElement &keyElement)
 
 void KWDLoader::loadStyleTemplates(const KoXmlElement &stylesElem)
 {
-    KoStyleManager *styleManager = dynamic_cast<KoStyleManager *>(m_document->dataCenterMap()["StyleManager"]);
+    KoStyleManager *styleManager = m_document->resourceManager()->resource(KoText::StyleManager).value<KoStyleManager*>();
+
     Q_ASSERT(styleManager);
 
     KoXmlElement style;
@@ -1180,24 +1172,24 @@ void KWDLoader::insertAnchors()
     foreach (const AnchorData &anchor, m_anchors) {
         KWFrameSet *fs = m_document->frameSetByName(anchor.frameSetName);
         if (fs == 0) {
-            kWarning() << "Anchored frameset not found: '" << anchor.frameSetName;
+            kWarning(32001) << "Anchored frameset not found: '" << anchor.frameSetName;
             continue;
         }
         if (fs->frames().count() == 0)  continue;
         KWFrame *frame = fs->frames().first();
         frame->shape()->setPosition(QPointF(0, 0));
-        KoShapeContainer* container = dynamic_cast<KoShapeContainer*>(anchor.textShape);
+        KoShapeContainer *container = dynamic_cast<KoShapeContainer*>(anchor.textShape);
         Q_ASSERT(container);
         if (! container) continue;
-        container->addChild(frame->shape());   // attach here & avoid extra layouts
+        container->addShape(frame->shape());   // attach here & avoid extra layouts
         KoTextAnchor *textAnchor = new KoTextAnchor(frame->shape());
         QTextCursor cursor(anchor.document);
         cursor.setPosition(anchor.cursorPosition);
         cursor.setPosition(anchor.cursorPosition + 1, QTextCursor::KeepAnchor);
-        KoTextDocumentLayout *layout = dynamic_cast<KoTextDocumentLayout*>(cursor.block().document()->documentLayout());
+        KoTextDocumentLayout *layout = qobject_cast<KoTextDocumentLayout*>(cursor.block().document()->documentLayout());
         Q_ASSERT(layout);
-        Q_ASSERT(layout->inlineObjectTextManager());
-        layout->inlineObjectTextManager()->insertInlineObject(cursor, textAnchor);
+        Q_ASSERT(layout->inlineTextObjectManager());
+        layout->inlineTextObjectManager()->insertInlineObject(cursor, textAnchor);
     }
     m_anchors.clear();
 }
@@ -1207,13 +1199,13 @@ void KWDLoader::insertNotes()
     foreach (const NotesData &note, m_notes) {
         KWFrameSet *fs = m_document->frameSetByName(note.frameSetName);
         if (fs == 0) {
-            kWarning() << "Frameset data for note not found: '" << note.frameSetName;
+            kWarning(32001) << "Frameset data for note not found: '" << note.frameSetName;
             continue;
         }
-        KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*> (fs);
+        KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         if (tfs && tfs->document()) {
             note.note->setText(tfs->document()->toPlainText());
-kDebug() << "setting the text to" << note.note->text();
+//kDebug(32001) << "setting the text to" << note.note->text();
         }
         m_document->removeFrameSet(fs);
         delete fs; // we don't want to keep it around.
