@@ -149,8 +149,13 @@ void KWGui::pageSetupChanged()
 
 void KWGui::updateMousePos(const QPoint &point)
 {
-    m_horizontalRuler->updateMouseCoordinate(point.x());
-    m_verticalRuler->updateMouseCoordinate(point.y());
+    QPoint canvasOffset(m_canvasController->canvasOffsetX(), m_canvasController->canvasOffsetY() );
+    // the offset is positive it the canvas is shown fully visible
+    canvasOffset.setX(canvasOffset.x() < 0 ? canvasOffset.x(): 0);
+    canvasOffset.setY(canvasOffset.y() < 0 ? canvasOffset.y(): 0);
+    QPoint viewPos = point - canvasOffset;
+    m_horizontalRuler->updateMouseCoordinate(viewPos.x());
+    m_verticalRuler->updateMouseCoordinate(viewPos.y());
 }
 
 void KWGui::updateRulers() const
