@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2010 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Pierre Ducroquet <pinaraf@pinaraf.info>
  *
  * This library is free software; you can redistribute it and/or
@@ -28,10 +28,14 @@
 #include "KWDocument.h"
 
 #include <KoShapeRegistry.h>
-#include <KoShapeFactory.h>
+#include <KoShapeFactoryBase.h>
 
+#include <QTextLayout>
+#include <QTextDocument>
+#include <QTextBlock>
 #include <klocale.h>
 #include <kdebug.h>
+#include <limits.h>
 
 KWFrameLayout::KWFrameLayout(const KWPageManager *pageManager, const QList<KWFrameSet*> &frameSets)
         : m_pageManager(pageManager),
@@ -91,7 +95,7 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
                 m_parent = parent;
             }
             void create(const KWPage &page, KWTextFrameSet *fs) {
-                KWFrame* frame;
+                KWFrame *frame;
                 if (fs->textFrameSetType() == KWord::MainTextFrameSet)
                     frame = new KWTextFrame(m_parent->createTextShape(page), fs);
                 else
@@ -179,7 +183,9 @@ void KWFrameLayout::createNewFramesForPage(int pageNumber)
                 f = new KWTextFrame(createTextShape(page), tfs);
             } else {
                 Q_ASSERT(frame->newFrameBehavior() == KWord::CopyNewFrame);
-                f = new KWFrame(new KWCopyShape(frame->shape()), frame->frameSet());
+                Q_ASSERT(frame->frameSet());
+                Q_ASSERT(frame->frameSet()->frameCount() > 0);
+                f = new KWFrame(new KWCopyShape(frame->frameSet()->frames().first()->shape(), m_pageManager), frame->frameSet());
             }
             const qreal y = frame->shape()->position().y();
             qreal offsetFromPage = y - pageRect.top();
@@ -206,78 +212,94 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     Q_ASSERT(page.isValid());
 
     /* +-----------------+
-       |  0              | <- m_pageStyle->pageLayout()->top
+       |  0              | <- m_pageStyle->pageLayout()->topMargin + d:o->topPadding
        |  1  [ header ]  |
        |  2              | <- m_pageStyle->headerDistance()
        |  3  [ maintxt ] |
        |  4              | <- m_pageStyle->endNoteDistance()
        |  5  [ endnote ] |
-       |  6              | <- m_pageStyle->footnoteDistance()
-       |  7 [ footnote ] |
-       |  8              | <- m_pageStyle->footerDistance()
-       |  9  [ footer ]  |
-       | 10               | <- m_pageStyle->pageLayout()->bottom
+       |  6              | <- m_pageStyle->footerDistance()
+       |  7  [ footer ]  |
+       |  8              | <- m_pageStyle->pageLayout()->bottomMargin + d:o->bottomPadding
        +-----------------+ */
 
     // Create some data structures used for the layouting of the frames later
-    qreal requestedHeight[11], minimumHeight[11], resultingPositions[11];
-    for (int i = 0; i < 11; i++) { // zero fill.
+    int minZIndex = INT_MAX;
+    qreal requestedHeight[9], minimumHeight[9], resultingPositions[9];
+    for (int i = 0; i < 9; i++) { // zero fill.
         requestedHeight[i] = 0;
         minimumHeight[i] = 0;
         resultingPositions[i] = 0;
     }
-    minimumHeight[0] = page.topMargin();
-    minimumHeight[10] = page.bottomMargin();
+    minimumHeight[0] = page.topMargin() + page.topPadding();
+    minimumHeight[8] = page.bottomMargin() + page.bottomPadding();
 
     KoPageLayout layout = page.pageStyle().pageLayout();
-    layout.left = page.leftMargin();
-    layout.right = page.rightMargin();
+    layout.leftMargin = page.leftMargin();
+    layout.rightMargin = page.rightMargin();
+    layout.leftPadding = page.leftPadding();
+    layout.rightPadding = page.rightPadding();
     qreal left = 0, width = page.width();
     if (page.pageSide() == KWPage::PageSpread) {
         width /= 2;
         if (page.pageNumber() != pageNumber) { // doing the 'right' one
             left = width;
-            qreal x = layout.left; // swap margins
-            layout.left = layout.right;
-            layout.right = x;
+            qreal x = layout.leftMargin; // swap margins
+            layout.leftMargin = layout.rightMargin;
+            layout.rightMargin = x;
+            x = layout.leftPadding; // swap paddings
+            layout.leftPadding = layout.rightPadding;
+            layout.rightPadding = x;
         }
     }
-    qreal textWidth = width - layout.left - layout.right;
+    qreal textWidth = width - layout.leftMargin - layout.rightMargin
+                            - layout.leftPadding - layout.rightPadding;
 
-    const int columns = page.pageStyle().columns().columns/* *
-        (page.pageSide() == KWPage::PageSpread ? 2: 1)*/;
+    KWPageStyle pageStyle = page.pageStyle();
+    const int columns = pageStyle.hasMainTextFrame() ? pageStyle.columns().columns * (page.pageSide() == KWPage::PageSpread ? 2 : 1) : 0;
     int columnsCount = columns;
-    KWTextFrame **main, *footer = 0, *endnote = 0, *header = 0, *footnote = 0;
+    KWTextFrame **main, *footer = 0, *endnote = 0, *header = 0;
     main = new KWTextFrame*[columnsCount];
-    main[0] = 0;
+    if (columns > 0)
+        main[0] = 0;
     QRectF pageRect(left, page.offsetInDocument(), width, page.height());
     foreach (KWFrame *frame, framesInPage(pageRect)) {
         KWTextFrameSet *textFrameSet = dynamic_cast<KWTextFrameSet*>(frame->frameSet());
+        if (textFrameSet == 0 || textFrameSet->textFrameSetType() == KWord::OtherTextFrameSet)
+            minZIndex = qMin(minZIndex, frame->shape()->zIndex());
         if (textFrameSet == 0) continue;
         switch (textFrameSet->textFrameSetType()) {
         case KWord::OddPagesHeaderTextFrameSet:
         case KWord::EvenPagesHeaderTextFrameSet: {
             header = static_cast<KWTextFrame *>(frame);
-            minimumHeight[1] = 10;
-            requestedHeight[1] = header->shape()->size().height();
-            minimumHeight[2] = page.pageStyle().headerDistance();
+            minimumHeight[1] = qMax((qreal)10, page.pageStyle().headerMinimumHeight());
+            requestedHeight[1] = static_cast<KWTextFrame *>(textFrameSet->frames().first())->minimumFrameHeight();
+            if (minimumHeight[1] < page.pageStyle().headerDistance())
+                minimumHeight[2] = page.pageStyle().headerDistance() - minimumHeight[1];
             break;
         }
         case KWord::OddPagesFooterTextFrameSet:
         case KWord::EvenPagesFooterTextFrameSet: {
             footer = static_cast<KWTextFrame *>(frame);
-            minimumHeight[9] = 10;
-            requestedHeight[9] = footer->shape()->size().height();
-            minimumHeight[8] = page.pageStyle().headerDistance();
+            minimumHeight[7] = qMax((qreal)10, page.pageStyle().footerMinimumHeight());
+            requestedHeight[7] = static_cast<KWTextFrame *>(textFrameSet->frames().first())->minimumFrameHeight();
+            if(minimumHeight[7] < page.pageStyle().footerDistance())
+                minimumHeight[6] = page.pageStyle().footerDistance() - minimumHeight[7];
             break;
         }
         case KWord::MainTextFrameSet: {
             if (columnsCount < 1) {
-                kWarning() << "Too many columns present on page, ignoring 1" << endl;
+                kWarning(32001) << "Too many columns present on page, ignoring 1";
                 break;
             }
             main[--columnsCount] = static_cast<KWTextFrame *>(frame);
             minimumHeight[3] = 10;
+            // make at least one line fit lest we add endless pages.
+            QTextLayout *layout = textFrameSet->document()->begin().layout();
+            if (layout && layout->lineCount() > 0) {
+                minimumHeight[3] = qMax((qreal) 10, layout->lineAt(0).height());
+            }
+
             requestedHeight[3] = -1; // rest
             break;
         }
@@ -285,10 +307,21 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         default:;
         }
     }
+    if (minZIndex < INT_MAX) {
+        --minZIndex;
+        for (int i = 0; i < columns; ++i)
+            main[i]->shape()->setZIndex(minZIndex);
+        if (footer)
+            footer->shape()->setZIndex(minZIndex);
+        if (endnote)
+            endnote->shape()->setZIndex(minZIndex);
+        if (header)
+            header->shape()->setZIndex(minZIndex);
+    }
 
     // spread space across items.
     qreal heightLeft = page.height();
-    for (int i = 0; i < 11; i++)
+    for (int i = 0; i < 9; i++)
         heightLeft -= qMax(minimumHeight[i], requestedHeight[i]);
     if (heightLeft >= 0) { // easy; plenty of space
         if (minimumHeight[5] > 0) // if we have an endnote
@@ -296,7 +329,7 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         else
             minimumHeight[3] += heightLeft; // add space to main text frame
         qreal y = page.offsetInDocument();
-        for (int i = 0; i < 11; i++) {
+        for (int i = 0; i < 9; i++) {
             resultingPositions[i] = y;
             y += qMax(minimumHeight[i], requestedHeight[i]);
         }
@@ -304,12 +337,11 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         // for situations where the header + footer are too big to fit together with a
         // minimum sized main text frame.
         minimumHeight[5] = 0; // no end note
-        minimumHeight[7] = 0; // no footnote
         heightLeft = page.height();
-        for (int i = 0; i < 11; i++)
+        for (int i = 0; i < 9; i++)
             heightLeft -= minimumHeight[i];
         qreal y = page.offsetInDocument();
-        for (int i = 0; i < 11; i++) {
+        for (int i = 0; i < 9; i++) {
             resultingPositions[i] = y;
             qreal row = minimumHeight[i];
             if (requestedHeight[i] > row) {
@@ -320,11 +352,12 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
     }
 
     // actually move / size the frames.
-    if (main[0]) {
+    if (columns > 0 && main[0]) {
         const qreal columnWidth = textWidth / columns;
         QPointF *points = new QPointF[columns];
         for (int i = columns - 1; i >= 0; i--)
-            points[i] = QPointF(left + layout.left + columnWidth * i, resultingPositions[3]);
+            points[i] = QPointF(left + layout.leftMargin + layout.leftPadding
+                                + columnWidth * i, resultingPositions[3]);
         for (int i = 0; i < columns; i++) {
             for (int f = 0; f < columns; f++) {
                 if (f == i) continue;
@@ -348,28 +381,20 @@ void KWFrameLayout::layoutFramesOnPage(int pageNumber)
         }
         delete[] points;
     }
-#ifdef __GNUC__
-#warning can never reach that code as footnote is a constant 0
-#endif
-    if (footnote) {
-        footnote->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[7]));
-        footnote->shape()->setSize(QSizeF(textWidth, resultingPositions[8] - resultingPositions[7]));
-    }
     if (endnote) {
         endnote->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[5]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[5]));
         endnote->shape()->setSize(QSizeF(textWidth, resultingPositions[6] - resultingPositions[5]));
     }
     if (header) {
         header->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[1]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[1]));
         header->shape()->setSize(QSizeF(textWidth, resultingPositions[2] - resultingPositions[1]));
     }
     if (footer) {
         footer->shape()->setPosition(
-            QPointF(left + layout.left, resultingPositions[9]));
-        footer->shape()->setSize(QSizeF(textWidth, resultingPositions[10] - resultingPositions[9]));
+            QPointF(left + layout.leftMargin + layout.leftPadding, resultingPositions[7]));
+        footer->shape()->setSize(QSizeF(textWidth, resultingPositions[8] - resultingPositions[7]));
     }
     delete [] main;
 // TODO footnotes, endnotes
@@ -397,7 +422,7 @@ bool KWFrameLayout::shouldHaveHeaderOrFooter(int pageNumber, bool header, KWord:
     return true;
 }
 
-QList<KWFrame *> KWFrameLayout::framesInPage(QRectF page)
+QList<KWFrame *> KWFrameLayout::framesInPage(const QRectF &page)
 {
     // hopefully replaced with a tree
     QList<KWFrame*> answer;
@@ -478,8 +503,8 @@ void KWFrameLayout::setup()
             break;
         case KWord::MainTextFrameSet:
             m_maintext = tfs;
-            disconnect (tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
-            connect (tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
+            disconnect(tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
+            connect(tfs, SIGNAL(frameRemoved(KWFrame*)), this, SLOT(mainframeRemoved(KWFrame*)));
         default: ;// ignore
         }
         if (tfs->pageStyle().isValid())
@@ -491,12 +516,12 @@ void KWFrameLayout::setup()
 KoShape *KWFrameLayout::createTextShape(const KWPage &page)
 {
     Q_ASSERT(page.isValid());
-    KoShapeFactory *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
+    KoShapeFactoryBase *factory = KoShapeRegistry::instance()->value(TextShape_SHAPEID);
     Q_ASSERT(factory);
-    QMap<QString, KoDataCenter *> dataCenterMap;
+    KoResourceManager *rm = 0;
     if (m_document)
-        dataCenterMap = m_document->dataCenterMap();
-    KoShape *shape = factory->createDefaultShapeAndInit(dataCenterMap);
+        rm = m_document->resourceManager();
+    KoShape *shape = factory->createDefaultShape(rm);
     shape->setPosition(QPointF(0, page.offsetInDocument()));
     return shape;
 }
@@ -638,21 +663,30 @@ void KWFrameLayout::createNewFrameForPage(KWTextFrameSet *fs, int pageNumber)
     }
 }
 
-KWFrame* KWFrameLayout::createCopyFrame(KWFrameSet *fs, const KWPage &page)
+KWFrame *KWFrameLayout::createCopyFrame(KWFrameSet *fs, const KWPage &page)
 {
     Q_ASSERT(page.isValid());
     if (fs->frameCount() == 0) { // special case for the headers. Just return a new textframe.
         KWTextFrameSet *tfs = dynamic_cast<KWTextFrameSet*>(fs);
         Q_ASSERT(tfs); // an empty, non-text frameset asking for a copy? Thats a bug.
-        KWTextFrame *frame = new KWTextFrame(createTextShape(page), tfs);
+        KoShape *shape = createTextShape(page);
+        shape->setSize(QSize(20, 10));
+        KWTextFrame *frame = new KWTextFrame(shape, tfs);
 
         return frame;
     }
-    KWFrame *lastFrame = fs->frames().last();
-    KWCopyShape *shape = new KWCopyShape(lastFrame->shape());
+
+    KoShape *orig = 0;
+    foreach (KWFrame *candidate, fs->frames()) {
+        if (candidate->isCopy())
+            break;
+        orig = candidate->shape();
+    }
+    Q_ASSERT(orig); // can't have a frameset with only copy frames.
+
+    KWCopyShape *shape = new KWCopyShape(orig, m_pageManager);
     shape->setPosition(QPointF(0, page.offsetInDocument()));
     KWFrame *frame = new KWFrame(shape, fs);
-    frame->makeCopyFrame();
     return frame;
 }
 
