@@ -27,6 +27,7 @@
 #include "KoShapeSavingContext.h"
 #include "KoConnectionShapeLoadingUpdater.h"
 #include "KoPathShapeLoader.h"
+#include "KoPathPoint.h"
 #include <KoXmlReader.h>
 #include <KoXmlWriter.h>
 #include <KoXmlNS.h>
@@ -386,12 +387,7 @@ bool KoConnectionShape::loadOdf(const KoXmlElement & element, KoShapeLoadingCont
             // if there is none, use the bounding rectangle of the parsed path
             viewBox = outline().boundingRect();
         }
-        // convert path to viewbox coordinates to have a bounding rect of (0,0 1x1)
-        // which can later be fitted back into the target rect once we have all
-        // the required information
         QMatrix viewMatrix;
-        viewMatrix.scale(viewBox.width() ? static_cast<qreal>(1.0) / viewBox.width() : 1.0,
-                         viewBox.height() ? static_cast<qreal>(1.0) / viewBox.height() : 1.0);
         viewMatrix.translate(-viewBox.left(), -viewBox.top());
         d->map(viewMatrix);
 
@@ -432,11 +428,25 @@ void KoConnectionShape::finishLoadingConnection()
             } else {
                 p2 = d->handles[EndHandle];
             }
+
+            QPointF bgp = m_subpaths.first()->first()->point();
+            QPointF egp = m_subpaths.last()->last()->point();
+            qreal x1 = bgp.x(), y1 = bgp.y(), x2 = egp.x(), y2 = egp.y(),
+                  kX = (p2.x()-p1.x())/(egp.x()-bgp.x()),
+                  kY = (p2.y()-p1.y())/(egp.y()-bgp.y()),
+                  width = outline().boundingRect().width(),
+                  height = outline().boundingRect().height();
+
+            p1.setY(p1.y()-y1*kY);
+            p1.setX(p1.x()-x1*kX);
+            p2.setY(p2.y()+(width-y2)*kY);
+            p2.setX(p2.x()+(height-x2)*kX);
+
             QRectF targetRect = QRectF(p1, p2).normalized();
             // transform the normalized coordinates back to our target rectangle
             QMatrix viewMatrix;
             viewMatrix.translate(targetRect.x(), targetRect.y());
-            viewMatrix.scale(targetRect.width(), targetRect.height());
+            viewMatrix.scale(kX, kY);
             d->map(viewMatrix);
 
             // pretend we are during a forced update, so normalize()
