@@ -64,7 +64,9 @@ public:
         CenteredPosition ///< the centred corner
     };
 
-
+    /// Maximal possible shape z-index
+    static int maxZIndex() { return 0x2000-1; }
+    
 private:
     KoFlake();
 };
