@@ -18,11 +18,12 @@
  * Boston, MA 02110-1301, USA.
  */
 #include "KoToolProxy.h"
+#include "KoToolProxy_p.h"
 
-#include "KoTool.h"
+#include "KoToolBase.h"
 #include "KoPointerEvent.h"
 #include "KoInputDevice.h"
-#include "KoToolManager.h"
+#include "KoToolManager_p.h"
 #include "KoToolSelection.h"
 #include "KoCanvasBase.h"
 #include "KoCanvasController.h"
@@ -33,132 +34,76 @@
 #include <kdebug.h>
 #include <QTimer>
 
-/* Unused for now.
-#if 0
-namespace
+KoToolProxyPrivate::KoToolProxyPrivate(KoToolProxy *p)
+    : activeTool(0),
+    tabletPressed(false),
+    hasSelection(false),
+    controller(0),
+    parent(p)
 {
-
-    // Helper class to determine when the user might switch between
-    // tablet and mouse.
-    class TabletProximityFilter {
-    public:
-
-        TabletProximityFilter( KoToolManager * manager )
-            : m_manager( manager )
-            {
-            }
-
-        virtual ~TabletProximityFilter();
-
-        bool eventFilter( QObject * object,  QEvent * event ) {
-
-            if ( object == qApp ) {
-                switch( event->type() ) {
-                    case QEvent::TabletEnterProximity:
-                        break;
-                    case QEvent::TabletLeaveProximity:
-                        break;
-                    default:
-                        break;
-                }
-            }
-            return false;
-        }
-
-    private:
-
-        KoToolManager * m_manager;
-    };
-
+    scrollTimer.setInterval(100);
+    mouseLeaveWorkaround = false;
 }
-#endif */
 
-
-class KoToolProxy::Private
+void KoToolProxyPrivate::timeout() // Auto scroll the canvas
 {
-public:
-    Private(KoToolProxy *p)
-        : activeTool(0),
-        tabletPressed(false),
-        hasSelection(false),
-        controller(0),
-        parent(p)
-    {
-        scrollTimer.setInterval(100);
-        mouseLeaveWorkaround = false;
-    }
+    Q_ASSERT(controller);
+    int offsetX = controller->canvasOffsetX();
+    int offsetY = controller->canvasOffsetY();
+    // get the points version of 10 pixels offset.
+    QPointF offset = controller->canvas()->viewConverter()->viewToDocument(QPointF(10, 10));
+    QRectF mouseArea(scrollEdgePoint, QSizeF(offset.x(), offset.y()));
+    mouseArea.setTopLeft(mouseArea.center());
 
-    void timeout() // Auto scroll the canvas
-    {
-        Q_ASSERT(controller);
-        int offsetX = controller->canvasOffsetX();
-        int offsetY = controller->canvasOffsetY();
-        // get the points version of 10 pixels offset.
-        QPointF offset = controller->canvas()->viewConverter()->viewToDocument(QPointF(10, 10));
-        QRectF mouseArea(scrollEdgePoint, QSizeF(offset.x(), offset.y()));
-        mouseArea.setTopLeft(mouseArea.center());
+    controller->ensureVisible(mouseArea, true);
 
-        controller->ensureVisible(mouseArea, true);
+    QPoint moved(offsetX - controller->canvasOffsetX(), offsetY - controller->canvasOffsetY());
+    if (moved.x() == 0 && moved.y() == 0)
+        return;
+    scrollEdgePoint += controller->canvas()->viewConverter()->viewToDocument(moved);
 
-        QPoint moved(offsetX - controller->canvasOffsetX(), offsetY - controller->canvasOffsetY());
-        if (moved.x() == 0 && moved.y() == 0)
-            return;
-        scrollEdgePoint += controller->canvas()->viewConverter()->viewToDocument(moved);
+    QMouseEvent event(QEvent::MouseMove, scrollEdgePoint.toPoint(), Qt::LeftButton, Qt::LeftButton, 0);
+    KoPointerEvent ev(&event, scrollEdgePoint);
+    activeTool->mouseMoveEvent(&ev);
+}
 
-        QMouseEvent event(QEvent::MouseMove, scrollEdgePoint.toPoint(), Qt::LeftButton, Qt::LeftButton, 0);
-        KoPointerEvent ev(&event, scrollEdgePoint);
-        activeTool->mouseMoveEvent(&ev);
-    }
+void KoToolProxyPrivate::checkAutoScroll(const KoPointerEvent &event)
+{
+    if (controller == 0) return;
+    if (!activeTool) return;
+    if (!activeTool->wantsAutoScroll()) return;
+    if (!event.isAccepted()) return;
+    if (event.buttons() != Qt::LeftButton) return;
+    scrollEdgePoint = event.point;
+    if (! scrollTimer.isActive())
+        scrollTimer.start();
+}
 
-    void checkAutoScroll(const KoPointerEvent &event)
-    {
-        if (controller == 0) return;
-        if (!activeTool->wantsAutoScroll()) return;
-        if (!event.isAccepted()) return;
-        if (event.buttons() != Qt::LeftButton) return;
-        scrollEdgePoint = event.point;
-        if (! scrollTimer.isActive())
-            scrollTimer.start();
-    }
+void KoToolProxyPrivate::selectionChanged(bool newSelection)
+{
+    if (hasSelection == newSelection)
+        return;
+    hasSelection = newSelection;
+    emit parent->selectionChanged(hasSelection);
+}
 
-    void selectionChanged(bool newSelection)
-    {
-        if (hasSelection == newSelection)
-            return;
-        hasSelection = newSelection;
-        emit parent->selectionChanged(hasSelection);
-    }
+bool KoToolProxyPrivate::isActiveLayerEditable()
+{
+    if (!activeTool)
+        return false;
 
-    bool isActiveLayerEditable()
-    {
-        if( ! activeTool )
-            return false;
-
-        KoShapeManager * shapeManager = activeTool->canvas()->shapeManager();
-        KoShapeLayer * activeLayer = shapeManager->selection()->activeLayer();
-        if( activeLayer && ! activeLayer->isEditable() )
-            return false;
-        return true;
-    }
-
-    KoTool *activeTool;
-    bool tabletPressed;
-    bool hasSelection;
-    QTimer scrollTimer;
-    QPointF scrollEdgePoint;
-    KoCanvasController *controller;
-    KoToolProxy *parent;
-
-    QPoint mouseDownPoint; // used to determine if the mouse-release is after a drag or a simple click
-
-    bool mouseLeaveWorkaround; // up until at least 4.3.0 we get a mouse move event when the tablet leaves the canvas.
-};
+    KoShapeManager * shapeManager = activeTool->canvas()->shapeManager();
+    KoShapeLayer * activeLayer = shapeManager->selection()->activeLayer();
+    if (activeLayer && !activeLayer->isEditable())
+        return false;
+    return true;
+}
 
 KoToolProxy::KoToolProxy(KoCanvasBase *canvas, QObject *parent)
         : QObject(parent),
-        d(new Private(this))
+        d(new KoToolProxyPrivate(this))
 {
-    KoToolManager::instance()->registerToolProxy(this, canvas);
+    KoToolManager::instance()->priv()->registerToolProxy(this, canvas);
 
     connect(&d->scrollTimer, SIGNAL(timeout()), this, SLOT(timeout()));
 }
@@ -181,13 +126,13 @@ void KoToolProxy::repaintDecorations()
 void KoToolProxy::tabletEvent(QTabletEvent *event, const QPointF &point)
 {
     KoInputDevice id(event->device(), event->pointerType(), event->uniqueId());
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
 
     KoPointerEvent ev(event, point);
     switch (event->type()) {
     case QEvent::TabletPress:
         ev.setTabletButton(Qt::LeftButton);
-        if (! d->tabletPressed && d->activeTool)
+        if (!d->tabletPressed && d->activeTool)
             d->activeTool->mousePressEvent(&ev);
         d->tabletPressed = true;
         break;
@@ -195,11 +140,14 @@ void KoToolProxy::tabletEvent(QTabletEvent *event, const QPointF &point)
         ev.setTabletButton(Qt::LeftButton);
         d->tabletPressed = false;
         d->scrollTimer.stop();
-        if (d->activeTool) d->activeTool->mouseReleaseEvent(&ev);
+        if (d->activeTool)
+            d->activeTool->mouseReleaseEvent(&ev);
         break;
     case QEvent::TabletMove:
-        if (d->tabletPressed) ev.setTabletButton(Qt::LeftButton);
-        if (d->activeTool) d->activeTool->mouseMoveEvent(&ev);
+        if (d->tabletPressed)
+            ev.setTabletButton(Qt::LeftButton);
+        if (d->activeTool)
+            d->activeTool->mouseMoveEvent(&ev);
         d->checkAutoScroll(ev);
     default:
         ; // ignore the rest.
@@ -215,27 +163,33 @@ void KoToolProxy::mousePressEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     d->mouseDownPoint = event->pos();
 
     if (d->tabletPressed) // refuse to send a press unless there was a release first.
         return;
 
     KoPointerEvent ev(event, point);
-    if (d->activeTool) d->activeTool->mousePressEvent(&ev);
+    if (d->activeTool)
+        d->activeTool->mousePressEvent(&ev);
+    else
+        event->ignore();
 }
 
 void KoToolProxy::mouseDoubleClickEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
-    if (d->activeTool == 0)  return;
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    if (d->activeTool == 0) {
+        event->ignore();
+        return;
+    }
 
     KoPointerEvent ev(event, point);
     d->activeTool->mouseDoubleClickEvent(&ev);
     if (! event->isAccepted())
-        d->activeTool->m_canvas->shapeManager()->suggestChangeTool(&ev);
+        d->activeTool->canvas()->shapeManager()->suggestChangeTool(&ev);
 }
 
 void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
@@ -245,9 +199,11 @@ void KoToolProxy::mouseMoveEvent(QMouseEvent *event, const QPointF &point)
         return;
     }
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
-    if (d->activeTool == 0)
+    KoToolManager::instance()->priv()->switchInputDevice(id);
+    if (d->activeTool == 0) {
+        event->ignore();
         return;
+    }
 
     KoPointerEvent ev(event, point);
     d->activeTool->mouseMoveEvent(&ev);
@@ -259,7 +215,7 @@ void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
 {
     d->mouseLeaveWorkaround = false;
     KoInputDevice id;
-    KoToolManager::instance()->switchInputDevice(id);
+    KoToolManager::instance()->priv()->switchInputDevice(id);
     d->scrollTimer.stop();
 
     KoPointerEvent ev(event, point);
@@ -270,8 +226,8 @@ void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
                 && qAbs(d->mouseDownPoint.x() - event->x()) < 5
                 && qAbs(d->mouseDownPoint.y() - event->y()) < 5) {
             // we potentially will change the selection
-            Q_ASSERT(d->activeTool->m_canvas);
-            KoShapeManager *manager = d->activeTool->m_canvas->shapeManager();
+            Q_ASSERT(d->activeTool->canvas());
+            KoShapeManager *manager = d->activeTool->canvas()->shapeManager();
             Q_ASSERT(manager);
             // only change the selection if that will not lead to losing a complex selection
             if (manager->selection()->count() <= 1) {
@@ -286,23 +242,34 @@ void KoToolProxy::mouseReleaseEvent(QMouseEvent *event, const QPointF &point)
                 }
             }
         }
+    } else {
+        event->ignore();
     }
 }
 
 void KoToolProxy::keyPressEvent(QKeyEvent *event)
 {
-    if (d->activeTool) d->activeTool->keyPressEvent(event);
+    if (d->activeTool)
+        d->activeTool->keyPressEvent(event);
+    else
+        event->ignore();
 }
 
 void KoToolProxy::keyReleaseEvent(QKeyEvent *event)
 {
-    if (d->activeTool) d->activeTool->keyReleaseEvent(event);
+    if (d->activeTool)
+        d->activeTool->keyReleaseEvent(event);
+    else
+        event->ignore();
 }
 
-void KoToolProxy::wheelEvent(QWheelEvent * event, const QPointF &point)
+void KoToolProxy::wheelEvent(QWheelEvent *event, const QPointF &point)
 {
     KoPointerEvent ev(event, point);
-    if (d->activeTool) d->activeTool->wheelEvent(&ev);
+    if (d->activeTool)
+        d->activeTool->wheelEvent(&ev);
+    else
+        event->ignore();
 }
 
 QVariant KoToolProxy::inputMethodQuery(Qt::InputMethodQuery query, const KoViewConverter &converter) const
@@ -324,7 +291,7 @@ KoToolSelection* KoToolProxy::selection()
     return 0;
 }
 
-void KoToolProxy::setActiveTool(KoTool *tool)
+void KoToolProxy::setActiveTool(KoToolBase *tool)
 {
     if (d->activeTool)
         disconnect(d->activeTool, SIGNAL(selectionChanged(bool)), this, SLOT(selectionChanged(bool)));
@@ -336,9 +303,9 @@ void KoToolProxy::setActiveTool(KoTool *tool)
     }
 }
 
-void KoToolProxy::setCanvasController(KoCanvasController *controller)
+void KoToolProxyPrivate::setCanvasController(KoCanvasController *c)
 {
-    d->controller = controller;
+    controller = c;
 }
 
 QHash<QString, KAction*> KoToolProxy::actions() const
@@ -388,4 +355,9 @@ void KoToolProxy::deleteSelection()
         return d->activeTool->deleteSelection();
 }
 
+KoToolProxyPrivate *KoToolProxy::priv()
+{
+    return d;
+}
+
 #include <KoToolProxy.moc>
