@@ -24,17 +24,17 @@
 #include <KLocale>
 #include <KDebug>
 
-#include <KoZoomAction.h>
 #include <KoZoomHandler.h>
 #include <KoCanvasController.h>
 
 class KoZoomController::Private
 {
 public:
-    Private(KoZoomController *p, bool doSpecialAspectMode)
+    Private(KoZoomController *p, KoZoomAction::SpecialButtons specialButtons)
         : canvasController(0), zoomHandler(0), fitMargin(0), parent(p)
     {
-        action = new KoZoomAction(KoZoomMode::ZOOM_WIDTH | KoZoomMode::ZOOM_PAGE, i18n("Zoom"), doSpecialAspectMode, 0);
+        action = new KoZoomAction(KoZoomMode::ZOOM_WIDTH | KoZoomMode::ZOOM_PAGE, i18n("Zoom"), 0);
+        action->setSpecialButtons(specialButtons);
     }
     ~Private()
     {
@@ -45,21 +45,21 @@ public:
     void setAvailableSize()
     {
         if(zoomHandler->zoomMode() == KoZoomMode::ZOOM_WIDTH)
-            setZoom(KoZoomMode::ZOOM_WIDTH, 0);
+            setZoom(KoZoomMode::ZOOM_WIDTH, -1);
         if(zoomHandler->zoomMode() == KoZoomMode::ZOOM_PAGE)
-            setZoom(KoZoomMode::ZOOM_PAGE, 0);
+            setZoom(KoZoomMode::ZOOM_PAGE, -1);
     }
 
     /// when the canvas controller wants us to change zoom
-    void requestZoomBy(const double factor)
+    void requestZoomBy(const qreal factor)
     {
-        double zoom = zoomHandler->zoomInPercent() / 100.0;
+        qreal zoom = zoomHandler->zoom();
         action->setZoom(factor*zoom);
         setZoom(KoZoomMode::ZOOM_CONSTANT, factor*zoom);
         action->setEffectiveZoom(factor*zoom);
     }
 
-    void setZoom(KoZoomMode::Mode mode, double zoom)
+    void setZoom(KoZoomMode::Mode mode, qreal zoom)
     {
         parent->setZoom(mode, zoom);
     }
@@ -73,23 +73,28 @@ public:
     KoZoomController *parent;
 };
 
-KoZoomController::KoZoomController(KoCanvasController *co, KoZoomHandler *zh, KActionCollection *actionCollection, bool doSpecialAspectMode)
-    : d(new Private(this, doSpecialAspectMode))
+KoZoomController::KoZoomController(KoCanvasController *co, KoZoomHandler *zh, KActionCollection *actionCollection, KoZoomAction::SpecialButtons specialButtons)
+    : d(new Private(this, specialButtons))
 {
     d->canvasController = co;
+    d->fitMargin = co->margin();
     d->zoomHandler = zh;
-    connect(d->action, SIGNAL(zoomChanged(KoZoomMode::Mode, double)),
-            this, SLOT(setZoom(KoZoomMode::Mode, double)));
+    connect(d->action, SIGNAL(zoomChanged(KoZoomMode::Mode, qreal)),
+            this, SLOT(setZoom(KoZoomMode::Mode, qreal)));
     connect(d->action, SIGNAL(aspectModeChanged(bool)),
             this, SIGNAL(aspectModeChanged(bool)));
+    connect(d->action, SIGNAL(zoomedToSelection()),
+            this, SIGNAL(zoomedToSelection()));
+    connect(d->action, SIGNAL(zoomedToAll()),
+            this, SIGNAL(zoomedToAll()));
 
     actionCollection->addAction("view_zoom", d->action);
-    actionCollection->addAction(KStandardAction::ZoomIn,  "zood->in", d->action, SLOT(zoomIn()));
-    actionCollection->addAction(KStandardAction::ZoomOut,  "zood->out", d->action, SLOT(zoomOut()));
+    actionCollection->addAction(KStandardAction::ZoomIn,  "zoom_in", d->action, SLOT(zoomIn()));
+    actionCollection->addAction(KStandardAction::ZoomOut,  "zoom_out", d->action, SLOT(zoomOut()));
 
     connect(d->canvasController, SIGNAL( sizeChanged(const QSize & ) ), this, SLOT( setAvailableSize() ) );
 
-    connect(d->canvasController, SIGNAL( zoomBy(const double ) ), this, SLOT( requestZoomBy( const double ) ) );
+    connect(d->canvasController, SIGNAL( zoomBy(const qreal ) ), this, SLOT( requestZoomBy( const qreal ) ) );
 }
 
 KoZoomController::~KoZoomController()
@@ -127,7 +132,7 @@ void KoZoomController::setDocumentSize( const QSizeF &documentSize )
     d->canvasController->recenterPreferred();
 }
 
-void KoZoomController::setZoom(KoZoomMode::Mode mode, double zoom)
+void KoZoomController::setZoom(KoZoomMode::Mode mode, qreal zoom)
 {
     if (d->zoomHandler->zoomMode() == mode && d->zoomHandler->zoom() == zoom)
         return; // no change
@@ -142,7 +147,7 @@ void KoZoomController::setZoom(KoZoomMode::Mode mode, double zoom)
     {
         zoom = (d->canvasController->viewport()->size().width() - 2*d->fitMargin)
                          / (d->zoomHandler->resolutionX() * d->pageSize.width());
-        d->action->setCurrentAction(KoZoomMode::toString(mode));
+        d->action->setSelectedZoomMode(mode);
         d->action->setEffectiveZoom(zoom);
     }
     else if(mode == KoZoomMode::ZOOM_PAGE)
@@ -152,7 +157,7 @@ void KoZoomController::setZoom(KoZoomMode::Mode mode, double zoom)
         zoom = qMin(zoom, (d->canvasController->viewport()->size().height() - 2*d->fitMargin)
                      / (d->zoomHandler->resolutionY() * d->pageSize.height()));
 
-        d->action->setCurrentAction(KoZoomMode::toString(mode));
+        d->action->setSelectedZoomMode(mode);
         d->action->setEffectiveZoom(zoom);
     }
 
@@ -175,9 +180,4 @@ void KoZoomController::setZoom(KoZoomMode::Mode mode, double zoom)
     d->canvasController->recenterPreferred();
 }
 
-void KoZoomController::setFitMargin( int margin )
-{
-    d->fitMargin = margin;
-}
-
 #include "KoZoomController.moc"
