@@ -20,8 +20,9 @@
 
 #include "KoPADocumentModel.h"
 
-#include <KoPADocument.h>
-#include <KoPAPageBase.h>
+#include "KoPADocument.h"
+#include "KoPAPageBase.h"
+#include "KoPAPageProvider.h"
 #include <KoShapePainter.h>
 #include <KoShapeManager.h>
 #include <KoShapeBorderModel.h>
@@ -46,6 +47,8 @@
 #include <QtCore/QAbstractItemModel>
 #include <QtCore/QMimeData>
 
+#include "commands/KoPAPageMoveCommand.h"
+
 KoPADocumentModel::KoPADocumentModel( QObject* parent, KoPADocument *document )
 : KoDocumentSectionModel( parent )
 , m_master(false)
@@ -77,7 +80,7 @@ int KoPADocumentModel::rowCount( const QModelIndex &parent ) const
     if( ! parentShape )
         return 0;
 
-    return parentShape->childCount();
+    return parentShape->shapeCount();
 }
 
 int KoPADocumentModel::columnCount( const QModelIndex & ) const
@@ -106,7 +109,7 @@ QModelIndex KoPADocumentModel::index( int row, int column, const QModelIndex &pa
     if( ! parentShape )
         return QModelIndex();
 
-    if( row < parentShape->childCount() )
+    if( row < parentShape->shapeCount() )
         return createIndex( row, column, childFromIndex( parentShape, row ) );
     else
         return QModelIndex();
@@ -196,11 +199,11 @@ QVariant KoPADocumentModel::data( const QModelIndex &index, int role ) const
             if( ! selection )
                 return false;
 
-            KoShapeLayer *layer = dynamic_cast<KoShapeLayer*>( shape );
+            /* KoShapeLayer *layer = dynamic_cast<KoShapeLayer*>( shape );
             if( layer )
                 return (layer == selection->activeLayer() );
-            else
-                return selection->isSelected( shape );
+            else */
+            return selection->isSelected( shape );
         }
         case PropertiesRole: return QVariant::fromValue( properties( shape ) );
         case AspectRatioRole:
@@ -211,7 +214,7 @@ QVariant KoPADocumentModel::data( const QModelIndex &index, int role ) const
             if( container )
             {
                 bbox = QRectF();
-                foreach( KoShape* shape, container->iterator() )
+                foreach( KoShape* shape, container->shapes() )
                     bbox = bbox.united( shape->outline().boundingRect() );
             }
             return qreal(bbox.width()) / bbox.height();
@@ -263,15 +266,16 @@ bool KoPADocumentModel::setData(const QModelIndex &index, const QVariant &value,
             setProperties( shape, value.value<PropertyList>());
             break;
         case ActiveRole:
-            if (value.toBool())
+            /* if (value.toBool())
             {
                 KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
                 KoSelection * selection = canvasController->canvas()->shapeManager()->selection();
 
                 KoShapeLayer *layer = dynamic_cast<KoShapeLayer*>( shape );
-                if( layer && selection )
+                if( layer && selection ) {
                     selection->setActiveLayer( layer );
-            }
+                }
+            } */
             break;
         default:
             return false;
@@ -285,19 +289,19 @@ KoDocumentSectionModel::PropertyList KoPADocumentModel::properties( KoShape* sha
 {
     PropertyList l;
     l << Property(i18n("Visible"), SmallIcon("14_layer_visible"), SmallIcon("14_layer_novisible"), shape->isVisible());
-    l << Property(i18n("Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isLocked());
+    l << Property(i18n("Locked"), SmallIcon("object-locked"), SmallIcon("object-unlocked"), shape->isGeometryProtected());
     return l;
 }
 
 void KoPADocumentModel::setProperties( KoShape* shape, const PropertyList &properties )
 {
     bool oldVisibleState = shape->isVisible();
-    bool oldLockedState = shape->isLocked();
+    bool oldLockedState = shape->isGeometryProtected();
 
     shape->setVisible( properties.at( 0 ).state.toBool() );
-    shape->setLocked( properties.at( 1 ).state.toBool() );
+    shape->setGeometryProtected( properties.at( 1 ).state.toBool() );
 
-    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isLocked() ) )
+    if( ( oldVisibleState != shape->isVisible() ) || ( oldLockedState != shape->isGeometryProtected() ) )
         shape->update();
 }
 
@@ -319,34 +323,33 @@ QImage KoPADocumentModel::createThumbnail( KoShape* shape, const QSize &thumbSiz
         else {
             size.setWidth( size.height() * ratio );
         }
-        QPixmap pixmap = page->thumbnail( size );
+        QPixmap pixmap = m_document->pageThumbnail( page, size );
         return pixmap.toImage();
     }
 
+    shapes.append( shape );
     KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
     if( container )
-        shapes = container->iterator();
-    else
-        shapes.append( shape );
+        shapes = container->shapes();
 
     shapePainter.setShapes( shapes );
 
     QImage thumb( size, QImage::Format_RGB32 );
     // draw the background of the thumbnail
     thumb.fill( QColor( Qt::white ).rgb() );
-    shapePainter.paintShapes( thumb );
+    shapePainter.paint(thumb);
 
     return thumb;
 }
 
 KoShape * KoPADocumentModel::childFromIndex( KoShapeContainer *parent, int row ) const
 {
-    return parent->iterator().at(row);
+    return parent->shapes().at(row);
 
     if( parent != m_lastContainer )
     {
         m_lastContainer = parent;
-        m_childs = parent->iterator();
+        m_childs = parent->shapes();
         qSort( m_childs.begin(), m_childs.end(), KoShape::compareShapeZIndex );
     }
     return m_childs.at( row );
@@ -357,12 +360,12 @@ int KoPADocumentModel::indexFromChild( KoShapeContainer *parent, KoShape *child
     if ( !m_document )
         return 0;
 
-    return parent->iterator().indexOf( child );
+    return parent->shapes().indexOf( child );
 
     if( parent != m_lastContainer )
     {
         m_lastContainer = parent;
-        m_childs = parent->iterator();
+        m_childs = parent->shapes();
         qSort( m_childs.begin(), m_childs.end(), KoShape::compareShapeZIndex );
     }
     return m_childs.indexOf( child );
@@ -435,20 +438,27 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
 
     QList<KoShape*> toplevelShapes;
     QList<KoShapeLayer*> layers;
+    QList<KoPAPageBase *> pages;
     // remove shapes having its parent in the list
     // and separate the layers
     foreach( KoShape * shape, shapes )
     {
-        KoShapeContainer * parent = shape->parent();
+        // check whether the selection contains page
+        // by the UI rules, the selection should contains page only
+        KoPAPageBase *page = dynamic_cast<KoPAPageBase *>( shape );
+        if ( page ) {
+            pages.append( page );
+            continue;
+        }
+
+        KoShapeContainer *parentShape = shape->parent();
         bool hasParentInList = false;
-        while( parent )
-        {
-            if( shapes.contains( parent ) )
-            {
+        while ( parentShape ) {
+            if ( shapes.contains( parentShape ) ) {
                 hasParentInList = true;
                 break;
             }
-            parent = parent->parent();
+            parentShape = parentShape->parent();
         }
         if( hasParentInList )
             continue;
@@ -460,15 +470,27 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
             toplevelShapes.append( shape );
     }
 
-    if( ! parent.isValid() )
-    {
-        kDebug(30010) <<"KoPADocumentModel::dropMimeData parent = root";
-        return false;
+    // dropping to root, only page(s) is allowed
+    if ( !parent.isValid() ) {
+        if ( !pages.isEmpty() ) {
+            if ( row < 0 ) {
+                return false;
+            }
+            KoPAPageBase *after = ( row != 0 ) ? m_document->pageByIndex( row - 1, false ) : 0;
+            KoPAPageMoveCommand *command = new KoPAPageMoveCommand( m_document, pages, after );
+            m_document->addCommand( command );
+            kDebug(30010) << "KoPADocumentModel::dropMimeData parent = root, dropping page(s) as root, moving page(s)";
+            return true;
+        }
+        else {
+            kDebug(30010) << "KoPADocumentModel::dropMimeData parent = root, dropping non-page as root, returning false";
+            return false;
+        }
     }
+
     KoShape *shape = static_cast<KoShape*>( parent.internalPointer() );
     KoShapeContainer * container = dynamic_cast<KoShapeContainer*>( shape );
-    if( container )
-    {
+    if( container ) {
         KoShapeGroup * group = dynamic_cast<KoShapeGroup*>( container );
         if( group )
         {
@@ -477,13 +499,13 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
                 return false;
 
             emit layoutAboutToBeChanged();
-            beginInsertRows( parent, group->childCount(), group->childCount()+toplevelShapes.count() );
+            beginInsertRows( parent, group->shapeCount(), group->shapeCount()+toplevelShapes.count() );
 
             QUndoCommand * cmd = new QUndoCommand();
             cmd->setText( i18n("Reparent shapes") );
 
             foreach( KoShape * shape, toplevelShapes )
-                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
 
             new KoShapeGroupCommand( group, toplevelShapes, cmd );
             KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
@@ -498,25 +520,28 @@ bool KoPADocumentModel::dropMimeData( const QMimeData * data, Qt::DropAction act
             if( toplevelShapes.count() )
             {
                 emit layoutAboutToBeChanged();
-                beginInsertRows( parent, container->childCount(), container->childCount()+toplevelShapes.count() );
+                beginInsertRows( parent, container->shapeCount(), container->shapeCount()+toplevelShapes.count() );
 
                 QUndoCommand * cmd = new QUndoCommand();
                 cmd->setText( i18n("Reparent shapes") );
 
                 QList<bool> clipped;
+                QList<bool> inheritsTransform;
                 foreach( KoShape * shape, toplevelShapes )
                 {
                     if( ! shape->parent() )
                     {
                         clipped.append( false );
+                        inheritsTransform.append(false);
                         continue;
                     }
 
-                    clipped.append( shape->parent()->childClipped( shape ) );
-                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, cmd );
+                    clipped.append( shape->parent()->isClipped( shape ) );
+                    inheritsTransform.append(shape->parent()->inheritsTransform(shape));
+                    new KoShapeUngroupCommand( shape->parent(), QList<KoShape*>() << shape, QList<KoShape*>(), cmd );
                 }
                 // shapes are dropped on a container, so add them to the container
-                new KoShapeGroupCommand( container, toplevelShapes, clipped, cmd );
+                new KoShapeGroupCommand(container, toplevelShapes, clipped, inheritsTransform, cmd);
                 KoCanvasController * canvasController = KoToolManager::instance()->activeCanvasController();
                 canvasController->canvas()->addCommand( cmd );
 
@@ -600,4 +625,4 @@ void KoPADocumentModel::setMasterMode(bool master)
     update(); // Rebuild the model
 }
 
-#include "KoPADocumentModel.moc"
+#include <KoPADocumentModel.moc>
