@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
  * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -20,7 +20,8 @@
 #include "KoInlineTextObjectManager.h"
 #include "InsertNamedVariableAction_p.h"
 #include "InsertTextReferenceAction_p.h"
-#include "KoVariableRegistry.h"
+#include "InsertTextLocator_p.h"
+#include "KoInlineObjectRegistry.h"
 #include "KoTextLocator.h"
 #include "KoBookmark.h"
 
@@ -55,7 +56,8 @@ KoInlineObject *KoInlineTextObjectManager::inlineTextObject(int id) const
 void KoInlineTextObjectManager::insertInlineObject(QTextCursor &cursor, KoInlineObject *object)
 {
     QTextCharFormat oldCf = cursor.charFormat();
-    // create a new format out of the old so that the current formating is also used for the inserted object
+    // create a new format out of the old so that the current formatting is
+    // also used for the inserted object.  KoVariables render text too ;)
     QTextCharFormat cf(oldCf);
     cf.setObjectType(QTextFormat::UserObject + 1);
     cf.setProperty(InlineInstanceId, ++m_lastObjectId);
@@ -72,8 +74,9 @@ void KoInlineTextObjectManager::insertInlineObject(QTextCursor &cursor, KoInline
     }
 
     KoBookmark *bookmark = dynamic_cast<KoBookmark *>(object);
-    if (bookmark &&
-            (bookmark->type() == KoBookmark::StartBookmark || bookmark->type() == KoBookmark::SinglePosition))
+    if (bookmark
+            && (bookmark->type() == KoBookmark::StartBookmark
+                || bookmark->type() == KoBookmark::SinglePosition))
         m_bookmarkManager.insert(bookmark->name(), bookmark);
 
     // reset to use old format so that the InlineInstanceId is no longer set.
@@ -120,6 +123,13 @@ bool KoInlineTextObjectManager::removeInlineObject(QTextCursor &cursor)
     return false;
 }
 
+void KoInlineTextObjectManager::removeInlineObject(KoInlineObject *object)
+{
+    Q_ASSERT(object);
+    m_objects.remove(object->id());
+    // TODO dirty the document somehow
+}
+
 void KoInlineTextObjectManager::setProperty(KoInlineObject::Property key, const QVariant &value)
 {
     if (m_properties.contains(key)) {
@@ -128,8 +138,8 @@ void KoInlineTextObjectManager::setProperty(KoInlineObject::Property key, const
         m_properties.remove(key);
     }
     m_properties.insert(key, value);
-    foreach(KoInlineObject *obj, m_listeners)
-    obj->propertyChanged(key, value);
+    foreach (KoInlineObject *obj, m_listeners)
+        obj->propertyChanged(key, value);
 }
 
 QVariant KoInlineTextObjectManager::property(KoInlineObject::Property key) const
@@ -180,12 +190,13 @@ void KoInlineTextObjectManager::removeProperty(KoInlineObject::Property key)
 
 QList<QAction*> KoInlineTextObjectManager::createInsertVariableActions(KoCanvasBase *host) const
 {
-    QList<QAction *> answer = KoVariableRegistry::instance()->createInsertVariableActions(host);
+    QList<QAction *> answer = KoInlineObjectRegistry::instance()->createInsertVariableActions(host);
     int i = 0;
     foreach(const QString & name, m_variableManager.variables()) {
         answer.insert(i++, new InsertNamedVariableAction(host, this, name));
     }
 
+    answer.append(new InsertTextLocator(host));
     answer.append(new InsertTextReferenceAction(host, this));
     return answer;
 }
@@ -239,4 +250,9 @@ void KoInlineTextObjectManager::documentInformationUpdated(const QString &info,
         setProperty(KoInlineObject::SenderCompany, data);
 }
 
-#include "KoInlineTextObjectManager.moc"
+QList<KoInlineObject*> KoInlineTextObjectManager::inlineTextObjects() const
+{
+    return m_objects.values();
+}
+
+#include <KoInlineTextObjectManager.moc>
