@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2007 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007, 2009 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -26,6 +26,9 @@
 #include <QPointF>
 
 class KoShape;
+class KoTextAnchorPrivate;
+class KoXmlElement;
+class KoShapeLoadingContext;
 
 /**
  * This class is the object that is positioned in the text to be an anchor for a shape.
@@ -46,10 +49,10 @@ class KoShape;
  *
  * Steps to use a KoTextAnchor are; <ol>
  * <li> Create a new instance with e.g. new KoTextAnchor(myshape);
- * <li> Use loadOdfFromShape() to load additional attributes like the "text:anchor-type"
+ * <li> Use loadOdf() to load additional attributes like the "text:anchor-type"
  * <li> Position the anchor with updatePosition() what will attach the KoTextAnchor-instance to
  *    the TextShape's \a KoTextShapeContainerModel . </ol>
- * The position of the shape relative to the anchor is called the offset. It's loaded by loadOdfFromShape().
+ * The position of the shape relative to the anchor is called the offset. It's loaded by loadOdf().
  * @see KWAnchorStrategy for more information about the layout of anchors/shapes in KWord.
  */
 class KOTEXT_EXPORT KoTextAnchor : public KoInlineObject
@@ -63,7 +66,11 @@ public:
         BelowCurrentLine,   ///< Align the anchors bottom to the bottom of the line it is anchord in.
         BottomOfParagraph,  ///< Align the anchors bottom to the bottom of the paragraph it is anchord in.
         BottomOfFrame,      ///< Align the anchors bottom to the bottom of the frame.
-        VerticalOffset      ///< Move the anchor to be an exact vertical distance from the (baseline) of the anchor.
+        VerticalOffset,     ///< Move the anchor to be an exact vertical distance from the (baseline) of the anchor.
+        TopOfPage,          ///< Align the anchors top to the top of the page
+        BottomOfPage,       ///< Align the anchors bottom to the bottom of the page
+        TopOfPageContent,   ///< Align the anchors top to the top of the page content (top margin of page)
+        BottomOfPageContent ///< Align the anchors bottom to the bottom of the content (bottom margin of page)
     };
     /// the horizontal alignment options for the shape this anchor holds.
     enum AnchorHorizontal {
@@ -72,16 +79,9 @@ public:
         Center,             ///< Align the anchors center to the center of the frame it is laid-out in.
         ClosestToBinding,   ///< Like Left when on an odd page, or Right otherwise.
         FurtherFromBinding, ///< Like Left when on an even page, or Right otherwise.
-        HorizontalOffset    ///< Move the anchor to be an exact horizontal distance from the the anchor.
-    };
-
-    /// the anchor type we are using
-    enum AnchorType {
-        Page,               ///< Relative to the page
-        Frame,              ///< Relative to the frame containing the anchor
-        Paragraph,          ///< Relative to the paragraph
-        Char,               ///< Relative to the character before the anchor
-        AsChar              ///< The shape attached using this anchor must be considered as a character
+        HorizontalOffset,   ///< Move the anchor to be an exact horizontal distance from the the anchor.
+        LeftOfPage,         ///< Align the anchors left to the left of the page
+        RightOfPage,        ///< Align the anchors right to the right of the page
     };
 
     /**
@@ -89,7 +89,7 @@ public:
      * @param shape the anchored shape that this anchor links to.
      */
     KoTextAnchor(KoShape *shape);
-    ~KoTextAnchor();
+    virtual ~KoTextAnchor();
 
     /**
      * Return the shape that is linked to from the text anchor.
@@ -120,9 +120,6 @@ public:
     /// returns the cursor position in the document where this anchor is positioned.
     int positionInDocument() const;
 
-    /// returns the page number in case this is a page anchor
-    int pageNumber() const;
-
     /// returns the document that this anchor is associated with.
     const QTextDocument *document() const;
 
@@ -142,13 +139,21 @@ public:
     void setOffset(const QPointF &offset);
 
     /// Load the additional attributes.
-    bool loadOdfFromShape();
+    bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
     /// Save the additional attributes.
-    void saveOdf(KoShapeSavingContext & context);
+    void saveOdf(KoShapeSavingContext &context);
+
+    /**
+     * Returns true if the anchored frame is positioned as a big character in the text layout
+     * or false when it will not take any space as an inline object.
+     * An anchor with HorizontalOffset/VerticalOffset as alignment and an offset() that
+     * keeps it inside the text size it is placed on will act as a big character in the
+     * text flow potentially changing the ascent/descent of the line.
+     */
+    bool isPositionedInline() const;
 
 private:
-    class Private;
-    Private * const d;
+    Q_DECLARE_PRIVATE(KoTextAnchor)
 };
 
 #endif
