@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2006, 2009-2010 Thomas Zander <zander@kde.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -25,14 +25,15 @@
 
 #include <KoShapeUserData.h>
 #include <KoInsets.h>
+#include <KoXmlReaderForward.h>
 
 class QTextDocument;
-class KoXmlElement;
 class KoXmlWriter;
 class KoShapeLoadingContext;
 class KoShapeSavingContext;
 class KoTextShapeDataPrivate;
 class KoTextPage;
+class KoDocumentRdfBase;
 
 /**
  * The data store that is held by each KoTextShape instance.
@@ -49,7 +50,7 @@ class KOTEXT_EXPORT KoTextShapeData : public KoShapeUserData
 public:
     /// constructor
     KoTextShapeData();
-    ~KoTextShapeData();
+    virtual ~KoTextShapeData();
 
     /**
      * Replace the QTextDocument this shape will render.
@@ -121,6 +122,17 @@ public:
      */
     KoInsets shapeMargins() const;
 
+    /**
+     * Calling this method will do a layout run of the text for this shape using the
+     * provided textPage. The currently set page() will not be touched.
+     * This is a special method designed for small texts that are used on more than
+     * one page and have page-specific variables.
+     * In this case the text has to be re-laid out for each shape showing the
+     * document using a different textPage variable.
+     * @param textPage the page to layout the text for.
+     */
+    void relayoutFor(KoTextPage &textPage);
+
     /// Set the provider that provides us the number of the \p page this shape is on.
     void setPage(KoTextPage* textpage);
     /// Returns the provider that provides us the number of the page this shape is on.
@@ -130,15 +142,15 @@ public:
     * Load the TextShape from ODF.
     *
     * @see the @a TextShape::loadOdf() method which calls this method.
-    * @see the @a KoTextLoader::loadBoy() method which got called by this method
+    * @see the @a KoTextLoader::loadBody() method which got called by this method
     * to load the ODF.
     */
-    bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext & context);
+    bool loadOdf(const KoXmlElement & element, KoShapeLoadingContext & context, KoDocumentRdfBase *rdfData = 0);
     /**
     * Store the TextShape data as ODF.
     * @see TextShape::saveOdf()
     */
-    void saveOdf(KoShapeSavingContext & context, int from = 0, int to = -1) const;
+    void saveOdf(KoShapeSavingContext & context, int from = 0, int to = -1, KoDocumentRdfBase *rdfData = 0) const;
 
     /**
      * Set the page direction.
@@ -153,6 +165,11 @@ public:
      */
     KoText::Direction pageDirection() const;
 
+    /** Sets the vertical alignment of all the text inside the shape. */
+    void setVerticalAlignment(Qt::Alignment alignment);
+    /** Returns the vertical alignment of all the text in the shape */
+    Qt::Alignment verticalAlignment() const;
+
 signals:
     /**
      * emitted when the shape thinks it should be relayouted, for example after
