@@ -1,6 +1,7 @@
 /* This file is part of the KDE project
- * Copyright (C) 2006-2008 Thomas Zander <zander@kde.org>
- * Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
+ * Copyright (C) 2006-2009 Thomas Zander <zander@kde.org>
+ * Copyright (C) 2007 Sebastian Sauer <mail@dipe.org>
+*  Copyright (C) 2008 Thorsten Zachmann <zachmann@kde.org>
  * Copyright (C) 2008 Girish Ramakrishnan <girish@forwardbias.in>
  *
  * This library is free software; you can redistribute it and/or
@@ -24,6 +25,7 @@
 
 #include <QTextBlock>
 #include <QTextCursor>
+#include <QFontMetrics>
 
 #include <KoOdfLoadingContext.h>
 #include <KoOdfStylesReader.h>
@@ -97,7 +99,7 @@ KoCharacterStyle::Private::Private()
 void KoCharacterStyle::Private::setApplicationDefaults(KoOdfLoadingContext &context)
 {
     KoStyleStack defaultStyleStack;
-    const KoXmlElement* appDef = context.defaultStylesReader().defaultStyle("paragraph");
+    const KoXmlElement *appDef = context.defaultStylesReader().defaultStyle("paragraph");
     if (appDef) {
         defaultStyleStack.push(*appDef);
         defaultStyleStack.setTypeProperties("text");
@@ -116,9 +118,10 @@ void KoCharacterStyle::Private::ensureMinimalProperties(QTextCursor &cursor, boo
     QTextCharFormat format = cursor.charFormat();
     QList<int> keys = hardCodedDefaultStyle.keys();
     for (int i = 0; i < keys.count(); i++) {
-        QVariant variant = hardCodedDefaultStyle.value(keys.at(i));
-        if (!variant.isNull() && !format.hasProperty(keys.at(i))) {
-            format.setProperty(keys.at(i), variant);
+        const int key = keys.at(i);
+        QVariant variant = hardCodedDefaultStyle.value(key);
+        if (!variant.isNull() && !format.hasProperty(key)) {
+            format.setProperty(key, variant);
         }
     }
     cursor.mergeCharFormat(format);
@@ -164,8 +167,7 @@ QPen KoCharacterStyle::textOutline() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::TextOutline);
     if (variant.isNull()) {
-        QPen pen(Qt::NoPen);
-        return pen;
+        return QPen(Qt::NoPen);
     }
     return qvariant_cast<QPen>(variant);
 }
@@ -175,8 +177,7 @@ QBrush KoCharacterStyle::background() const
     QVariant variant = d->stylesPrivate.value(QTextFormat::BackgroundBrush);
 
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -190,8 +191,7 @@ QBrush KoCharacterStyle::foreground() const
 {
     QVariant variant = d->stylesPrivate.value(QTextFormat::ForegroundBrush);
     if (variant.isNull()) {
-        QBrush brush;
-        return brush;
+        return QBrush();
     }
     return qvariant_cast<QBrush>(variant);
 }
@@ -231,10 +231,52 @@ void KoCharacterStyle::applyStyle(QTextCursor *selection) const
     d->ensureMinimalProperties(*selection, false);
 }
 
+void KoCharacterStyle::unapplyStyle(QTextCharFormat &format) const
+{
+    QList<int> keys = d->stylesPrivate.keys();
+    for (int i = 0; i < keys.count(); i++) {
+        QVariant variant = d->stylesPrivate.value(keys[i]);
+        if (!variant.isNull()) {
+            if (variant == format.property(keys[i]))
+                format.clearProperty(keys[i]);
+        }
+    }
+
+    keys = d->hardCodedDefaultStyle.keys();
+    for (int i = 0; i < keys.count(); i++) {
+        QVariant variant = d->hardCodedDefaultStyle.value(keys.at(i));
+        if (!variant.isNull() && !format.hasProperty(keys.at(i))) {
+            format.setProperty(keys.at(i), variant);
+        }
+    }
+}
+
+void KoCharacterStyle::unapplyStyle(QTextBlock &block) const
+{
+    QTextCursor cursor(block);
+    QTextCharFormat cf = cursor.blockCharFormat();
+    unapplyStyle(cf);
+    cursor.setBlockCharFormat(cf);
+
+    if (block.length() == 1) // only the linefeed
+        return;
+    QTextBlock::iterator iter = block.end();
+    do {
+        iter--;
+        QTextFragment fragment = iter.fragment();
+        cursor.setPosition(fragment.position() + 1);
+        cf = cursor.charFormat();
+        unapplyStyle(cf);
+        cursor.setPosition(fragment.position());
+        cursor.setPosition(fragment.position() + fragment.length(), QTextCursor::KeepAnchor);
+        cursor.setCharFormat(cf);
+    } while (iter != block.begin());
+}
+
 // OASIS 14.2.29
-static void importOdfLine(const QString& type, const QString& style, const QString& width,
-                          KoCharacterStyle::LineStyle& lineStyle, KoCharacterStyle::LineType& lineType,
-                          KoCharacterStyle::LineWeight& lineWeight, qreal& lineWidth)
+static void importOdfLine(const QString &type, const QString &style, const QString &width,
+                          KoCharacterStyle::LineStyle &lineStyle, KoCharacterStyle::LineType &lineType,
+                          KoCharacterStyle::LineWeight &lineWeight, qreal &lineWidth)
 {
     lineStyle = KoCharacterStyle::NoLineStyle;
     lineType = KoCharacterStyle::NoLineType;
@@ -243,10 +285,14 @@ static void importOdfLine(const QString& type, const QString& style, const QStri
 
     QString fixedType = type;
     QString fixedStyle = style;
-    if (fixedType.isEmpty() && !fixedStyle.isEmpty())
+    if (fixedStyle == "none")
+        fixedType.clear();
+    else if (fixedType.isEmpty() && !fixedStyle.isEmpty())
         fixedType = "single";
-    else if (!fixedType.isEmpty() && fixedStyle.isEmpty())
+    else if (!fixedType.isEmpty() && fixedType != "none" && fixedStyle.isEmpty()) {
+        // don't set a style when the type is none
         fixedStyle = "solid";
+    }
 
     if (fixedType == "single")
         lineType = KoCharacterStyle::SingleLine;
@@ -548,7 +594,7 @@ void KoCharacterStyle::setStrikeOutWidth(LineWeight weight, qreal width)
     d->setProperty(KoCharacterStyle::StrikeOutWidth, width);
 }
 
-void KoCharacterStyle::strikeOutWidth(LineWeight& weight, qreal& width) const
+void KoCharacterStyle::strikeOutWidth(LineWeight &weight, qreal &width) const
 {
     weight = (KoCharacterStyle::LineWeight) d->propertyInt(KoCharacterStyle::StrikeOutWeight);
     width = d->propertyDouble(KoCharacterStyle::StrikeOutWidth);
@@ -558,7 +604,7 @@ void KoCharacterStyle::setStrikeOutMode(LineMode lineMode)
     d->setProperty(StrikeOutMode, lineMode);
 }
 
-void KoCharacterStyle::setStrikeOutText(const QString& text)
+void KoCharacterStyle::setStrikeOutText(const QString &text)
 {
     d->setProperty(StrikeOutText, text);
 }
@@ -607,7 +653,7 @@ void KoCharacterStyle::setUnderlineWidth(LineWeight weight, qreal width)
     d->setProperty(KoCharacterStyle::UnderlineWidth, width);
 }
 
-void KoCharacterStyle::underlineWidth(LineWeight& weight, qreal& width) const
+void KoCharacterStyle::underlineWidth(LineWeight &weight, qreal &width) const
 {
     weight = (KoCharacterStyle::LineWeight) d->propertyInt(KoCharacterStyle::UnderlineWeight);
     width = d->propertyDouble(KoCharacterStyle::UnderlineWidth);
@@ -656,7 +702,7 @@ QFont::Capitalization KoCharacterStyle::fontCapitalization() const
 
 void KoCharacterStyle::setCountry(const QString &country)
 {
-    if (country.isNull())
+    if (country.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Country);
     else
         d->setProperty(KoCharacterStyle::Country, country);
@@ -664,7 +710,7 @@ void KoCharacterStyle::setCountry(const QString &country)
 
 void KoCharacterStyle::setLanguage(const QString &language)
 {
-    if (language.isNull())
+    if (language.isEmpty())
         d->stylesPrivate.remove(KoCharacterStyle::Language);
     else
         d->setProperty(KoCharacterStyle::Language, language);
@@ -685,8 +731,78 @@ bool KoCharacterStyle::hasProperty(int key) const
     return d->stylesPrivate.contains(key);
 }
 
-//in 1.6 this was defined in KoTextFormat::load(KoOasisContext& context)
-void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
+static KoCharacterStyle::RotationAngle intToRotationAngle(int angle)
+{
+    KoCharacterStyle::RotationAngle rotationAngle = KoCharacterStyle::Zero;
+    if (angle == 90) {
+        rotationAngle = KoCharacterStyle::Ninety;
+    } else if (angle == 270) {
+        rotationAngle = KoCharacterStyle::TwoHundredSeventy;
+    }
+    return rotationAngle;
+}
+
+static int rotationAngleToInt(KoCharacterStyle::RotationAngle rotationAngle)
+{
+    int angle = 0;
+    if (rotationAngle == KoCharacterStyle::Ninety) {
+        angle = 90;
+    } else if (rotationAngle == KoCharacterStyle::TwoHundredSeventy) {
+        angle = 270;
+    }
+    return angle;
+}
+
+static QString rotationScaleToString(KoCharacterStyle::RotationScale rotationScale)
+{
+    QString scale = "line-height";
+    if (rotationScale == KoCharacterStyle::Fixed) {
+        scale = "fixed";
+    }
+    return scale;
+}
+
+static KoCharacterStyle::RotationScale stringToRotationScale(const QString &scale)
+{
+    KoCharacterStyle::RotationScale rotationScale = KoCharacterStyle::LineHeight;
+    if (scale == "fixed") {
+        rotationScale = KoCharacterStyle::Fixed;
+    }
+    return rotationScale;
+}
+
+void KoCharacterStyle::setTextRotationAngle(RotationAngle angle)
+{
+    d->setProperty(TextRotationAngle, rotationAngleToInt(angle));
+}
+
+KoCharacterStyle::RotationAngle KoCharacterStyle::textRotationAngle() const
+{
+    return intToRotationAngle(d->propertyInt(TextRotationAngle));
+}
+
+void KoCharacterStyle::setTextRotationScale(RotationScale scale)
+{
+    d->setProperty(TextRotationScale, rotationScaleToString(scale));
+}
+
+KoCharacterStyle::RotationScale KoCharacterStyle::textRotationScale() const
+{
+    return stringToRotationScale(d->propertyString(TextRotationScale));
+}
+
+void KoCharacterStyle::setTextScale(int scale)
+{
+    d->setProperty(TextScale, scale);
+}
+
+int KoCharacterStyle::textScale() const
+{
+    return d->propertyInt(TextScale);
+}
+
+//in 1.6 this was defined in KoTextFormat::load(KoOasisContext &context)
+void KoCharacterStyle::loadOdf(KoOdfLoadingContext &context)
 {
     d->setApplicationDefaults(context);
     KoStyleStack &styleStack = context.styleStack();
@@ -700,7 +816,7 @@ void KoCharacterStyle::loadOdf(KoOdfLoadingContext& context)
         if (fontFace != 0)
             fontName = fontFace->attributeNS(KoXmlNS::svg, "font-family", "");
     }
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
     // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -718,12 +834,11 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
 {
     // The fo:color attribute specifies the foreground color of text.
     if (styleStack.hasProperty(KoXmlNS::fo, "color")) {     // 3.10.3
-        if (styleStack.property(KoXmlNS::style, "use-window-font-color") != "true") {
-            QColor color(styleStack.property(KoXmlNS::fo, "color"));   // #rrggbb format
-            if (color.isValid()) {
-                setForeground(QBrush(color));
-            }
-        }
+        QColor color(styleStack.property(KoXmlNS::fo, "color"));   // #rrggbb format
+        if (color.isValid())
+            setForeground(QBrush(color));
+        // if (styleStack.property(KoXmlNS::style, "use-window-font-color") == "true")
+            // we should store this property to allow the layout to ignore the above set color in some situations.
     }
 
     QString fontName;
@@ -765,7 +880,7 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
     if (styleStack.hasProperty(KoXmlNS::style, "font-family"))
         fontName = styleStack.property(KoXmlNS::style, "font-family");
 
-    if (! fontName.isNull()) {
+    if (! fontName.isEmpty()) {
         // Hmm, the remove "'" could break it's in the middle of the fontname...
         fontName = fontName.remove('\'');
 
@@ -926,6 +1041,16 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
             setVerticalAlignment(QTextCharFormat::AlignSuperScript);
         else if (textPosition.startsWith("sub"))
             setVerticalAlignment(QTextCharFormat::AlignSubScript);
+        else {
+            QRegExp re("(-?[\\d.]+)%.*");
+            if (re.exactMatch(textPosition)) {
+                float value = re.capturedTexts()[1].toFloat();
+                if (value > 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSuperScript);
+                else if (value < 0)
+                    setVerticalAlignment(QTextCharFormat::AlignSubScript);
+            }
+        }
     }
 
     // The fo:font-variant attribute provides the option to display text as small capitalized letters.
@@ -971,14 +1096,18 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
     if (styleStack.hasProperty(KoXmlNS::style, "use-window-font-color")) {
         if (styleStack.property(KoXmlNS::style, "use-window-font-color") == "true") {
             // Do like OpenOffice.org : change the foreground font if its color is too close to the background color...
+            
             QColor back = background().color();
             QColor front = foreground().color();
-            if ((abs(qGray(back.rgb()) - qGray(front.rgb())) < 10) && (background().style() != Qt::NoBrush) && (foreground().style() != Qt::NoBrush)) {
+            if ((abs(qGray(back.rgb()) - qGray(front.rgb())) < 10) && (background().style() != Qt::NoBrush)) {
                 front.setRed(255 - front.red());
                 front.setGreen(255 - front.green());
                 front.setBlue(255 - front.blue());
                 QBrush frontBrush = foreground();
                 frontBrush.setColor(front);
+                if (frontBrush.style() == Qt::NoBrush) {
+                    frontBrush.setStyle(Qt::SolidPattern);
+                }
                 setForeground(frontBrush);
             }
         }
@@ -992,6 +1121,12 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
         }
     }
 
+    if (styleStack.hasProperty(KoXmlNS::fo, "letter-spacing")) {
+        qreal space = KoUnit::parseValue(styleStack.property(KoXmlNS::fo, "letter-spacing"));
+        QFontMetrics fm(font());
+        setFontLetterSpacing(100+100*space/fm.averageCharWidth());
+    }
+
     if (styleStack.hasProperty(KoXmlNS::style, "text-outline")) {
         if (styleStack.property(KoXmlNS::style, "text-outline") == "true") {
             setTextOutline(QPen((foreground().style() != Qt::NoBrush)?foreground():QBrush(Qt::black) , 0));
@@ -1001,6 +1136,21 @@ void KoCharacterStyle::loadOdfProperties(KoStyleStack &styleStack)
         }
     }
 
+    if (styleStack.hasProperty(KoXmlNS::style, "text-rotation-angle")) {
+        int angle = styleStack.property(KoXmlNS::style, "text-rotation-angle").toInt();
+        setTextRotationAngle(intToRotationAngle(angle));
+    }
+
+    if (styleStack.hasProperty(KoXmlNS::style, "text-rotation-scale")) {
+        const QString scale = styleStack.property(KoXmlNS::style, "text-rotation-scale");
+        setTextRotationScale(stringToRotationScale(scale));
+    }
+
+    if (styleStack.hasProperty(KoXmlNS::style, "text-scale")) {
+        const QString scaleString(styleStack.property(KoXmlNS::style, "text-scale"));
+        const int scale = (scaleString.endsWith('%') ? scaleString.left(scaleString.length()-1) : scaleString).toInt();
+        setTextScale(scale);
+    }
 
 //TODO
 #if 0
@@ -1056,7 +1206,7 @@ bool KoCharacterStyle::isEmpty() const
 
 void KoCharacterStyle::saveOdf(KoGenStyle &style)
 {
-    if (!d->name.isEmpty()) {
+    if (!d->name.isEmpty() && !style.isDefaultStyle()) {
         style.addAttribute("style:display-name", d->name);
     }
     QList<int> keys = d->stylesPrivate.keys();
@@ -1065,13 +1215,13 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
             bool ok = false;
             int boldness = d->stylesPrivate.value(key).toInt(&ok);
             if (ok) {
-                if (boldness == 50) {
+                if (boldness == QFont::Normal) {
                     style.addProperty("fo:font-weight", "normal", KoGenStyle::TextType);
-                } else if (boldness == 75) {
+                } else if (boldness == QFont::Bold) {
                     style.addProperty("fo:font-weight", "bold", KoGenStyle::TextType);
                 } else {
-                    // Remember : Qt and CSS/XSL doesn't have the same scale...
-                    style.addProperty("fo:font-weight", boldness*10, KoGenStyle::TextType);
+                    // Remember : Qt and CSS/XSL doesn't have the same scale. Its 100-900 instead of Qts 0-100
+                    style.addProperty("fo:font-weight", qBound(10, boldness, 90) * 10, KoGenStyle::TextType);
                 }
             }
         } else if (key == QTextFormat::FontItalic) {
@@ -1184,11 +1334,22 @@ void KoCharacterStyle::saveOdf(KoGenStyle &style)
             style.addProperty("fo:country", d->stylesPrivate.value(KoCharacterStyle::Country).toString(), KoGenStyle::TextType);
         } else if (key == KoCharacterStyle::Language) {
             style.addProperty("fo:language", d->stylesPrivate.value(KoCharacterStyle::Language).toString(), KoGenStyle::TextType);
+        } else if (key == QTextCharFormat::FontLetterSpacing) {
+            style.addProperty("fo:letter-spacing", (int) fontLetterSpacing(), KoGenStyle::TextType);
         } else if (key == QTextFormat::TextOutline) {
             QPen outline = textOutline();
             style.addProperty("style:text-outline", outline.style() == Qt::NoPen ? "false" : "true", KoGenStyle::TextType);
         } else if (key == KoCharacterStyle::FontCharset) {
             style.addProperty("style:font-charset", d->stylesPrivate.value(KoCharacterStyle::FontCharset).toString(), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextRotationAngle) {
+            RotationAngle angle = textRotationAngle();
+            style.addProperty("style:text-rotation-angle", rotationAngleToInt(angle), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextRotationScale) {
+            RotationScale scale = textRotationScale();
+            style.addProperty("style:text-rotation-scale", rotationScaleToString(scale), KoGenStyle::TextType);
+        } else if (key == KoCharacterStyle::TextScale) {
+            int scale = textScale();
+            style.addProperty("style:text-scale", QString::number(scale) + '%', KoGenStyle::TextType);
         }
     }
     //TODO: font name and family
@@ -1199,4 +1360,9 @@ QVariant KoCharacterStyle::value(int key) const
     return d->stylesPrivate.value(key);
 }
 
-#include "KoCharacterStyle.moc"
+void KoCharacterStyle::removeHardCodedDefaults()
+{
+    d->hardCodedDefaultStyle.clearAll();
+}
+
+#include <KoCharacterStyle.moc>
