@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2004-2006 David Faure <faure@kde.org>
    Copyright (C) 2007 Jan Hambrecht <jaham@gmx.net>
-   Copyright (C) 2007-2008 Thorsten Zachmann <zachmann@kde.org>
+   Copyright (C) 2007-2008,2010 Thorsten Zachmann <zachmann@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -38,7 +38,6 @@
 
 #include "KoOdfLoadingContext.h"
 #include "KoOdfStylesReader.h"
-#include "kopicture/KoPictureShared.h"
 
 void KoOdfGraphicStyles::saveOdfFillStyle(KoGenStyle &styleFill, KoGenStyles& mainStyles, const QBrush & brush)
 {
@@ -119,7 +118,7 @@ void KoOdfGraphicStyles::saveOdfStrokeStyle(KoGenStyle &styleStroke, KoGenStyles
     default: { // must be a dashed line
         styleStroke.addProperty("draw:stroke", "dash");
         // save stroke dash (14.14.7) which is severly limited, but still
-        KoGenStyle dashStyle(KoGenStyle::StyleStrokeDash);
+        KoGenStyle dashStyle(KoGenStyle::StrokeDashStyle);
         dashStyle.addAttribute("draw:style", "rect");
         QVector<qreal> dashes = pen.dashPattern();
         dashStyle.addAttribute("draw:dots1", static_cast<int>(1));
@@ -129,7 +128,7 @@ void KoOdfGraphicStyles::saveOdfStrokeStyle(KoGenStyle &styleStroke, KoGenStyles
             dashStyle.addAttribute("draw:dots2", static_cast<int>(1));
             dashStyle.addAttributePt("draw:dots2-length", dashes[2]*pen.widthF());
         }
-        QString dashStyleName = mainStyles.lookup(dashStyle, "dash");
+        QString dashStyleName = mainStyles.insert(dashStyle, "dash");
         styleStroke.addProperty("draw:stroke-dash", dashStyleName);
         break;
     }
@@ -163,7 +162,7 @@ void KoOdfGraphicStyles::saveOdfStrokeStyle(KoGenStyle &styleStroke, KoGenStyles
 
 QString KoOdfGraphicStyles::saveOdfHatchStyle(KoGenStyles& mainStyles, const QBrush &brush)
 {
-    KoGenStyle hatchStyle(KoGenStyle::StyleHatch /*no family name*/);
+    KoGenStyle hatchStyle(KoGenStyle::HatchStyle /*no family name*/);
     hatchStyle.addAttribute("draw:color", brush.color().name());
     //hatchStyle.addAttribute( "draw:distance", m_distance ); not implemented into kpresenter
     switch (brush.style()) {
@@ -195,7 +194,7 @@ QString KoOdfGraphicStyles::saveOdfHatchStyle(KoGenStyles& mainStyles, const QBr
         break;
     }
 
-    return mainStyles.lookup(hatchStyle, "hatch");
+    return mainStyles.insert(hatchStyle, "hatch");
 }
 
 QString KoOdfGraphicStyles::saveOdfGradientStyle(KoGenStyles &mainStyles, const QBrush &brush)
@@ -203,24 +202,24 @@ QString KoOdfGraphicStyles::saveOdfGradientStyle(KoGenStyles &mainStyles, const
     KoGenStyle gradientStyle;
     if (brush.style() == Qt::RadialGradientPattern) {
         const QRadialGradient *gradient = static_cast<const QRadialGradient*>(brush.gradient());
-        gradientStyle = KoGenStyle(KoGenStyle::StyleGradientRadial /*no family name*/);
-        gradientStyle.addAttributePt("svg:cx", gradient->center().x());
-        gradientStyle.addAttributePt("svg:cy", gradient->center().y());
-        gradientStyle.addAttributePt("svg:r",  gradient->radius());
-        gradientStyle.addAttributePt("svg:fx", gradient->focalPoint().x());
-        gradientStyle.addAttributePt("svg:fy", gradient->focalPoint().y());
+        gradientStyle = KoGenStyle(KoGenStyle::RadialGradientStyle /*no family name*/);
+        gradientStyle.addAttribute("svg:cx", QString("%1%").arg(gradient->center().x() * 100));
+        gradientStyle.addAttribute("svg:cy", QString("%1%").arg(gradient->center().y() * 100));
+        gradientStyle.addAttribute("svg:r",  QString("%1%").arg(gradient->radius() * 100));
+        gradientStyle.addAttribute("svg:fx", QString("%1%").arg(gradient->focalPoint().x() * 100));
+        gradientStyle.addAttribute("svg:fy", QString("%1%").arg(gradient->focalPoint().y() * 100));
     } else if (brush.style() == Qt::LinearGradientPattern) {
         const QLinearGradient *gradient = static_cast<const QLinearGradient*>(brush.gradient());
-        gradientStyle = KoGenStyle(KoGenStyle::StyleGradientLinear /*no family name*/);
-        gradientStyle.addAttributePt("svg:x1", gradient->start().x());
-        gradientStyle.addAttributePt("svg:y1", gradient->start().y());
-        gradientStyle.addAttributePt("svg:x2", gradient->finalStop().x());
-        gradientStyle.addAttributePt("svg:y2", gradient->finalStop().y());
+        gradientStyle = KoGenStyle(KoGenStyle::LinearGradientStyle /*no family name*/);
+        gradientStyle.addAttribute("svg:x1", QString("%1%").arg(gradient->start().x() * 100));
+        gradientStyle.addAttribute("svg:y1", QString("%1%").arg(gradient->start().y() * 100));
+        gradientStyle.addAttribute("svg:x2", QString("%1%").arg(gradient->finalStop().x() * 100));
+        gradientStyle.addAttribute("svg:y2", QString("%1%").arg(gradient->finalStop().y() * 100));
     } else if (brush.style() == Qt::ConicalGradientPattern) {
         const QConicalGradient * gradient = static_cast<const QConicalGradient*>(brush.gradient());
-        gradientStyle = KoGenStyle(KoGenStyle::StyleGradientConical /*no family name*/);
-        gradientStyle.addAttributePt("svg:cx", gradient->center().x());
-        gradientStyle.addAttributePt("svg:cy", gradient->center().y());
+        gradientStyle = KoGenStyle(KoGenStyle::ConicalGradientStyle /*no family name*/);
+        gradientStyle.addAttribute("svg:cx", QString("%1%").arg(gradient->center().x() * 100));
+        gradientStyle.addAttribute("svg:cy", QString("%1%").arg(gradient->center().y() * 100));
         gradientStyle.addAttribute("draw:angle", QString("%1").arg(gradient->angle()));
     }
     const QGradient * gradient = brush.gradient();
@@ -253,7 +252,7 @@ QString KoOdfGraphicStyles::saveOdfGradientStyle(KoGenStyles &mainStyles, const
     QString elementContents = QString::fromUtf8(buffer.buffer(), buffer.buffer().size());
     gradientStyle.addChildElement("svg:stop", elementContents);
 
-    return mainStyles.lookup(gradientStyle, "gradient");
+    return mainStyles.insert(gradientStyle, "gradient");
 }
 
 QBrush KoOdfGraphicStyles::loadOdfGradientStyle(const KoStyleStack &styleStack, const KoOdfStylesReader & stylesReader, const QSizeF &size)
@@ -262,7 +261,26 @@ QBrush KoOdfGraphicStyles::loadOdfGradientStyle(const KoStyleStack &styleStack,
     return loadOdfGradientStyleByName(stylesReader, styleName, size);
 }
 
-QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader & stylesReader, const QString styleName, const QSizeF &size)
+qreal percent(const KoXmlElement &element, const char *ns, const char *type, const QString &defaultValue, qreal absolute)
+{
+    qreal tmp = 0.0;
+    QString value = element.attributeNS(ns, type, defaultValue);
+    if (value.indexOf('%') > -1) { // percent value
+        tmp = value.remove('%').toDouble() / 100.0;
+    }
+    else { // fixed value
+        tmp = KoUnit::parseValue(value) / absolute;
+        // The following is done so that we get the same data as when we save/load.
+        // This is needed that we get the same values due to rounding differences
+        // of absolute and relative values.
+        QString value = QString("%1").arg(tmp * 100.0);
+        tmp = value.toDouble() / 100;
+    }
+
+    return tmp;
+}
+
+QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader &stylesReader, const QString &styleName, const QSizeF &size)
 {
     KoXmlElement* e = stylesReader.drawStyles()[styleName];
     if (! e)
@@ -279,26 +297,27 @@ QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader &
         // so what should we do?
         QString type = e->attributeNS(KoXmlNS::draw, "style", QString());
         if (type == "radial") {
-            QRadialGradient * rg = new QRadialGradient();
-            // TODO : find out whether Oasis works with boundingBox only?
+            // Zagge: at the moment the only objectBoundingBox is supported:
+            // 18.539 svg:gradientUnits
+            // See §13.2.2 and §13.2.3 of [SVG].
+            // The default value for this attribute is objectBoundingBox.
+            // The only value of the svg:gradientUnits attribute is objectBoundingBox.
+
             qreal cx = KoUnit::parseValue(e->attributeNS(KoXmlNS::draw, "cx", QString()).remove('%'));
             qreal cy = KoUnit::parseValue(e->attributeNS(KoXmlNS::draw, "cy", QString()).remove('%'));
-            rg->setCenter(QPointF(size.width() * 0.01 * cx, size.height() * 0.01 * cy));
-            rg->setFocalPoint(rg->center());
-            qreal dx = 0.5 * size.width();
-            qreal dy = 0.5 * size.height();
-            rg->setRadius(sqrt(dx*dx + dy*dy));
-            gradient = rg;
+            gradient = new QRadialGradient(QPointF(cx * 0.01, cy * 0.01), sqrt(0.5));
+            gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
         } else if (type == "linear" || type == "axial") {
             QLinearGradient * lg = new QLinearGradient();
+            lg->setCoordinateMode(QGradient::ObjectBoundingMode);
             // Dividing by 10 here because OOo saves as degree * 10
             qreal angle = 90 + e->attributeNS(KoXmlNS::draw, "angle", "0").toDouble() / 10;
-            qreal radius = 0.5 * sqrt(size.width() * size.width() + size.height() * size.height());
+            qreal radius = sqrt(0.5);
 
             qreal sx = cos(angle * M_PI / 180) * radius;
             qreal sy = sin(angle * M_PI / 180) * radius;
-            lg->setStart(QPointF(0.5 * size.width() + sx, 0.5 * size.height() - sy));
-            lg->setFinalStop(QPointF(0.5 * size.width() - sx, 0.5 * size.height() + sy));
+            lg->setStart(QPointF(0.5 + sx, 0.5 - sy));
+            lg->setFinalStop(QPointF(0.5 - sx, 0.5 + sy));
             gradient = lg;
         } else
             return QBrush();
@@ -341,28 +360,22 @@ QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader &
         gradient->setStops(stops);
     } else if (e->namespaceURI() == KoXmlNS::svg) {
         if (e->localName() == "linearGradient") {
-            QLinearGradient * lg = new QLinearGradient();
             QPointF start, stop;
-            start.setX(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "x1", QString())));
-            start.setY(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "y1", QString())));
-            stop.setX(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "x2", QString())));
-            stop.setY(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "y2", QString())));
-            lg->setStart(start);
-            lg->setFinalStop(stop);
-            gradient = lg;
+            start.setX(percent(*e, KoXmlNS::svg, "x1", "0%", size.width()));
+            start.setY(percent(*e, KoXmlNS::svg, "y1", "0%", size.height()));
+            stop.setX(percent(*e, KoXmlNS::svg, "x2", "100%", size.width()));
+            stop.setY(percent(*e, KoXmlNS::svg, "y2", "100%", size.height()));
+            gradient = new QLinearGradient(start, stop);
         } else if (e->localName() == "radialGradient") {
-            QRadialGradient * rg = new QRadialGradient();
             QPointF center, focalPoint;
-            center.setX(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "cx", QString())));
-            center.setY(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "cy", QString())));
-            qreal r = KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "r", QString()));
-            focalPoint.setX(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "fx", QString())));
-            focalPoint.setY(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "fy", QString())));
-            rg->setCenter(center);
-            rg->setFocalPoint(focalPoint);
-            rg->setRadius(r);
-            gradient = rg;
+            center.setX(percent(*e, KoXmlNS::svg, "cx", "50%", size.width()));
+            center.setY(percent(*e, KoXmlNS::svg, "cy", "50%", size.height()));
+            qreal r = percent(*e, KoXmlNS::svg, "r", "50%", sqrt(size.width() * size.width() + size.height() * size.height()));
+            focalPoint.setX(percent(*e, KoXmlNS::svg, "fx", QString(), size.width()));
+            focalPoint.setY(percent(*e, KoXmlNS::svg, "fy", QString(), size.height()));
+            gradient = new QRadialGradient(center, r, focalPoint );
         }
+        gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
 
         QString strSpread(e->attributeNS(KoXmlNS::svg, "spreadMethod", "pad"));
         if (strSpread == "repeat")
@@ -392,13 +405,11 @@ QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader &
     } else if (e->namespaceURI() == KoXmlNS::koffice) {
         if (e->localName() == "conicalGradient") {
             QPointF center;
-            center.setX(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "cx", QString())));
-            center.setY(KoUnit::parseValue(e->attributeNS(KoXmlNS::svg, "cy", QString())));
+            center.setX(percent(*e, KoXmlNS::svg, "cx", "50%", size.width()));
+            center.setY(percent(*e, KoXmlNS::svg, "cy", "50%", size.height()));
             qreal angle = KoUnit::parseValue(e->attributeNS(KoXmlNS::draw, "angle", QString()));
-            QConicalGradient * g = new QConicalGradient();
-            g->setCenter(center);
-            g->setAngle(angle);
-            gradient = g;
+            gradient = new QConicalGradient(center, angle);
+            gradient->setCoordinateMode(QGradient::ObjectBoundingMode);
 
             QString strSpread(e->attributeNS(KoXmlNS::svg, "spreadMethod", "pad"));
             if (strSpread == "repeat")
@@ -438,113 +449,10 @@ QBrush KoOdfGraphicStyles::loadOdfGradientStyleByName(const KoOdfStylesReader &
     return resultBrush;
 }
 
-QBrush KoOdfGraphicStyles::loadOdfPatternStyle(const KoStyleStack &styleStack, KoOdfLoadingContext & context, const QSizeF &size)
-{
-    QString styleName = styleStack.property(KoXmlNS::draw, "fill-image-name");
-
-    KoXmlElement* e = context.stylesReader().drawStyles()[styleName];
-    if (! e)
-        return QBrush();
-
-    const QString href = e->attributeNS(KoXmlNS::xlink, "href", QString());
-
-    if (href.isEmpty())
-        return QBrush();
-
-    QString strExtension;
-    const int result = href.lastIndexOf(".");
-    if (result >= 0) {
-        strExtension = href.mid(result + 1); // As we are using KoPicture, the extension should be without the dot.
-    }
-    QString filename(href);
-
-    KoPictureShared picture;
-    KoStore* store = context.store();
-    if (store->open(filename)) {
-        KoStoreDevice dev(store);
-        if (! picture.load(&dev, strExtension))
-            kWarning() << "Cannot load picture: " << filename << " " << href;
-        store->close();
-    }
-
-    // read the pattern repeat style
-    QString style = styleStack.property(KoXmlNS::style, "repeat");
-    kDebug(30003) << "pattern style =" << style;
-
-    QSize imageSize = picture.getOriginalSize();
-
-    if (style == "stretch") {
-        imageSize = size.toSize();
-    } else {
-        // optional attributes which can override original image size
-        if (styleStack.hasProperty(KoXmlNS::draw, "fill-image-height") && styleStack.hasProperty(KoXmlNS::draw, "fill-image-width")) {
-            QString height = styleStack.property(KoXmlNS::draw, "fill-image-height");
-            qreal newHeight = 0.0;
-            if (height.endsWith('%'))
-                newHeight = 0.01 * height.remove('%').toDouble() * imageSize.height();
-            else
-                newHeight = KoUnit::parseValue(height);
-            QString width = styleStack.property(KoXmlNS::draw, "fill-image-width");
-            qreal newWidth = 0.0;
-            if (width.endsWith('%'))
-                newWidth = 0.01 * width.remove('%').toDouble() * imageSize.width();
-            else
-                newWidth = KoUnit::parseValue(width);
-            if (newHeight > 0.0)
-                imageSize.setHeight(static_cast<int>(newHeight));
-            if (newWidth > 0.0)
-                imageSize.setWidth(static_cast<int>(newWidth));
-        }
-    }
-
-    kDebug(30003) << "shape size =" << size;
-    kDebug(30003) << "original image size =" << picture.getOriginalSize();
-    kDebug(30003) << "resulting image size =" << imageSize;
-
-    QBrush resultBrush(picture.generatePixmap(imageSize, true));
-
-    if (style == "repeat") {
-        QMatrix matrix;
-        if (styleStack.hasProperty(KoXmlNS::draw, "fill-image-ref-point")) {
-            // align pattern to the given size
-            QString align = styleStack.property(KoXmlNS::draw, "fill-image-ref-point");
-            kDebug(30003) << "pattern align =" << align;
-            if (align == "top-left")
-                matrix.translate(0, 0);
-            else if (align == "top")
-                matrix.translate(0.5*size.width(), 0);
-            else if (align == "top-right")
-                matrix.translate(size.width(), 0);
-            else if (align == "left")
-                matrix.translate(0, 0.5*size.height());
-            else if (align == "center")
-                matrix.translate(0.5*size.width(), 0.5*size.height());
-            else if (align == "right")
-                matrix.translate(size.width(), 0.5*size.height());
-            else if (align == "bottom-left")
-                matrix.translate(0, size.height());
-            else if (align == "bottom")
-                matrix.translate(0.5*size.width(), size.height());
-            else if (align == "bottom-right")
-                matrix.translate(size.width(), size.height());
-        }
-        if (styleStack.hasProperty(KoXmlNS::draw, "fill-image-ref-point-x")) {
-            QString pointX = styleStack.property(KoXmlNS::draw, "fill-image-ref-point-x");
-            matrix.translate(0.01 * pointX.remove('%').toDouble() * imageSize.width(), 0);
-        }
-        if (styleStack.hasProperty(KoXmlNS::draw, "fill-image-ref-point-y")) {
-            QString pointY = styleStack.property(KoXmlNS::draw, "fill-image-ref-point-y");
-            matrix.translate(0, 0.01 * pointY.remove('%').toDouble() * imageSize.height());
-        }
-        resultBrush.setMatrix(matrix);
-    }
-
-    return resultBrush;
-}
-
 QBrush KoOdfGraphicStyles::loadOdfFillStyle(const KoStyleStack &styleStack, const QString & fill,  const KoOdfStylesReader & stylesReader)
 {
-    QBrush tmpBrush;
+    QBrush tmpBrush; // default brush for "none" is a Qt::NoBrush
+
     if (fill == "solid") {
         tmpBrush.setStyle(static_cast<Qt::BrushStyle>(1));
         if (styleStack.hasProperty(KoXmlNS::draw, "fill-color"))
@@ -633,7 +541,7 @@ QBrush KoOdfGraphicStyles::loadOdfFillStyle(const KoStyleStack &styleStack, cons
                         kDebug(30003) << " draw:rotation 'angle' :" << angle;
                         break;
                     }
-                } else if (styleHash == "qreal") {
+                } else if (styleHash == "double") {
                     switch (angle) {
                     case 0:
                     case 180:
@@ -665,9 +573,13 @@ QBrush KoOdfGraphicStyles::loadOdfFillStyle(const KoStyleStack &styleStack, cons
 
 QPen KoOdfGraphicStyles::loadOdfStrokeStyle(const KoStyleStack &styleStack, const QString & stroke, const KoOdfStylesReader & stylesReader)
 {
-    QPen tmpPen;
+    QPen tmpPen(Qt::NoPen); // default pen for "none" is a Qt::NoPen
 
     if (stroke == "solid" || stroke == "dash") {
+        // If solid or dash is set then we assume that the color is black and the penWidth
+        // is zero till defined otherwise with the following attributes.
+        tmpPen = QPen();
+
         if (styleStack.hasProperty(KoXmlNS::svg, "stroke-color"))
             tmpPen.setColor(styleStack.property(KoXmlNS::svg, "stroke-color"));
         if (styleStack.hasProperty(KoXmlNS::svg, "stroke-opacity")) {
