@@ -26,36 +26,67 @@
 
 #define PICTURESHAPEID "PictureShape"
 
-class KoImageData;
 class KoImageCollection;
-class KUrl;
+class RenderQueue;
 
 class PictureShape : public KoShape, public KoFrameShape
 {
 public:
-    explicit PictureShape();
+    enum PictureMode {
+        Standard,
+        Greyscale,
+        Mono,
+        Watermark
+    };
+
+    PictureShape();
     virtual ~PictureShape();
 
     // reimplemented
-    virtual void paint( QPainter& painter, const KoViewConverter& converter );
+    virtual void paint(QPainter &painter, const KoViewConverter &converter);
     // reimplemented
-    virtual void saveOdf( KoShapeSavingContext & context ) const;
+    virtual void saveOdf(KoShapeSavingContext &context) const;
     // reimplemented
-    virtual bool loadOdf( const KoXmlElement & element, KoShapeLoadingContext &context );
+    virtual bool loadOdf(const KoXmlElement &element, KoShapeLoadingContext &context);
+    // reimplemented
+    virtual void waitUntilReady(const KoViewConverter &converter, bool asynchronous) const;
 
-    /// Load data from a file - data will be saved in odf - the referenced file is not modified
-    bool loadFromUrl( KUrl & );
+    /**
+     * Get the collection used in the shape.
+     */
+    KoImageCollection *imageCollection() const;
 
-    /// reimplemented
-    void init(const QMap<QString, KoDataCenter *> & dataCenterMap);
+    void setImageCollection(KoImageCollection *collection) { m_imageCollection = collection; }
+
+    void setMode( PictureMode mode );
+    PictureMode mode() const;
 
 protected:
-    virtual bool loadOdfFrameElement( const KoXmlElement & element, KoShapeLoadingContext & context );
+    virtual bool loadOdfFrameElement(const KoXmlElement &element, KoShapeLoadingContext &context);
+
+    virtual void loadStyle(const KoXmlElement& element, KoShapeLoadingContext& context);
 
 private:
     KoImageCollection *m_imageCollection;
-    KoImageData *m_imageData;
+    RenderQueue *m_renderQueue;
+    mutable QImage m_printQualityImage;
+    PictureMode m_mode;
 };
 
+class RenderQueue : public QObject
+{
+    Q_OBJECT
+public:
+    RenderQueue(PictureShape *shape) : m_pictureShape(shape) { }
+
+    void addSize(const QSize &size) { m_wantedImageSize << size; }
+
+public slots:
+    void renderImage();
+
+private:
+    KoShape *m_pictureShape;
+    QList<QSize> m_wantedImageSize;
+};
 
 #endif
