@@ -51,13 +51,14 @@
 #include <QVBoxLayout>
 
 ChangeTrackingTool::ChangeTrackingTool(KoCanvasBase* canvas): KoToolBase(canvas),
+    m_disableShowChangesOnExit(false),
     m_textEditor(0),
     m_textShapeData(0),
+    m_canvas(canvas),
     m_textShape(0),
     m_model(0),
     m_trackedChangeManager(0),
-    m_changesTreeView(0),
-    m_canvas(canvas)
+    m_changesTreeView(0)
 {
     KAction *action;
     action = new KAction(i18n("Tracked change manager"), this);
@@ -86,17 +87,12 @@ void ChangeTrackingTool::mousePressEvent(KoPointerEvent* event)
 {
     if (event->button() != Qt::RightButton)
         updateSelectedShape(event->point);
-    KoSelection *selection = canvas()->shapeManager()->selection();
-    if (!selection->isSelected(m_textShape)) {
-        selection->deselectAll();
-        selection->select(m_textShape);
-    }
 
     int position = pointToPosition(event->point);
     QTextCursor cursor(m_textShapeData->document());
     cursor.setPosition(position);
 
-    KoChangeTracker *changeTracker = KoTextDocument(m_textShapeData->document()).changeTracker();
+    //KoChangeTracker *changeTracker = KoTextDocument(m_textShapeData->document()).changeTracker();
     int changeId = cursor.charFormat().property(KoCharacterStyle::ChangeTrackerId).toInt();
     QModelIndex index = m_model->indexForChangeId(changeId);
     m_changesTreeView->setCurrentIndex(index);
@@ -247,7 +243,6 @@ void ChangeTrackingTool::keyPressEvent(QKeyEvent* event)
 void ChangeTrackingTool::activate(ToolActivation toolActivation, const QSet<KoShape*> &shapes)
 {
     Q_UNUSED(toolActivation);
-    KoSelection *selection = canvas()->shapeManager()->selection();
     foreach(KoShape *shape, shapes) {
         m_textShape = dynamic_cast<TextShape*>(shape);
         if (m_textShape)
@@ -257,11 +252,6 @@ void ChangeTrackingTool::activate(ToolActivation toolActivation, const QSet<KoSh
         emit done();
         return;
     }
-    foreach(KoShape *shape, selection->selectedShapes()) {
-        // deselect others.
-        if (m_textShape == shape) continue;
-        selection->deselect(shape);
-    }
     setShapeData(static_cast<KoTextShapeData*>(m_textShape->userData()));
     useCursor(Qt::ArrowCursor);
 
@@ -272,12 +262,14 @@ void ChangeTrackingTool::activate(ToolActivation toolActivation, const QSet<KoSh
 void ChangeTrackingTool::setShapeData(KoTextShapeData *data)
 {
     bool docChanged = data == 0 || m_textShapeData == 0 || m_textShapeData->document() != data->document();
+/*
     if (m_textShapeData) {
 //        disconnect(m_textShapeData, SIGNAL(destroyed (QObject*)), this, SLOT(shapeDataRemoved()));
         KoTextDocumentLayout *lay = qobject_cast<KoTextDocumentLayout*>(m_textShapeData->document()->documentLayout());
 //        if (lay)
 //            disconnect(lay, SIGNAL(shapeAdded(KoShape*)), this, SLOT(shapeAddedToDoc(KoShape*)));
     }
+*/
     if (!data) {
         if (m_disableShowChangesOnExit) {
             ShowChangesCommand *command = new ShowChangesCommand(false, m_textShapeData->document(), m_canvas);
@@ -374,6 +366,7 @@ void ChangeTrackingTool::rejectChange()
 
 void ChangeTrackingTool::selectedChangeChanged(QModelIndex newItem, QModelIndex previousItem)
 {
+    Q_UNUSED(newItem);
     Q_UNUSED(previousItem);
     canvas()->updateCanvas(m_textShape->boundingRect());
 }
