@@ -18,94 +18,89 @@
 
 #include "EmfEnums.h"
 #include "EmfRecords.h"
+#include "Bitmap.h"
 
 #include <KDebug>
 
 namespace Libemf
 {
 
-/*****************************************************************************/
-
-BitmapInfoHeader::BitmapInfoHeader( QDataStream &stream )
-{
-    stream >> m_headerSize;
-    stream >> m_width;
-    //kDebug(31000) << "Width:" << m_width;
-    stream >> m_height;
-    //kDebug(31000) << "Height:" << m_height;
-    stream >> m_planes;
-    // kDebug(33100) << "planes:" << m_planes;
-    stream >> m_bitCount;
-    // kDebug(33100) << "BitCount:" << m_bitCount;
-    stream >> m_compression;
-    //kDebug(31000) << "Compression:" << m_compression;
-    stream >> m_imageSize;
-    // kDebug(33100) << "ImageSize:" << m_imageSize;
-    stream >> m_xPelsPerMeter;
-    stream >> m_yPelsPerMeter;
-    stream >> m_colorUsed;
-    stream >> m_colorImportant;
-}
-
-BitmapInfoHeader::~BitmapInfoHeader()
-{
-}
-
 
 /*****************************************************************************/
-BitBltRecord::BitBltRecord( QDataStream &stream )
+
+
+BitBltRecord::BitBltRecord( QDataStream &stream, quint32 recordSize )
+    : m_bitmap(0)
 {
-#if 0
-    qint32 x, y, width, height;
-    stream >> x >> y >> width >> height;
-    kDebug(31000) << "Bounds" << x << y << width << height;
-    m_Bounds = QRect( QPoint(x, y), QSize(width, height));
-#else
-    stream >> m_Bounds;
-#endif
-    stream >> m_xDest;
+    kDebug(31000) << "stream position at the start: " << stream.device()->pos();
+    kDebug(31000) << "record size: " << recordSize;
+    stream >> m_bounds;
+
+    stream >> m_xDest;          // x, y of upper left corner of the destination.
     stream >> m_yDest;
-    stream >> m_cxDest;
+    stream >> m_cxDest;         // width, height of the rectangle in logical coords.
     stream >> m_cyDest;
-    //kDebug(31000) << "Destination" << m_xDest << m_yDest << m_cxDest << m_cyDest;
+    kDebug(31000) << "Destination" << m_xDest << m_yDest << m_cxDest << m_cyDest;
 
     stream >> m_BitBltRasterOperation;
-    //kDebug(31000) << "bitblt raster operation:" << m_BitBltRasterOperation;
-    stream >> m_xSrc;
-    stream >> m_ySrc;
-    //kDebug(31000) << "Sourcd" << m_xSrc << m_ySrc;
+    kDebug(31000) << "bitblt raster operation:" << hex << m_BitBltRasterOperation << dec;
 
+    stream >> m_xSrc;           // x, y of the source
+    stream >> m_ySrc;
+    kDebug(31000) << "Source" << m_xSrc << m_ySrc;
+
+    kDebug(31000) << "position before the matrix: " << stream.device()->pos();
+#if 1 // it seems that the floats all use 8 bytes instead of the wanted 4.
+    stream.setFloatingPointPrecision(QDataStream::SinglePrecision);
     float M11, M12, M21, M22, Dx, Dy;
-    stream >> M11;
+    stream >> M11;              // Transformation matrix
     stream >> M12;
     stream >> M21;
     stream >> M22;
     stream >> Dx;
     stream >> Dy;
     m_XFormSrc = QMatrix( M11, M12, M21, M22, Dx, Dy );
-    //kDebug(31000) << m_XFormSrc;
+    kDebug(31000) << "Matrix" << m_XFormSrc;
+#else
+    quint32 M11, M12, M21, M22, Dx, Dy;
+    stream >> M11;              // Transformation matrix
+    stream >> M12;
+    stream >> M21;
+    stream >> M22;
+    stream >> Dx;
+    stream >> Dy;
+    //m_XFormSrc = QMatrix( M11, M12, M21, M22, Dx, Dy );
+#endif
+    kDebug(31000) << "position after the matrix: " << stream.device()->pos();
 
     stream >> m_red >> m_green >> m_blue >> m_reserved;
+    kDebug(31000) << "Background color" << m_red << m_green << m_blue << m_reserved;
+    kDebug(31000) << "position after background color: " << stream.device()->pos();
 
     stream >> m_UsageSrc;
-    stream >> m_offBmiSrc;
+    kDebug(31000) << "Color table interpretation" << m_UsageSrc;
+
+    stream >> m_offBmiSrc;      // Offset to start of bitmap header from start of record
     stream >> m_cbBmiSrc;       // Size of source bitmap header
-    stream >> m_offBitsSrc;
-    stream >> m_cbBitsSrc;      // Size of source bitmap itself
+    stream >> m_offBitsSrc;     // Offset to source bitmap from start of record
+    stream >> m_cbBitsSrc;      // Size of source bitmap
+#if 1
+    kDebug(31000) << "header offset:" << m_offBmiSrc;
+    kDebug(31000) << "header size:  " << m_cbBmiSrc;
+    kDebug(31000) << "bitmap offset:" << m_offBitsSrc;
+    kDebug(31000) << "bitmap size:  " << m_cbBitsSrc;
+#endif
 
-    // Some basic checks
-    if ( ( m_cbBmiSrc == 0 ) && ( m_cbBmiSrc == 0 ) ) {
-	return;
-    }
-    if ( m_cbBmiSrc == 40 ) {
-	m_BmiSrc = new BitmapInfoHeader( stream );
-    } else {
-	kDebug(31000) << "BUG!!! m_cbBmiSrc:" << m_cbBmiSrc;
-	//Q_ASSERT( 0 );
+    kDebug(31000) << "stream position before the image: " << stream.device()->pos();
+    if (m_cbBmiSrc > 0) {
+        m_bitmap = new Bitmap( stream, recordSize, 8 + 23 * 4, // header + 23 ints
+                               m_offBmiSrc, m_cbBmiSrc,
+                               m_offBitsSrc, m_cbBitsSrc );
     }
 
-    m_imageData.resize( m_cbBitsSrc );
-    stream.readRawData( m_imageData.data(), m_cbBitsSrc );
+    kDebug(31000) << "stream position at the end: " << stream.device()->pos();
+    //m_imageData.resize( m_cbBitsSrc );
+    //stream.readRawData( m_imageData.data(), m_cbBitsSrc );
 }
 
 BitBltRecord::~BitBltRecord()
@@ -114,12 +109,14 @@ BitBltRecord::~BitBltRecord()
 
 bool BitBltRecord::hasImage() const
 {
-    // FIXME: Check the same twice??
-    return ( ( m_cbBmiSrc != 0 ) && ( m_cbBmiSrc != 0 ) );
+    return m_bitmap && m_bitmap->hasImage();
+    //return ( ( m_cbBmiSrc != 0 ) && ( m_cbBitsSrc != 0 ) );
 }
 
-QImage* BitBltRecord::image() 
+QImage BitBltRecord::image() 
 {
+    return m_bitmap->image();
+#if 0
     if ( ! hasImage() ) {
         return 0;
     }
@@ -147,13 +144,16 @@ QImage* BitBltRecord::image()
                           m_BmiSrc->width(), m_BmiSrc->height(), format );
 
     return m_image;
+#endif
 }
 
 /*****************************************************************************/
-StretchDiBitsRecord::StretchDiBitsRecord( QDataStream &stream )
-    : m_BmiSrc(0)
-    , m_image(0)
+StretchDiBitsRecord::StretchDiBitsRecord( QDataStream &stream, quint32 recordSize )
+    : m_bitmap(0)
 {
+    kDebug(31000) << "stream position at the start: " << stream.device()->pos();
+    kDebug(31000) << "recordSize =" << recordSize;
+
     stream >> m_Bounds;
     stream >> m_xDest;
     stream >> m_yDest;
@@ -170,14 +170,24 @@ StretchDiBitsRecord::StretchDiBitsRecord( QDataStream &stream )
     stream >> m_cxDest;
     stream >> m_cyDest;
 
-#if 0
-    kDebug(31000) << "m_cbBmiSrc =" << m_cbBmiSrc;
-    kDebug(31000) << "m_offBmiSrc =" << m_offBmiSrc;
-    kDebug(31000) << "m_cbBitsSrc  =" << m_cbBitsSrc;
-    kDebug(31000) << "m_offBitsSrc =" << m_offBitsSrc;
-    //kDebug(31000) << "m_BitBltRasterOperation =" << hex << m_BitBltRasterOperation << dec;
+#if 1
+    kDebug(31000) << "header offset:" << m_offBmiSrc;
+    kDebug(31000) << "header size:  " << m_cbBmiSrc;
+    kDebug(31000) << "bitmap offset:" << m_offBitsSrc;
+    kDebug(31000) << "bitmap size:  " << m_cbBitsSrc;
+
+    kDebug(31000) << "m_BitBltRasterOperation =" << hex << m_BitBltRasterOperation << dec;
 #endif
 
+    kDebug(31000) << "stream position before the image: " << stream.device()->pos();
+    if (m_cbBmiSrc > 0) {
+        m_bitmap = new Bitmap( stream, recordSize, 8 + 18 * 4, // header + 18 ints
+                               m_offBmiSrc, m_cbBmiSrc,
+                               m_offBitsSrc, m_cbBitsSrc );
+    }
+
+    kDebug(31000) << "stream position at the end: " << stream.device()->pos();
+#if 0
     // Read away those bytes that preceed the header.  These are undefined
     // according to the spec.  80 is the size of the record above.
     qint32 dummy;
@@ -186,7 +196,7 @@ StretchDiBitsRecord::StretchDiBitsRecord( QDataStream &stream )
         stream >> dummy;
         padding += 4;
     }
-    m_BmiSrc = new BitmapInfoHeader( stream );
+    m_BmiSrc = new BitmapHeader( stream, m_cbBmiSrc );
 
     // 40 is the size of the header record.
     while (m_offBitsSrc - padding > 80 + 40) {
@@ -195,11 +205,13 @@ StretchDiBitsRecord::StretchDiBitsRecord( QDataStream &stream )
     }
     m_imageData.resize( m_cbBitsSrc );
     stream.readRawData( m_imageData.data(), m_cbBitsSrc );
+#endif
 }
 
 StretchDiBitsRecord::~StretchDiBitsRecord()
 {
-    delete m_image;
+    delete m_bitmap;
+    //delete m_image;
     // delete m_BmiSrc;
 }
 
@@ -208,13 +220,17 @@ QRect StretchDiBitsRecord::bounds() const
     return m_Bounds;
 }
 
-QImage* StretchDiBitsRecord::image() 
+QImage StretchDiBitsRecord::image() 
 {
+    return m_bitmap->image();
+#if 0
     if ( m_image != 0 ) {
         return m_image;
     }
 
     QImage::Format format = QImage::Format_Invalid;
+
+    // Start by determining which QImage format we are going to use.
     if (m_BmiSrc->bitCount() == BI_BITCOUNT_1) {
         format = QImage::Format_Mono;
     } else if ( m_BmiSrc->bitCount() == BI_BITCOUNT_4 ) {
@@ -238,7 +254,12 @@ QImage* StretchDiBitsRecord::image()
         // This bitmap is a top-down bitmap without compression.
         m_image = new QImage( (const uchar*)m_imageData.constData(),
                               m_BmiSrc->width(), m_BmiSrc->height(), format );
-        // we have to mirror this bitmap in the X axis.
+
+        // The WMF images are in the BGR color order.
+        if (format == QImage::Format_RGB888)
+            *m_image = m_image->rgbSwapped();
+
+        // We have to mirror this bitmap in the X axis since WMF images are stored bottom-up.
         *m_image = m_image->mirrored(false, true);
     } else {
         // This bitmap is a bottom-up bitmap which uses compression.
@@ -246,6 +267,8 @@ QImage* StretchDiBitsRecord::image()
         case BI_RGB:
             m_image = new QImage( (const uchar*)m_imageData.constData(),
                                   m_BmiSrc->width(), -m_BmiSrc->height(), format );
+            // The WMF images are in the BGR color order.
+            *m_image = m_image->rgbSwapped();
             break;
 
         // These compressions are not yet supported, so return an empty image.
@@ -264,6 +287,7 @@ QImage* StretchDiBitsRecord::image()
     }
 
     return m_image;
+#endif
 }
 
 /*****************************************************************************/
