@@ -22,6 +22,7 @@
  *
  */
 
+#include "OfficeInterface.h"
 #include "MainWindow.h"
 #include "ui_MainWindow.h"
 #include "Common.h"
@@ -110,6 +111,33 @@ void MainWindow::init()
     QMenuBar* menu = menuBar();
     menu->addAction(m_ui->actionOpen);
     menu->addAction(m_ui->actionAbout);
+    
+    // false here means that they are not plugins
+    m_ui->actionOpen->setData(QVariant(false));
+    m_ui->actionAbout->setData(QVariant(false));
+
+    const QDir pluginDir("/usr/lib/freoffice/");
+    const QStringList plugins = pluginDir.entryList(QDir::Files);
+
+    for (int i = 0; i < plugins.size(); ++i) {
+        QPluginLoader test(pluginDir.absoluteFilePath(plugins.at(i)));
+        QObject *plug = test.instance();
+        plug->setParent(this);
+        if (plug != 0)
+        {
+            OfficeInterface* inter = qobject_cast<OfficeInterface*>(plug);
+            const QString plugName = inter->pluginName();
+            loadedPlugins[plugName] = inter;
+            connect(plug, SIGNAL(openDocument(bool, const QString&)), this, SLOT(pluginOpen(bool, const QString&)));
+            QAction *action = new QAction(plugName, this);
+            
+            // True states that this action is a plugin            
+            action->setData(QVariant(true));
+            menu->addAction(action);
+        }
+    }
+
+    connect(menu, SIGNAL(triggered(QAction*)), this, SLOT(menuClicked(QAction*)));
 
     m_search = new QLineEdit(this);
     m_ui->SearchToolBar->insertWidget(m_ui->actionSearchOption, m_search);
@@ -136,7 +164,6 @@ void MainWindow::init()
     connect(m_ui->actionNextPage, SIGNAL(triggered()), this, SLOT(nextPage()));
     connect(m_ui->actionPrevPage, SIGNAL(triggered()), this, SLOT(prevPage()));
 
-    connect(m_ui->actionPanningOn, SIGNAL(triggered()), this, SLOT(toggleSelection()));
     m_ui->actionCopy->setEnabled(false);
     connect(m_ui->actionCopy, SIGNAL(triggered()), this, SLOT(copy()));
 
@@ -170,11 +197,22 @@ void MainWindow::init()
 
     m_copyShortcut = new QShortcut(QKeySequence::Copy, this);
     connect(m_copyShortcut, SIGNAL(activated()), this, SLOT(copy()));
+
+    // Toolbar should be shown only when we open a document
+    m_ui->viewToolBar->hide();
 }
 
 MainWindow::~MainWindow()
 {
     closeDocument();
+
+    QMapIterator<QString, OfficeInterface*> i(loadedPlugins);
+    while (i.hasNext())
+    {
+        i.next();
+        delete i.value();
+    }
+
     delete m_ui;
     m_ui = 0;
 }
@@ -311,6 +349,7 @@ void MainWindow::closeDocument()
 
     m_ui->actionZoomLevel->setText(i18n("%1 %", 100));
     m_ui->actionPageNumber->setText(i18n("%1 of %2", 0, 0));
+    m_ui->viewToolBar->hide();
 }
 
 void MainWindow::doOpenDocument()
@@ -334,6 +373,8 @@ void MainWindow::raiseWindow(void)
 
 void MainWindow::openDocument(const QString &fileName)
 {
+    m_ui->viewToolBar->show();
+
     if (!checkFiletype(fileName))
         return;
 
@@ -391,6 +432,8 @@ void MainWindow::openDocument(const QString &fileName)
 
     if (!QString::compare(ext, EXT_ODP, Qt::CaseInsensitive) ||
             !QString::compare(ext, EXT_PPTX, Qt::CaseInsensitive) ||
+            !QString::compare(ext, EXT_PPS, Qt::CaseInsensitive) ||
+            !QString::compare(ext, EXT_PPSX, Qt::CaseInsensitive) ||
             !QString::compare(ext, EXT_PPT, Qt::CaseInsensitive)) {
         m_type = Presentation;
     } else if (!QString::compare(ext, EXT_ODS, Qt::CaseInsensitive) ||
@@ -399,6 +442,8 @@ void MainWindow::openDocument(const QString &fileName)
         m_type = Spreadsheet;
     } else {
         m_type = Text;
+        // We need to get the page count again after layout rounds.
+        connect(m_doc, SIGNAL(pageSetupChanged()), this, SLOT(updateUI()));
     }
 
     setWindowTitle(QString("%1 - %2").arg(i18n("Office Viewer"), fname));
@@ -433,8 +478,8 @@ bool MainWindow::checkFiletype(const QString &fileName)
     QList<QString> extensions;
     //Add Txt extension after adding ascii filter to koffice package
     /*extensions << EXT_DOC << EXT_DOCX << EXT_ODT << EXT_TXT \*/
-    extensions << EXT_DOC << EXT_DOCX << EXT_ODT  \
-    << EXT_PPT << EXT_PPTX << EXT_ODP << EXT_RTF \
+    extensions << EXT_DOC << EXT_DOCX << EXT_ODT << EXT_TXT \
+    << EXT_PPT << EXT_PPTX << EXT_ODP << EXT_PPS << EXT_PPSX \
     << EXT_ODS << EXT_XLS << EXT_XLSX;
     QString ext = KMimeType::extractKnownExtension(fileName);
 
@@ -666,7 +711,7 @@ static void findTextShapesRecursive(KoShapeContainer* con, KoPAPageBase* page,
                                     QList<QPair<KoPAPageBase*, KoShape*> >& shapes,
                                     QList<QTextDocument*>& docs)
 {
-    foreach(KoShape* shape, con->childShapes()) {
+    foreach(KoShape* shape, con->shapes()) {
         KoTextShapeData* tsd = qobject_cast<KoTextShapeData*> (shape->userData());
         if (tsd) {
             shapes.append(qMakePair(page, shape));
@@ -900,10 +945,8 @@ void MainWindow::updateActions()
     m_ui->actionZoomLevel->setEnabled(docLoaded);
     m_ui->actionNextPage->setEnabled(docLoaded);
     m_ui->actionPrevPage->setEnabled(docLoaded);
-    m_ui->actionPanningOn->setEnabled(docLoaded);
 }
 
-
 bool MainWindow::eventFilter(QObject *watched, QEvent *event)
 {
     // show buttons in full screen mode if user taps anywhere in the screen
@@ -1020,7 +1063,6 @@ void MainWindow::activeToolChanged(KoCanvasController* canvas, int)
 
     canvas->setProperty("FingerScrollable", true);
     m_ui->actionCopy->setEnabled(newTool == TextTool_ID);
-    m_ui->actionPanningOn->setChecked(newTool == TextTool_ID);
 }
 
 void MainWindow::setShowProgressIndicator(bool visible)
@@ -1040,6 +1082,25 @@ void MainWindow::checkDBusActivation()
         openFileDialog();
 }
 
+void MainWindow::pluginOpen(bool newWindow, const QString& path)
+{
+    openDocument(path);
+}
+
+void MainWindow::menuClicked(QAction* action)
+{
+    if (!action->data().toBool())
+    {
+        return; // We return if it was not a plugin action
+    }
+
+    const QString activeText = action->text();
+    closeDocument();
+    OfficeInterface *nextPlugin = loadedPlugins[activeText];
+    nextPlugin->setDocument(m_doc);
+    setCentralWidget(nextPlugin->view());
+}
+
 void MainWindow::loadScrollAndQuit()
 {
     // if no document is loaded, simply quit
